package com.netpotlet.xul.component;

import javax.faces.FactoryFinder;
import javax.faces.application.Application;
import javax.faces.application.ApplicationFactory;
import javax.faces.application.FacesMessage;
import javax.faces.component.UIComponent;
import javax.faces.component.UIInput;
import javax.faces.context.FacesContext;
import javax.faces.convert.Converter;
import javax.faces.el.MethodBinding;
import javax.faces.validator.ValidatorException;

import com.netpotlet.xul.element.XulTextbox;
import com.netpotlet.xul.element.XulConfig;

public class TextboxComponent extends UIInput implements CoveComponent {
    public XulTextbox xulTextbox;

    public TextboxComponent(XulTextbox textbox) {
        super();
        xulTextbox = textbox;
        setId(textbox.getId());
        setup(textbox.getConfig());
    }

    public void addChild(UIComponent component) {
        getChildren().add(component);
    }

    public String getFamily() {
        return "Textbox";
    }

    private void setup(XulConfig xulConfig) {
        ApplicationFactory aFactory = 
            (ApplicationFactory) FactoryFinder.getFactory(FactoryFinder.APPLICATION_FACTORY);
        Application application = aFactory.getApplication();
        setupValidator(application, xulConfig);
        setupConverter(application, xulConfig);
    }

    private void setupValidator(Application application, XulConfig xulConfig) {
        String validator = xulConfig.getValidatorAsString();
        if ((validator == null) || ("".equals(validator))) return;
        Class[] args = {FacesContext.class, UIComponent.class, Object.class};
        MethodBinding methodBinding = 
            application.createMethodBinding(validator, args);
        super.setValidator(methodBinding);
    }

    private void setupConverter(Application application, XulConfig xulConfig) {
        String converterId = xulConfig.getConverterAsString();
        if ((converterId == null) || ("".equals(converterId))) return;
          Converter converter = 
              application.createConverter(converterId);
          super.setConverter(converter);
    }

    public void validate(FacesContext context) {
       MethodBinding methodBinding = getValidator(); 
       if (methodBinding == null) return;
       Object[] values = {context, (UIComponent)this, getValue()};
       try {
           methodBinding.invoke(context, values);
       } catch (ValidatorException e) {
           setValid(false);
           throw new ValidatorException(new FacesMessage(e.getMessage()));
       }
    }

    public void updateModel(FacesContext context) {
        Object value;
        Converter converter = getConverter();
        if (converter != null) {
            value = converter.getAsObject(context, (UIComponent)this, (String)getValue());
        } else {
            value = getValue();
        }
        ApplicationFactory aFactory = 
            (ApplicationFactory) FactoryFinder.getFactory(FactoryFinder.APPLICATION_FACTORY);
        Application application = aFactory.getApplication();
        String actionListener = xulTextbox.getConfig().getActionListenerAsString();
        if ((actionListener != null) && (!"".equals(actionListener))) {
            Class[] args = getArgs(value);
            Object[] values = {value};
            MethodBinding methodBinding = 
                application.createMethodBinding(actionListener, args);
            methodBinding.invoke(context, values);
        }
    }
    
    private Class[] getArgs(Object obj) {
        Class[] stringArgs = {String.class};
        Class[] integerArgs = {Integer.class};
        if (obj instanceof String) {
            return stringArgs;
        } else if (obj instanceof Integer) {
            return integerArgs;
        }
        return null;
    }

    public boolean isTransient() {
        return false;
    }

    public Object saveState(FacesContext context) {
        Object[] values = new Object[2];
        values[0] = super.saveState(context);
        values[1] = getValue();
        return values;
    }
    
    public void restoreState(FacesContext context, Object state) {
        Object[] values = (Object[])state;
        super.restoreState(context, values[0]);
        setValue(values[1]);
    }
}
