/*
 * The Relaxer artifact
 * Copyright (c) 2000-2004, ASAMI Tomoharu, All rights reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 *
 * - Redistributions of source code must retain the above copyright
 *   notice, this list of conditions and the following disclaimer. 
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
 * LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
 * OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 * WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */
package com.netpotlet.xul.element;

import java.io.IOException;
import java.io.PrintWriter;
import java.io.Writer;
import org.w3c.dom.*;

/**
 * <b>XulBlockModel</b> is generated from xul-jsf.rng by Relaxer.
 * This class is derived from:
 * 
 * <!-- for programmer
 * <define combine="choice" name="Block.model">
 *     <zeroOrMore>
 *       <ref name="Block.class"/>
 *     </zeroOrMore>
 *   </define>-->
 * <!-- for javadoc -->
 * <pre> &lt;define combine="choice" name="Block.model"&gt;
 *     &lt;zeroOrMore&gt;
 *       &lt;ref name="Block.class"/&gt;
 *     &lt;/zeroOrMore&gt;
 *   &lt;/define&gt;</pre>
 *
 * @version xul-jsf.rng (Sat Sep 25 09:38:02 JST 2004)
 * @author  Relaxer 1.1b (http://www.relaxer.org)
 */
public class XulBlockModel implements java.io.Serializable, Cloneable, IRNSContainer, IXulToolbarChoice, IRVisitable, IRNode, IXulRuleChoice, IXulTemplateChoice {
    private RNSContext rNSContext_ = new RNSContext(this, "http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul");
    // List<IXulBlockClassChoice>
    private java.util.List blockClass_ = new java.util.ArrayList();
    private java.util.Map xmlPIMap_;
    private IRNode parentRNode_;

    /**
     * Creates a <code>XulBlockModel</code>.
     *
     */
    public XulBlockModel() {
    }

    /**
     * Creates a <code>XulBlockModel</code>.
     *
     * @param source
     */
    public XulBlockModel(XulBlockModel source) {
        setup(source);
    }

    /**
     * Creates a <code>XulBlockModel</code> by the Stack <code>stack</code>
     * that contains Elements.
     * This constructor is supposed to be used internally
     * by the Relaxer system.
     *
     * @param stack
     */
    public XulBlockModel(RStack stack) {
        setup(stack);
    }

    /**
     * Initializes the <code>XulBlockModel</code> by the XulBlockModel <code>source</code>.
     *
     * @param source
     */
    public void setup(XulBlockModel source) {
        int size;
        this.blockClass_.clear();
        size = source.blockClass_.size();
        for (int i = 0;i < size;i++) {
            addBlockClass((IXulBlockClassChoice)source.getBlockClass(i).clone());
        }
    }

    /**
     * Initializes the <code>XulBlockModel</code> by the Stack <code>stack</code>
     * that contains Elements.
     * This constructor is supposed to be used internally
     * by the Relaxer system.
     *
     * @param stack
     */
    public void setup(RStack stack) {
        Element element = stack.getContextElement();
        blockClass_.clear();
        while (true) {
            if (XulArrowscrollboxClass.isMatch(stack)) {
                addBlockClass(new XulArrowscrollboxClass(stack));
            } else if (XulBoxesClass.isMatch(stack)) {
                addBlockClass(new XulBoxesClass(stack));
            } else if (XulBroadcastersetClass.isMatch(stack)) {
                addBlockClass(new XulBroadcastersetClass(stack));
            } else if (XulButtonClass.isMatch(stack)) {
                addBlockClass(new XulButtonClass(stack));
            } else if (XulCardsClass.isMatch(stack)) {
                addBlockClass(new XulCardsClass(stack));
            } else if (XulCheckboxClass.isMatch(stack)) {
                addBlockClass(new XulCheckboxClass(stack));
            } else if (XulCommandClass.isMatch(stack)) {
                addBlockClass(new XulCommandClass(stack));
            } else if (XulDescriptionClass.isMatch(stack)) {
                addBlockClass(new XulDescriptionClass(stack));
            } else if (XulGridClass.isMatch(stack)) {
                addBlockClass(new XulGridClass(stack));
            } else if (XulIframeClass.isMatch(stack)) {
                addBlockClass(new XulIframeClass(stack));
            } else if (XulImageClass.isMatch(stack)) {
                addBlockClass(new XulImageClass(stack));
            } else if (XulKeysetClass.isMatch(stack)) {
                addBlockClass(new XulKeysetClass(stack));
            } else if (XulLabelClass.isMatch(stack)) {
                addBlockClass(new XulLabelClass(stack));
            } else if (XulListboxClass.isMatch(stack)) {
                addBlockClass(new XulListboxClass(stack));
            } else if (XulMenulistClass.isMatch(stack)) {
                addBlockClass(new XulMenulistClass(stack));
            } else if (XulPopupsetClass.isMatch(stack)) {
                addBlockClass(new XulPopupsetClass(stack));
            } else if (XulRadiogroupClass.isMatch(stack)) {
                addBlockClass(new XulRadiogroupClass(stack));
            } else if (XulScriptClass.isMatch(stack)) {
                addBlockClass(new XulScriptClass(stack));
            } else if (XulSeparatorClass.isMatch(stack)) {
                addBlockClass(new XulSeparatorClass(stack));
            } else if (XulSpacerClass.isMatch(stack)) {
                addBlockClass(new XulSpacerClass(stack));
            } else if (XulSplitterClass.isMatch(stack)) {
                addBlockClass(new XulSplitterClass(stack));
            } else if (XulTabboxClass.isMatch(stack)) {
                addBlockClass(new XulTabboxClass(stack));
            } else if (XulTemplateClass.isMatch(stack)) {
                addBlockClass(new XulTemplateClass(stack));
            } else if (XulTextboxClass.isMatch(stack)) {
                addBlockClass(new XulTextboxClass(stack));
            } else if (XulToolboxClass.isMatch(stack)) {
                addBlockClass(new XulToolboxClass(stack));
            } else if (XulTreeClass.isMatch(stack)) {
                addBlockClass(new XulTreeClass(stack));
            } else {
                break;
            }
        }
    }

    /**
     * @return Object
     */
    public Object clone() {
        return (new XulBlockModel((XulBlockModel)this));
    }

    /**
     * Creates a DOM representation of the object.
     * Result is appended to the Node <code>parent</code>.
     *
     * @param parent
     */
    public void makeElement(Node parent) {
        Document doc = parent.getOwnerDocument();
        Element element = (Element)parent;
        int size;
        size = this.blockClass_.size();
        for (int i = 0;i < size;i++) {
            IXulBlockClassChoice value = (IXulBlockClassChoice)this.blockClass_.get(i);
            value.makeElement(element);
        }
    }

    /**
     * Gets the RNSContext property <b>RNSContext</b>.
     *
     * @return RNSContext
     */
    public RNSContext rGetRNSContext() {
        return (rNSContext_);
    }

    /**
     * Sets the RNSContext property <b>RNSContext</b>.
     *
     * @param rNSContext
     */
    public void rSetRNSContext(RNSContext rNSContext) {
        this.rNSContext_ = rNSContext;
    }

    /**
     * Gets the IXulBlockClassChoice property <b>BlockClass</b>.
     *
     * @return IXulBlockClassChoice[]
     */
    public IXulBlockClassChoice[] getBlockClass() {
        IXulBlockClassChoice[] array = new IXulBlockClassChoice[blockClass_.size()];
        return ((IXulBlockClassChoice[])blockClass_.toArray(array));
    }

    /**
     * Sets the IXulBlockClassChoice property <b>BlockClass</b>.
     *
     * @param blockClass
     */
    public void setBlockClass(IXulBlockClassChoice[] blockClass) {
        this.blockClass_.clear();
        for (int i = 0;i < blockClass.length;i++) {
            addBlockClass(blockClass[i]);
        }
        for (int i = 0;i < blockClass.length;i++) {
            blockClass[i].rSetParentRNode(this);
        }
    }

    /**
     * Sets the IXulBlockClassChoice property <b>BlockClass</b>.
     *
     * @param blockClass
     */
    public void setBlockClass(IXulBlockClassChoice blockClass) {
        this.blockClass_.clear();
        addBlockClass(blockClass);
        if (blockClass != null) {
            blockClass.rSetParentRNode(this);
        }
    }

    /**
     * Adds the IXulBlockClassChoice property <b>BlockClass</b>.
     *
     * @param blockClass
     */
    public void addBlockClass(IXulBlockClassChoice blockClass) {
        this.blockClass_.add(blockClass);
        if (blockClass != null) {
            blockClass.rSetParentRNode(this);
        }
    }

    /**
     * Adds the IXulBlockClassChoice property <b>BlockClass</b>.
     *
     * @param blockClass
     */
    public void addBlockClass(IXulBlockClassChoice[] blockClass) {
        for (int i = 0;i < blockClass.length;i++) {
            addBlockClass(blockClass[i]);
        }
        for (int i = 0;i < blockClass.length;i++) {
            blockClass[i].rSetParentRNode(this);
        }
    }

    /**
     * Gets number of the IXulBlockClassChoice property <b>BlockClass</b>.
     *
     * @return int
     */
    public int sizeBlockClass() {
        return (blockClass_.size());
    }

    /**
     * Gets the IXulBlockClassChoice property <b>BlockClass</b> by index.
     *
     * @param index
     * @return IXulBlockClassChoice
     */
    public IXulBlockClassChoice getBlockClass(int index) {
        return ((IXulBlockClassChoice)blockClass_.get(index));
    }

    /**
     * Sets the IXulBlockClassChoice property <b>BlockClass</b> by index.
     *
     * @param index
     * @param blockClass
     */
    public void setBlockClass(int index, IXulBlockClassChoice blockClass) {
        this.blockClass_.set(index, blockClass);
        if (blockClass != null) {
            blockClass.rSetParentRNode(this);
        }
    }

    /**
     * Adds the IXulBlockClassChoice property <b>BlockClass</b> by index.
     *
     * @param index
     * @param blockClass
     */
    public void addBlockClass(int index, IXulBlockClassChoice blockClass) {
        this.blockClass_.add(index, blockClass);
        if (blockClass != null) {
            blockClass.rSetParentRNode(this);
        }
    }

    /**
     * Remove the IXulBlockClassChoice property <b>BlockClass</b> by index.
     *
     * @param index
     */
    public void removeBlockClass(int index) {
        this.blockClass_.remove(index);
    }

    /**
     * Remove the IXulBlockClassChoice property <b>BlockClass</b> by object.
     *
     * @param blockClass
     */
    public void removeBlockClass(IXulBlockClassChoice blockClass) {
        this.blockClass_.remove(blockClass);
    }

    /**
     * Clear the IXulBlockClassChoice property <b>BlockClass</b>.
     *
     */
    public void clearBlockClass() {
        this.blockClass_.clear();
    }

    /**
     * Makes an XML text representation.
     *
     * @return String
     */
    public String makeTextDocument() {
        StringBuffer buffer = new StringBuffer();
        makeTextElement(buffer);
        return (new String(buffer));
    }

    /**
     * Makes an XML text representation.
     *
     * @param buffer
     */
    public void makeTextElement(StringBuffer buffer) {
        int size;
        String prefix = rNSContext_.getPrefixByUri("http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul");
        size = this.blockClass_.size();
        for (int i = 0;i < size;i++) {
            IXulBlockClassChoice value = (IXulBlockClassChoice)this.blockClass_.get(i);
            value.makeTextElement(buffer);
        }
    }

    /**
     * Makes an XML text representation.
     *
     * @param buffer
     * @exception IOException
     */
    public void makeTextElement(Writer buffer) throws IOException {
        int size;
        String prefix = rNSContext_.getPrefixByUri("http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul");
        size = this.blockClass_.size();
        for (int i = 0;i < size;i++) {
            IXulBlockClassChoice value = (IXulBlockClassChoice)this.blockClass_.get(i);
            value.makeTextElement(buffer);
        }
    }

    /**
     * Makes an XML text representation.
     *
     * @param buffer
     */
    public void makeTextElement(PrintWriter buffer) {
        int size;
        String prefix = rNSContext_.getPrefixByUri("http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul");
        size = this.blockClass_.size();
        for (int i = 0;i < size;i++) {
            IXulBlockClassChoice value = (IXulBlockClassChoice)this.blockClass_.get(i);
            value.makeTextElement(buffer);
        }
    }

    /**
     * Makes an XML text representation.
     *
     * @param buffer
     */
    public void makeTextAttribute(StringBuffer buffer) {
        int size;
        size = this.blockClass_.size();
        for (int i = 0;i < size;i++) {
            IXulBlockClassChoice value = (IXulBlockClassChoice)this.blockClass_.get(i);
            value.makeTextAttribute(buffer);
        }
    }

    /**
     * Makes an XML text representation.
     *
     * @param buffer
     * @exception IOException
     */
    public void makeTextAttribute(Writer buffer) throws IOException {
        int size;
        size = this.blockClass_.size();
        for (int i = 0;i < size;i++) {
            IXulBlockClassChoice value = (IXulBlockClassChoice)this.blockClass_.get(i);
            value.makeTextAttribute(buffer);
        }
    }

    /**
     * Makes an XML text representation.
     *
     * @param buffer
     */
    public void makeTextAttribute(PrintWriter buffer) {
        int size;
        size = this.blockClass_.size();
        for (int i = 0;i < size;i++) {
            IXulBlockClassChoice value = (IXulBlockClassChoice)this.blockClass_.get(i);
            value.makeTextAttribute(buffer);
        }
    }

    /**
     * Returns a String representation of this object.
     * While this method informs as XML format representaion, 
     *  it's purpose is just information, not making 
     * a rigid XML documentation.
     *
     * @return String
     */
    public String toString() {
        try {
            return (makeTextDocument());
        } catch (Exception e) {
            return (super.toString());
        }
    }

    /**
     * Accepts the Visitor for enter behavior.
     *
     * @param visitor
     * @return boolean
     */
    public boolean enter(IRVisitor visitor) {
        return (visitor.enter(this));
    }

    /**
     * Accepts the Visitor for leave behavior.
     *
     * @param visitor
     */
    public void leave(IRVisitor visitor) {
        visitor.leave(this);
    }

    /**
     * Gets PI by name.
     *
     * @param name
     * @return String
     */
    public String rGetPi(String name) {
        return ((String)xmlPIMap_.get(name));
    }

    /**
     * Gets PI map.
     *
     * @return java.util.Map
     */
    public java.util.Map rGetPiMap() {
        return (java.util.Collections.unmodifiableMap(xmlPIMap_));
    }

    /**
     * Gets the IRNode property <b>parentRNode</b>.
     *
     * @return IRNode
     */
    public IRNode rGetParentRNode() {
        return (parentRNode_);
    }

    /**
     * Sets the IRNode property <b>parentRNode</b>.
     *
     * @param parentRNode
     */
    public void rSetParentRNode(IRNode parentRNode) {
        this.parentRNode_ = parentRNode;
    }

    /**
     * Gets child RNodes.
     *
     * @return IRNode[]
     */
    public IRNode[] rGetRNodes() {
        java.util.List classNodes = new java.util.ArrayList();
        classNodes.addAll(blockClass_);
        IRNode[] nodes = new IRNode[classNodes.size()];
        return ((IRNode[])classNodes.toArray(nodes));
    }

    /**
     * Tests if elements contained in a Stack <code>stack</code>
     * is valid for the <code>XulBlockModel</code>.
     * This mehtod is supposed to be used internally
     * by the Relaxer system.
     *
     * @param stack
     * @return boolean
     */
    public static boolean isMatch(RStack stack) {
        return (isMatchHungry(stack.makeClone()));
    }

    /**
     * Tests if elements contained in a Stack <code>stack</code>
     * is valid for the <code>XulBlockModel</code>.
     * This method consumes the stack contents during matching operation.
     * This mehtod is supposed to be used internally
     * by the Relaxer system.
     *
     * @param stack
     * @return boolean
     */
    public static boolean isMatchHungry(RStack stack) {
        RStack target = stack.makeClone();
        boolean $match$ = false;
        Element element = target.peekElement();
        Element child;
        while (true) {
            if (XulArrowscrollboxClass.isMatchHungry(target)) {
                $match$ = true;
            } else if (XulBoxesClass.isMatchHungry(target)) {
                $match$ = true;
            } else if (XulBroadcastersetClass.isMatchHungry(target)) {
                $match$ = true;
            } else if (XulButtonClass.isMatchHungry(target)) {
                $match$ = true;
            } else if (XulCardsClass.isMatchHungry(target)) {
                $match$ = true;
            } else if (XulCheckboxClass.isMatchHungry(target)) {
                $match$ = true;
            } else if (XulCommandClass.isMatchHungry(target)) {
                $match$ = true;
            } else if (XulDescriptionClass.isMatchHungry(target)) {
                $match$ = true;
            } else if (XulGridClass.isMatchHungry(target)) {
                $match$ = true;
            } else if (XulIframeClass.isMatchHungry(target)) {
                $match$ = true;
            } else if (XulImageClass.isMatchHungry(target)) {
                $match$ = true;
            } else if (XulKeysetClass.isMatchHungry(target)) {
                $match$ = true;
            } else if (XulLabelClass.isMatchHungry(target)) {
                $match$ = true;
            } else if (XulListboxClass.isMatchHungry(target)) {
                $match$ = true;
            } else if (XulMenulistClass.isMatchHungry(target)) {
                $match$ = true;
            } else if (XulPopupsetClass.isMatchHungry(target)) {
                $match$ = true;
            } else if (XulRadiogroupClass.isMatchHungry(target)) {
                $match$ = true;
            } else if (XulScriptClass.isMatchHungry(target)) {
                $match$ = true;
            } else if (XulSeparatorClass.isMatchHungry(target)) {
                $match$ = true;
            } else if (XulSpacerClass.isMatchHungry(target)) {
                $match$ = true;
            } else if (XulSplitterClass.isMatchHungry(target)) {
                $match$ = true;
            } else if (XulTabboxClass.isMatchHungry(target)) {
                $match$ = true;
            } else if (XulTemplateClass.isMatchHungry(target)) {
                $match$ = true;
            } else if (XulTextboxClass.isMatchHungry(target)) {
                $match$ = true;
            } else if (XulToolboxClass.isMatchHungry(target)) {
                $match$ = true;
            } else if (XulTreeClass.isMatchHungry(target)) {
                $match$ = true;
            } else {
                break;
            }
        }
        if ($match$) {
            stack.eat(target);
        }
        return ($match$);
    }
}
