/*
 * The Relaxer artifact
 * Copyright (c) 2000-2004, ASAMI Tomoharu, All rights reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 *
 * - Redistributions of source code must retain the above copyright
 *   notice, this list of conditions and the following disclaimer. 
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
 * LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
 * OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 * WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */
package com.netpotlet.xul.element;

import java.io.*;
import java.io.IOException;
import java.io.PrintWriter;
import java.io.Writer;
import java.net.URL;
import javax.xml.parsers.*;
import org.w3c.dom.*;
import org.xml.sax.*;

/**
 * <b>XulListitem</b> is generated from xul-jsf.rng by Relaxer.
 * This class is derived from:
 * 
 * <!-- for programmer
 * <element name="listitem">
 *       <ref name="core.attlist"/>
 *       <ref name="checked.attrib"/>
 *       <ref name="command.attrib"/>
 *       <ref name="current.attrib"/>
 *       <ref name="disabled.attrib"/>
 *       <ref name="selected.attrib"/>
 *       <ref name="type.attrib"/>
 *       <ref name="value.attrib"/>
 *       <zeroOrMore>
 *         <ref name="listcell"/>
 *       </zeroOrMore>
 *     </element>-->
 * <!-- for javadoc -->
 * <pre> &lt;element name="listitem"&gt;
 *       &lt;ref name="core.attlist"/&gt;
 *       &lt;ref name="checked.attrib"/&gt;
 *       &lt;ref name="command.attrib"/&gt;
 *       &lt;ref name="current.attrib"/&gt;
 *       &lt;ref name="disabled.attrib"/&gt;
 *       &lt;ref name="selected.attrib"/&gt;
 *       &lt;ref name="type.attrib"/&gt;
 *       &lt;ref name="value.attrib"/&gt;
 *       &lt;zeroOrMore&gt;
 *         &lt;ref name="listcell"/&gt;
 *       &lt;/zeroOrMore&gt;
 *     &lt;/element&gt;</pre>
 *
 * @version xul-jsf.rng (Sat Sep 25 09:38:04 JST 2004)
 * @author  Relaxer 1.1b (http://www.relaxer.org)
 */
public class XulListitem implements java.io.Serializable, Cloneable, IRNSContainer, IRVisitable, IRNode {
    public static final String CROP_START = "start";
    public static final String CROP_END = "end";
    public static final String CROP_CENTER = "center";
    public static final String CROP_NONE = "none";

    private RNSContext rNSContext_ = new RNSContext(this, "http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul");
    private String accesskey_;
    private String crop_;
    private String image_;
    private String label_;
    private Boolean checked_;
    private String command_;
    private Boolean current_;
    private Boolean disabled_;
    private Boolean selected_;
    private String type_;
    private String value_;
    // List<XulListcell>
    private java.util.List listcell_ = new java.util.ArrayList();
    private java.util.Map xmlPIMap_;
    private IRNode parentRNode_;

    /**
     * Creates a <code>XulListitem</code>.
     *
     */
    public XulListitem() {
    }

    /**
     * Creates a <code>XulListitem</code>.
     *
     * @param source
     */
    public XulListitem(XulListitem source) {
        setup(source);
    }

    /**
     * Creates a <code>XulListitem</code> by the Stack <code>stack</code>
     * that contains Elements.
     * This constructor is supposed to be used internally
     * by the Relaxer system.
     *
     * @param stack
     */
    public XulListitem(RStack stack) {
        setup(stack);
    }

    /**
     * Creates a <code>XulListitem</code> by the Document <code>doc</code>.
     *
     * @param doc
     */
    public XulListitem(Document doc) {
        setup(doc.getDocumentElement());
    }

    /**
     * Creates a <code>XulListitem</code> by the Element <code>element</code>.
     *
     * @param element
     */
    public XulListitem(Element element) {
        setup(element);
    }

    /**
     * Creates a <code>XulListitem</code> by the File <code>file</code>.
     *
     * @param file
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public XulListitem(File file) throws IOException, SAXException, ParserConfigurationException {
        setup(file);
    }

    /**
     * Creates a <code>XulListitem</code>
     * by the String representation of URI <code>uri</code>.
     *
     * @param uri
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public XulListitem(String uri) throws IOException, SAXException, ParserConfigurationException {
        setup(uri);
    }

    /**
     * Creates a <code>XulListitem</code> by the URL <code>url</code>.
     *
     * @param url
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public XulListitem(URL url) throws IOException, SAXException, ParserConfigurationException {
        setup(url);
    }

    /**
     * Creates a <code>XulListitem</code> by the InputStream <code>in</code>.
     *
     * @param in
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public XulListitem(InputStream in) throws IOException, SAXException, ParserConfigurationException {
        setup(in);
    }

    /**
     * Creates a <code>XulListitem</code> by the InputSource <code>is</code>.
     *
     * @param is
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public XulListitem(InputSource is) throws IOException, SAXException, ParserConfigurationException {
        setup(is);
    }

    /**
     * Creates a <code>XulListitem</code> by the Reader <code>reader</code>.
     *
     * @param reader
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public XulListitem(Reader reader) throws IOException, SAXException, ParserConfigurationException {
        setup(reader);
    }

    /**
     * Initializes the <code>XulListitem</code> by the XulListitem <code>source</code>.
     *
     * @param source
     */
    public void setup(XulListitem source) {
        int size;
        accesskey_ = source.accesskey_;
        crop_ = source.crop_;
        image_ = source.image_;
        label_ = source.label_;
        checked_ = source.checked_;
        command_ = source.command_;
        current_ = source.current_;
        disabled_ = source.disabled_;
        selected_ = source.selected_;
        type_ = source.type_;
        value_ = source.value_;
        this.listcell_.clear();
        size = source.listcell_.size();
        for (int i = 0;i < size;i++) {
            addListcell((XulListcell)source.getListcell(i).clone());
        }
    }

    /**
     * Initializes the <code>XulListitem</code> by the Document <code>doc</code>.
     *
     * @param doc
     */
    public void setup(Document doc) {
        setup(doc.getDocumentElement());
    }

    /**
     * Initializes the <code>XulListitem</code> by the Element <code>element</code>.
     *
     * @param element
     */
    public void setup(Element element) {
        init(element);
    }

    /**
     * Initializes the <code>XulListitem</code> by the Stack <code>stack</code>
     * that contains Elements.
     * This constructor is supposed to be used internally
     * by the Relaxer system.
     *
     * @param stack
     */
    public void setup(RStack stack) {
        init(stack.popElement());
    }

    /**
     * @param element
     */
    private void init(Element element) {
        RStack stack = new RStack(element);
        rNSContext_.declareNamespace(element);
        accesskey_ = URelaxer.getAttributePropertyAsString(element, "accesskey");
        crop_ = URelaxer.getAttributePropertyAsString(element, "crop");
        image_ = URelaxer.getAttributePropertyAsString(element, "image");
        label_ = URelaxer.getAttributePropertyAsString(element, "label");
        checked_ = URelaxer.getAttributePropertyAsBooleanObject(element, "checked");
        command_ = URelaxer.getAttributePropertyAsString(element, "command");
        current_ = URelaxer.getAttributePropertyAsBooleanObject(element, "current");
        disabled_ = URelaxer.getAttributePropertyAsBooleanObject(element, "disabled");
        selected_ = URelaxer.getAttributePropertyAsBooleanObject(element, "selected");
        type_ = URelaxer.getAttributePropertyAsString(element, "type");
        value_ = URelaxer.getAttributePropertyAsString(element, "value");
        listcell_.clear();
        while (true) {
            if (XulListcell.isMatch(stack)) {
                addListcell(new XulListcell(stack));
            } else {
                break;
            }
        }
        xmlPIMap_ = stack.getPIMap();
    }

    /**
     * @return Object
     */
    public Object clone() {
        return (new XulListitem((XulListitem)this));
    }

    /**
     * Creates a DOM representation of the object.
     * Result is appended to the Node <code>parent</code>.
     *
     * @param parent
     */
    public void makeElement(Node parent) {
        Document doc;
        if (parent instanceof Document) {
            doc = (Document)parent;
        } else {
            doc = parent.getOwnerDocument();
        }
        Element element = doc.createElementNS("http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul", "listitem");
        rNSContext_.setupNamespace(element);
        int size;
        if (this.accesskey_ != null) {
            URelaxer.setAttributePropertyByString(element, "accesskey", this.accesskey_);
        }
        if (this.crop_ != null) {
            URelaxer.setAttributePropertyByString(element, "crop", this.crop_);
        }
        if (this.image_ != null) {
            URelaxer.setAttributePropertyByString(element, "image", this.image_);
        }
        if (this.label_ != null) {
            URelaxer.setAttributePropertyByString(element, "label", this.label_);
        }
        if (this.checked_ != null) {
            URelaxer.setAttributePropertyByBoolean(element, "checked", this.checked_);
        }
        if (this.command_ != null) {
            URelaxer.setAttributePropertyByString(element, "command", this.command_);
        }
        if (this.current_ != null) {
            URelaxer.setAttributePropertyByBoolean(element, "current", this.current_);
        }
        if (this.disabled_ != null) {
            URelaxer.setAttributePropertyByBoolean(element, "disabled", this.disabled_);
        }
        if (this.selected_ != null) {
            URelaxer.setAttributePropertyByBoolean(element, "selected", this.selected_);
        }
        if (this.type_ != null) {
            URelaxer.setAttributePropertyByString(element, "type", this.type_);
        }
        if (this.value_ != null) {
            URelaxer.setAttributePropertyByString(element, "value", this.value_);
        }
        size = this.listcell_.size();
        for (int i = 0;i < size;i++) {
            XulListcell value = (XulListcell)this.listcell_.get(i);
            value.makeElement(element);
        }
        parent.appendChild(element);
    }

    /**
     * Initializes the <code>XulListitem</code> by the File <code>file</code>.
     *
     * @param file
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public void setup(File file) throws IOException, SAXException, ParserConfigurationException {
        setup(file.toURL());
    }

    /**
     * Initializes the <code>XulListitem</code>
     * by the String representation of URI <code>uri</code>.
     *
     * @param uri
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public void setup(String uri) throws IOException, SAXException, ParserConfigurationException {
        setup(UJAXP.getDocument(uri, UJAXP.FLAG_NAMESPACE_AWARE));
    }

    /**
     * Initializes the <code>XulListitem</code> by the URL <code>url</code>.
     *
     * @param url
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public void setup(URL url) throws IOException, SAXException, ParserConfigurationException {
        setup(UJAXP.getDocument(url, UJAXP.FLAG_NAMESPACE_AWARE));
    }

    /**
     * Initializes the <code>XulListitem</code> by the InputStream <code>in</code>.
     *
     * @param in
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public void setup(InputStream in) throws IOException, SAXException, ParserConfigurationException {
        setup(UJAXP.getDocument(in, UJAXP.FLAG_NAMESPACE_AWARE));
    }

    /**
     * Initializes the <code>XulListitem</code> by the InputSource <code>is</code>.
     *
     * @param is
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public void setup(InputSource is) throws IOException, SAXException, ParserConfigurationException {
        setup(UJAXP.getDocument(is, UJAXP.FLAG_NAMESPACE_AWARE));
    }

    /**
     * Initializes the <code>XulListitem</code> by the Reader <code>reader</code>.
     *
     * @param reader
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public void setup(Reader reader) throws IOException, SAXException, ParserConfigurationException {
        setup(UJAXP.getDocument(reader, UJAXP.FLAG_NAMESPACE_AWARE));
    }

    /**
     * Creates a DOM document representation of the object.
     *
     * @exception ParserConfigurationException
     * @return Document
     */
    public Document makeDocument() throws ParserConfigurationException {
        Document doc = UJAXP.makeDocument();
        makeElement(doc);
        return (doc);
    }

    /**
     * Gets the RNSContext property <b>RNSContext</b>.
     *
     * @return RNSContext
     */
    public RNSContext rGetRNSContext() {
        return (rNSContext_);
    }

    /**
     * Sets the RNSContext property <b>RNSContext</b>.
     *
     * @param rNSContext
     */
    public void rSetRNSContext(RNSContext rNSContext) {
        this.rNSContext_ = rNSContext;
    }

    /**
     * Gets the String property <b>accesskey</b>.
     *
     * @return String
     */
    public String getAccesskey() {
        return (accesskey_);
    }

    /**
     * Sets the String property <b>accesskey</b>.
     *
     * @param accesskey
     */
    public void setAccesskey(String accesskey) {
        this.accesskey_ = accesskey;
    }

    /**
     * Gets the String property <b>crop</b>.
     *
     * @return String
     */
    public String getCrop() {
        return (crop_);
    }

    /**
     * Sets the String property <b>crop</b>.
     *
     * @param crop
     */
    public void setCrop(String crop) {
        this.crop_ = crop;
    }

    /**
     * Gets the String property <b>image</b>.
     *
     * @return String
     */
    public String getImage() {
        return (image_);
    }

    /**
     * Sets the String property <b>image</b>.
     *
     * @param image
     */
    public void setImage(String image) {
        this.image_ = image;
    }

    /**
     * Gets the String property <b>label</b>.
     *
     * @return String
     */
    public String getLabel() {
        return (label_);
    }

    /**
     * Sets the String property <b>label</b>.
     *
     * @param label
     */
    public void setLabel(String label) {
        this.label_ = label;
    }

    /**
     * Gets the boolean property <b>checked</b>.
     *
     * @return boolean
     */
    public boolean getChecked() {
        if (checked_ == null) {
            return(false);
        }
        return (checked_.booleanValue());
    }

    /**
     * Gets the boolean property <b>checked</b>.
     *
     * @param checked
     * @return boolean
     */
    public boolean getChecked(boolean checked) {
        if (checked_ == null) {
            return(checked);
        }
        return (this.checked_.booleanValue());
    }

    /**
     * Gets the boolean property <b>checked</b>.
     *
     * @return Boolean
     */
    public Boolean getCheckedAsBoolean() {
        return (checked_);
    }

    /**
     * Check the boolean property <b>checked</b>.
     *
     * @return boolean
     */
    public boolean checkChecked() {
        return (checked_ != null);
    }

    /**
     * Sets the boolean property <b>checked</b>.
     *
     * @param checked
     */
    public void setChecked(boolean checked) {
        this.checked_ = new Boolean(checked);
    }

    /**
     * Sets the boolean property <b>checked</b>.
     *
     * @param checked
     */
    public void setChecked(Boolean checked) {
        this.checked_ = checked;
    }

    /**
     * Gets the String property <b>command</b>.
     *
     * @return String
     */
    public String getCommand() {
        return (command_);
    }

    /**
     * Sets the String property <b>command</b>.
     *
     * @param command
     */
    public void setCommand(String command) {
        this.command_ = command;
    }

    /**
     * Gets the boolean property <b>current</b>.
     *
     * @return boolean
     */
    public boolean getCurrent() {
        if (current_ == null) {
            return(false);
        }
        return (current_.booleanValue());
    }

    /**
     * Gets the boolean property <b>current</b>.
     *
     * @param current
     * @return boolean
     */
    public boolean getCurrent(boolean current) {
        if (current_ == null) {
            return(current);
        }
        return (this.current_.booleanValue());
    }

    /**
     * Gets the boolean property <b>current</b>.
     *
     * @return Boolean
     */
    public Boolean getCurrentAsBoolean() {
        return (current_);
    }

    /**
     * Check the boolean property <b>current</b>.
     *
     * @return boolean
     */
    public boolean checkCurrent() {
        return (current_ != null);
    }

    /**
     * Sets the boolean property <b>current</b>.
     *
     * @param current
     */
    public void setCurrent(boolean current) {
        this.current_ = new Boolean(current);
    }

    /**
     * Sets the boolean property <b>current</b>.
     *
     * @param current
     */
    public void setCurrent(Boolean current) {
        this.current_ = current;
    }

    /**
     * Gets the boolean property <b>disabled</b>.
     *
     * @return boolean
     */
    public boolean getDisabled() {
        if (disabled_ == null) {
            return(false);
        }
        return (disabled_.booleanValue());
    }

    /**
     * Gets the boolean property <b>disabled</b>.
     *
     * @param disabled
     * @return boolean
     */
    public boolean getDisabled(boolean disabled) {
        if (disabled_ == null) {
            return(disabled);
        }
        return (this.disabled_.booleanValue());
    }

    /**
     * Gets the boolean property <b>disabled</b>.
     *
     * @return Boolean
     */
    public Boolean getDisabledAsBoolean() {
        return (disabled_);
    }

    /**
     * Check the boolean property <b>disabled</b>.
     *
     * @return boolean
     */
    public boolean checkDisabled() {
        return (disabled_ != null);
    }

    /**
     * Sets the boolean property <b>disabled</b>.
     *
     * @param disabled
     */
    public void setDisabled(boolean disabled) {
        this.disabled_ = new Boolean(disabled);
    }

    /**
     * Sets the boolean property <b>disabled</b>.
     *
     * @param disabled
     */
    public void setDisabled(Boolean disabled) {
        this.disabled_ = disabled;
    }

    /**
     * Gets the boolean property <b>selected</b>.
     *
     * @return boolean
     */
    public boolean getSelected() {
        if (selected_ == null) {
            return(false);
        }
        return (selected_.booleanValue());
    }

    /**
     * Gets the boolean property <b>selected</b>.
     *
     * @param selected
     * @return boolean
     */
    public boolean getSelected(boolean selected) {
        if (selected_ == null) {
            return(selected);
        }
        return (this.selected_.booleanValue());
    }

    /**
     * Gets the boolean property <b>selected</b>.
     *
     * @return Boolean
     */
    public Boolean getSelectedAsBoolean() {
        return (selected_);
    }

    /**
     * Check the boolean property <b>selected</b>.
     *
     * @return boolean
     */
    public boolean checkSelected() {
        return (selected_ != null);
    }

    /**
     * Sets the boolean property <b>selected</b>.
     *
     * @param selected
     */
    public void setSelected(boolean selected) {
        this.selected_ = new Boolean(selected);
    }

    /**
     * Sets the boolean property <b>selected</b>.
     *
     * @param selected
     */
    public void setSelected(Boolean selected) {
        this.selected_ = selected;
    }

    /**
     * Gets the String property <b>type</b>.
     *
     * @return String
     */
    public String getType() {
        return (type_);
    }

    /**
     * Sets the String property <b>type</b>.
     *
     * @param type
     */
    public void setType(String type) {
        this.type_ = type;
    }

    /**
     * Gets the String property <b>value</b>.
     *
     * @return String
     */
    public String getValue() {
        return (value_);
    }

    /**
     * Sets the String property <b>value</b>.
     *
     * @param value
     */
    public void setValue(String value) {
        this.value_ = value;
    }

    /**
     * Gets the XulListcell property <b>listcell</b>.
     *
     * @return XulListcell[]
     */
    public XulListcell[] getListcell() {
        XulListcell[] array = new XulListcell[listcell_.size()];
        return ((XulListcell[])listcell_.toArray(array));
    }

    /**
     * Sets the XulListcell property <b>listcell</b>.
     *
     * @param listcell
     */
    public void setListcell(XulListcell[] listcell) {
        this.listcell_.clear();
        for (int i = 0;i < listcell.length;i++) {
            addListcell(listcell[i]);
        }
        for (int i = 0;i < listcell.length;i++) {
            listcell[i].rSetParentRNode(this);
        }
    }

    /**
     * Sets the XulListcell property <b>listcell</b>.
     *
     * @param listcell
     */
    public void setListcell(XulListcell listcell) {
        this.listcell_.clear();
        addListcell(listcell);
        if (listcell != null) {
            listcell.rSetParentRNode(this);
        }
    }

    /**
     * Adds the XulListcell property <b>listcell</b>.
     *
     * @param listcell
     */
    public void addListcell(XulListcell listcell) {
        this.listcell_.add(listcell);
        if (listcell != null) {
            listcell.rSetParentRNode(this);
        }
    }

    /**
     * Adds the XulListcell property <b>listcell</b>.
     *
     * @param listcell
     */
    public void addListcell(XulListcell[] listcell) {
        for (int i = 0;i < listcell.length;i++) {
            addListcell(listcell[i]);
        }
        for (int i = 0;i < listcell.length;i++) {
            listcell[i].rSetParentRNode(this);
        }
    }

    /**
     * Gets number of the XulListcell property <b>listcell</b>.
     *
     * @return int
     */
    public int sizeListcell() {
        return (listcell_.size());
    }

    /**
     * Gets the XulListcell property <b>listcell</b> by index.
     *
     * @param index
     * @return XulListcell
     */
    public XulListcell getListcell(int index) {
        return ((XulListcell)listcell_.get(index));
    }

    /**
     * Sets the XulListcell property <b>listcell</b> by index.
     *
     * @param index
     * @param listcell
     */
    public void setListcell(int index, XulListcell listcell) {
        this.listcell_.set(index, listcell);
        if (listcell != null) {
            listcell.rSetParentRNode(this);
        }
    }

    /**
     * Adds the XulListcell property <b>listcell</b> by index.
     *
     * @param index
     * @param listcell
     */
    public void addListcell(int index, XulListcell listcell) {
        this.listcell_.add(index, listcell);
        if (listcell != null) {
            listcell.rSetParentRNode(this);
        }
    }

    /**
     * Remove the XulListcell property <b>listcell</b> by index.
     *
     * @param index
     */
    public void removeListcell(int index) {
        this.listcell_.remove(index);
    }

    /**
     * Remove the XulListcell property <b>listcell</b> by object.
     *
     * @param listcell
     */
    public void removeListcell(XulListcell listcell) {
        this.listcell_.remove(listcell);
    }

    /**
     * Clear the XulListcell property <b>listcell</b>.
     *
     */
    public void clearListcell() {
        this.listcell_.clear();
    }

    /**
     * Makes an XML text representation.
     *
     * @return String
     */
    public String makeTextDocument() {
        StringBuffer buffer = new StringBuffer();
        makeTextElement(buffer);
        return (new String(buffer));
    }

    /**
     * Makes an XML text representation.
     *
     * @param buffer
     */
    public void makeTextElement(StringBuffer buffer) {
        int size;
        String prefix = rNSContext_.getPrefixByUri("http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul");
        buffer.append("<");
        URelaxer.makeQName(prefix, "listitem", buffer);
        rNSContext_.makeNSMappings(buffer);
        if (accesskey_ != null) {
            buffer.append(" ");
            rNSContext_.makeQName("", "accesskey", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.escapeAttrQuot(URelaxer.getString(getAccesskey())));
            buffer.append("\"");
        }
        if (crop_ != null) {
            buffer.append(" ");
            rNSContext_.makeQName("", "crop", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.escapeAttrQuot(URelaxer.getString(getCrop())));
            buffer.append("\"");
        }
        if (image_ != null) {
            buffer.append(" ");
            rNSContext_.makeQName("", "image", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.escapeAttrQuot(URelaxer.getString(getImage())));
            buffer.append("\"");
        }
        if (label_ != null) {
            buffer.append(" ");
            rNSContext_.makeQName("", "label", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.escapeAttrQuot(URelaxer.getString(getLabel())));
            buffer.append("\"");
        }
        if (checked_ != null) {
            buffer.append(" ");
            rNSContext_.makeQName("", "checked", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.getString(getChecked()));
            buffer.append("\"");
        }
        if (command_ != null) {
            buffer.append(" ");
            rNSContext_.makeQName("", "command", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.escapeAttrQuot(URelaxer.getString(getCommand())));
            buffer.append("\"");
        }
        if (current_ != null) {
            buffer.append(" ");
            rNSContext_.makeQName("", "current", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.getString(getCurrent()));
            buffer.append("\"");
        }
        if (disabled_ != null) {
            buffer.append(" ");
            rNSContext_.makeQName("", "disabled", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.getString(getDisabled()));
            buffer.append("\"");
        }
        if (selected_ != null) {
            buffer.append(" ");
            rNSContext_.makeQName("", "selected", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.getString(getSelected()));
            buffer.append("\"");
        }
        if (type_ != null) {
            buffer.append(" ");
            rNSContext_.makeQName("", "type", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.escapeAttrQuot(URelaxer.getString(getType())));
            buffer.append("\"");
        }
        if (value_ != null) {
            buffer.append(" ");
            rNSContext_.makeQName("", "value", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.escapeAttrQuot(URelaxer.getString(getValue())));
            buffer.append("\"");
        }
        buffer.append(">");
        size = this.listcell_.size();
        for (int i = 0;i < size;i++) {
            XulListcell value = (XulListcell)this.listcell_.get(i);
            value.makeTextElement(buffer);
        }
        buffer.append("</");
        URelaxer.makeQName(prefix, "listitem", buffer);
        buffer.append(">");
    }

    /**
     * Makes an XML text representation.
     *
     * @param buffer
     * @exception IOException
     */
    public void makeTextElement(Writer buffer) throws IOException {
        int size;
        String prefix = rNSContext_.getPrefixByUri("http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul");
        buffer.write("<");
        URelaxer.makeQName(prefix, "listitem", buffer);
        rNSContext_.makeNSMappings(buffer);
        if (accesskey_ != null) {
            buffer.write(" ");
            rNSContext_.makeQName("", "accesskey", buffer);
            buffer.write("=\"");
            buffer.write(URelaxer.escapeAttrQuot(URelaxer.getString(getAccesskey())));
            buffer.write("\"");
        }
        if (crop_ != null) {
            buffer.write(" ");
            rNSContext_.makeQName("", "crop", buffer);
            buffer.write("=\"");
            buffer.write(URelaxer.escapeAttrQuot(URelaxer.getString(getCrop())));
            buffer.write("\"");
        }
        if (image_ != null) {
            buffer.write(" ");
            rNSContext_.makeQName("", "image", buffer);
            buffer.write("=\"");
            buffer.write(URelaxer.escapeAttrQuot(URelaxer.getString(getImage())));
            buffer.write("\"");
        }
        if (label_ != null) {
            buffer.write(" ");
            rNSContext_.makeQName("", "label", buffer);
            buffer.write("=\"");
            buffer.write(URelaxer.escapeAttrQuot(URelaxer.getString(getLabel())));
            buffer.write("\"");
        }
        if (checked_ != null) {
            buffer.write(" ");
            rNSContext_.makeQName("", "checked", buffer);
            buffer.write("=\"");
            buffer.write(URelaxer.getString(getChecked()));
            buffer.write("\"");
        }
        if (command_ != null) {
            buffer.write(" ");
            rNSContext_.makeQName("", "command", buffer);
            buffer.write("=\"");
            buffer.write(URelaxer.escapeAttrQuot(URelaxer.getString(getCommand())));
            buffer.write("\"");
        }
        if (current_ != null) {
            buffer.write(" ");
            rNSContext_.makeQName("", "current", buffer);
            buffer.write("=\"");
            buffer.write(URelaxer.getString(getCurrent()));
            buffer.write("\"");
        }
        if (disabled_ != null) {
            buffer.write(" ");
            rNSContext_.makeQName("", "disabled", buffer);
            buffer.write("=\"");
            buffer.write(URelaxer.getString(getDisabled()));
            buffer.write("\"");
        }
        if (selected_ != null) {
            buffer.write(" ");
            rNSContext_.makeQName("", "selected", buffer);
            buffer.write("=\"");
            buffer.write(URelaxer.getString(getSelected()));
            buffer.write("\"");
        }
        if (type_ != null) {
            buffer.write(" ");
            rNSContext_.makeQName("", "type", buffer);
            buffer.write("=\"");
            buffer.write(URelaxer.escapeAttrQuot(URelaxer.getString(getType())));
            buffer.write("\"");
        }
        if (value_ != null) {
            buffer.write(" ");
            rNSContext_.makeQName("", "value", buffer);
            buffer.write("=\"");
            buffer.write(URelaxer.escapeAttrQuot(URelaxer.getString(getValue())));
            buffer.write("\"");
        }
        buffer.write(">");
        size = this.listcell_.size();
        for (int i = 0;i < size;i++) {
            XulListcell value = (XulListcell)this.listcell_.get(i);
            value.makeTextElement(buffer);
        }
        buffer.write("</");
        URelaxer.makeQName(prefix, "listitem", buffer);
        buffer.write(">");
    }

    /**
     * Makes an XML text representation.
     *
     * @param buffer
     */
    public void makeTextElement(PrintWriter buffer) {
        int size;
        String prefix = rNSContext_.getPrefixByUri("http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul");
        buffer.print("<");
        URelaxer.makeQName(prefix, "listitem", buffer);
        rNSContext_.makeNSMappings(buffer);
        if (accesskey_ != null) {
            buffer.print(" ");
            rNSContext_.makeQName("", "accesskey", buffer);
            buffer.print("=\"");
            buffer.print(URelaxer.escapeAttrQuot(URelaxer.getString(getAccesskey())));
            buffer.print("\"");
        }
        if (crop_ != null) {
            buffer.print(" ");
            rNSContext_.makeQName("", "crop", buffer);
            buffer.print("=\"");
            buffer.print(URelaxer.escapeAttrQuot(URelaxer.getString(getCrop())));
            buffer.print("\"");
        }
        if (image_ != null) {
            buffer.print(" ");
            rNSContext_.makeQName("", "image", buffer);
            buffer.print("=\"");
            buffer.print(URelaxer.escapeAttrQuot(URelaxer.getString(getImage())));
            buffer.print("\"");
        }
        if (label_ != null) {
            buffer.print(" ");
            rNSContext_.makeQName("", "label", buffer);
            buffer.print("=\"");
            buffer.print(URelaxer.escapeAttrQuot(URelaxer.getString(getLabel())));
            buffer.print("\"");
        }
        if (checked_ != null) {
            buffer.print(" ");
            rNSContext_.makeQName("", "checked", buffer);
            buffer.print("=\"");
            buffer.print(URelaxer.getString(getChecked()));
            buffer.print("\"");
        }
        if (command_ != null) {
            buffer.print(" ");
            rNSContext_.makeQName("", "command", buffer);
            buffer.print("=\"");
            buffer.print(URelaxer.escapeAttrQuot(URelaxer.getString(getCommand())));
            buffer.print("\"");
        }
        if (current_ != null) {
            buffer.print(" ");
            rNSContext_.makeQName("", "current", buffer);
            buffer.print("=\"");
            buffer.print(URelaxer.getString(getCurrent()));
            buffer.print("\"");
        }
        if (disabled_ != null) {
            buffer.print(" ");
            rNSContext_.makeQName("", "disabled", buffer);
            buffer.print("=\"");
            buffer.print(URelaxer.getString(getDisabled()));
            buffer.print("\"");
        }
        if (selected_ != null) {
            buffer.print(" ");
            rNSContext_.makeQName("", "selected", buffer);
            buffer.print("=\"");
            buffer.print(URelaxer.getString(getSelected()));
            buffer.print("\"");
        }
        if (type_ != null) {
            buffer.print(" ");
            rNSContext_.makeQName("", "type", buffer);
            buffer.print("=\"");
            buffer.print(URelaxer.escapeAttrQuot(URelaxer.getString(getType())));
            buffer.print("\"");
        }
        if (value_ != null) {
            buffer.print(" ");
            rNSContext_.makeQName("", "value", buffer);
            buffer.print("=\"");
            buffer.print(URelaxer.escapeAttrQuot(URelaxer.getString(getValue())));
            buffer.print("\"");
        }
        buffer.print(">");
        size = this.listcell_.size();
        for (int i = 0;i < size;i++) {
            XulListcell value = (XulListcell)this.listcell_.get(i);
            value.makeTextElement(buffer);
        }
        buffer.print("</");
        URelaxer.makeQName(prefix, "listitem", buffer);
        buffer.print(">");
    }

    /**
     * Makes an XML text representation.
     *
     * @param buffer
     */
    public void makeTextAttribute(StringBuffer buffer) {
    }

    /**
     * Makes an XML text representation.
     *
     * @param buffer
     * @exception IOException
     */
    public void makeTextAttribute(Writer buffer) throws IOException {
    }

    /**
     * Makes an XML text representation.
     *
     * @param buffer
     */
    public void makeTextAttribute(PrintWriter buffer) {
    }

    /**
     * Gets the property value as String.
     *
     * @return String
     */
    public String getAccesskeyAsString() {
        return (URelaxer.getString(getAccesskey()));
    }

    /**
     * Gets the property value as String.
     *
     * @return String
     */
    public String getCropAsString() {
        return (URelaxer.getString(getCrop()));
    }

    /**
     * Gets the property value as String.
     *
     * @return String
     */
    public String getImageAsString() {
        return (URelaxer.getString(getImage()));
    }

    /**
     * Gets the property value as String.
     *
     * @return String
     */
    public String getLabelAsString() {
        return (URelaxer.getString(getLabel()));
    }

    /**
     * Gets the property value as String.
     *
     * @return String
     */
    public String getCheckedAsString() {
        return (URelaxer.getString(getChecked()));
    }

    /**
     * Gets the property value as String.
     *
     * @return String
     */
    public String getCommandAsString() {
        return (URelaxer.getString(getCommand()));
    }

    /**
     * Gets the property value as String.
     *
     * @return String
     */
    public String getCurrentAsString() {
        return (URelaxer.getString(getCurrent()));
    }

    /**
     * Gets the property value as String.
     *
     * @return String
     */
    public String getDisabledAsString() {
        return (URelaxer.getString(getDisabled()));
    }

    /**
     * Gets the property value as String.
     *
     * @return String
     */
    public String getSelectedAsString() {
        return (URelaxer.getString(getSelected()));
    }

    /**
     * Gets the property value as String.
     *
     * @return String
     */
    public String getTypeAsString() {
        return (URelaxer.getString(getType()));
    }

    /**
     * Gets the property value as String.
     *
     * @return String
     */
    public String getValueAsString() {
        return (URelaxer.getString(getValue()));
    }

    /**
     * Sets the property value by String.
     *
     * @param string
     */
    public void setAccesskeyByString(String string) {
        setAccesskey(string);
    }

    /**
     * Sets the property value by String.
     *
     * @param string
     */
    public void setCropByString(String string) {
        setCrop(string);
    }

    /**
     * Sets the property value by String.
     *
     * @param string
     */
    public void setImageByString(String string) {
        setImage(string);
    }

    /**
     * Sets the property value by String.
     *
     * @param string
     */
    public void setLabelByString(String string) {
        setLabel(string);
    }

    /**
     * Sets the property value by String.
     *
     * @param string
     */
    public void setCheckedByString(String string) {
        setChecked(new Boolean(string).booleanValue());
    }

    /**
     * Sets the property value by String.
     *
     * @param string
     */
    public void setCommandByString(String string) {
        setCommand(string);
    }

    /**
     * Sets the property value by String.
     *
     * @param string
     */
    public void setCurrentByString(String string) {
        setCurrent(new Boolean(string).booleanValue());
    }

    /**
     * Sets the property value by String.
     *
     * @param string
     */
    public void setDisabledByString(String string) {
        setDisabled(new Boolean(string).booleanValue());
    }

    /**
     * Sets the property value by String.
     *
     * @param string
     */
    public void setSelectedByString(String string) {
        setSelected(new Boolean(string).booleanValue());
    }

    /**
     * Sets the property value by String.
     *
     * @param string
     */
    public void setTypeByString(String string) {
        setType(string);
    }

    /**
     * Sets the property value by String.
     *
     * @param string
     */
    public void setValueByString(String string) {
        setValue(string);
    }

    /**
     * Returns a String representation of this object.
     * While this method informs as XML format representaion, 
     *  it's purpose is just information, not making 
     * a rigid XML documentation.
     *
     * @return String
     */
    public String toString() {
        try {
            return (makeTextDocument());
        } catch (Exception e) {
            return (super.toString());
        }
    }

    /**
     * Accepts the Visitor for enter behavior.
     *
     * @param visitor
     * @return boolean
     */
    public boolean enter(IRVisitor visitor) {
        return (visitor.enter(this));
    }

    /**
     * Accepts the Visitor for leave behavior.
     *
     * @param visitor
     */
    public void leave(IRVisitor visitor) {
        visitor.leave(this);
    }

    /**
     * Gets PI by name.
     *
     * @param name
     * @return String
     */
    public String rGetPi(String name) {
        return ((String)xmlPIMap_.get(name));
    }

    /**
     * Gets PI map.
     *
     * @return java.util.Map
     */
    public java.util.Map rGetPiMap() {
        return (java.util.Collections.unmodifiableMap(xmlPIMap_));
    }

    /**
     * Gets the IRNode property <b>parentRNode</b>.
     *
     * @return IRNode
     */
    public IRNode rGetParentRNode() {
        return (parentRNode_);
    }

    /**
     * Sets the IRNode property <b>parentRNode</b>.
     *
     * @param parentRNode
     */
    public void rSetParentRNode(IRNode parentRNode) {
        this.parentRNode_ = parentRNode;
    }

    /**
     * Gets child RNodes.
     *
     * @return IRNode[]
     */
    public IRNode[] rGetRNodes() {
        java.util.List classNodes = new java.util.ArrayList();
        classNodes.addAll(listcell_);
        IRNode[] nodes = new IRNode[classNodes.size()];
        return ((IRNode[])classNodes.toArray(nodes));
    }

    /**
     * Tests if a Element <code>element</code> is valid
     * for the <code>XulListitem</code>.
     *
     * @param element
     * @return boolean
     */
    public static boolean isMatch(Element element) {
        if (!URelaxer2.isTargetElement(element, "http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul", "listitem")) {
            return (false);
        }
        RStack target = new RStack(element);
        boolean $match$ = false;
        Element child;
        while (true) {
            if (!XulListcell.isMatchHungry(target)) {
                break;
            }
            $match$ = true;
        }
        if (!target.isEmptyElement()) {
            return (false);
        }
        return (true);
    }

    /**
     * Tests if elements contained in a Stack <code>stack</code>
     * is valid for the <code>XulListitem</code>.
     * This mehtod is supposed to be used internally
     * by the Relaxer system.
     *
     * @param stack
     * @return boolean
     */
    public static boolean isMatch(RStack stack) {
        Element element = stack.peekElement();
        if (element == null) {
            return (false);
        }
        return (isMatch(element));
    }

    /**
     * Tests if elements contained in a Stack <code>stack</code>
     * is valid for the <code>XulListitem</code>.
     * This method consumes the stack contents during matching operation.
     * This mehtod is supposed to be used internally
     * by the Relaxer system.
     *
     * @param stack
     * @return boolean
     */
    public static boolean isMatchHungry(RStack stack) {
        Element element = stack.peekElement();
        if (element == null) {
            return (false);
        }
        if (isMatch(element)) {
            stack.popElement();
            return (true);
        } else {
            return (false);
        }
    }
}
