/*
 * The Relaxer artifact
 * Copyright (c) 2000-2004, ASAMI Tomoharu, All rights reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 *
 * - Redistributions of source code must retain the above copyright
 *   notice, this list of conditions and the following disclaimer. 
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
 * LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
 * OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 * WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */
package com.netpotlet.xul.element;

import java.io.IOException;
import java.io.PrintWriter;
import java.io.Writer;
import org.w3c.dom.*;

/**
 * <b>XulRuleModel</b> is generated from xul-jsf.rng by Relaxer.
 * This class is derived from:
 * 
 * <!-- for programmer
 * <define name="rule.model">
 *     <ref name="conditions"/>
 *     <optional>
 *       <ref name="bindings"/>
 *     </optional>
 *     <optional>
 *       <ref name="action"/>
 *     </optional>
 *   </define>-->
 * <!-- for javadoc -->
 * <pre> &lt;define name="rule.model"&gt;
 *     &lt;ref name="conditions"/&gt;
 *     &lt;optional&gt;
 *       &lt;ref name="bindings"/&gt;
 *     &lt;/optional&gt;
 *     &lt;optional&gt;
 *       &lt;ref name="action"/&gt;
 *     &lt;/optional&gt;
 *   &lt;/define&gt;</pre>
 *
 * @version xul-jsf.rng (Sat Sep 25 09:38:00 JST 2004)
 * @author  Relaxer 1.1b (http://www.relaxer.org)
 */
public class XulRuleModel implements java.io.Serializable, Cloneable, IRNSContainer, IRVisitable, IRNode, IXulRuleChoice {
    private RNSContext rNSContext_ = new RNSContext(this, "http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul");
    private XulConditions conditions_;
    private XulBindings bindings_;
    private XulAction action_;
    private java.util.Map xmlPIMap_;
    private IRNode parentRNode_;

    /**
     * Creates a <code>XulRuleModel</code>.
     *
     */
    public XulRuleModel() {
    }

    /**
     * Creates a <code>XulRuleModel</code>.
     *
     * @param source
     */
    public XulRuleModel(XulRuleModel source) {
        setup(source);
    }

    /**
     * Creates a <code>XulRuleModel</code> by the Stack <code>stack</code>
     * that contains Elements.
     * This constructor is supposed to be used internally
     * by the Relaxer system.
     *
     * @param stack
     */
    public XulRuleModel(RStack stack) {
        setup(stack);
    }

    /**
     * Initializes the <code>XulRuleModel</code> by the XulRuleModel <code>source</code>.
     *
     * @param source
     */
    public void setup(XulRuleModel source) {
        int size;
        if (source.conditions_ != null) {
            setConditions((XulConditions)source.getConditions().clone());
        }
        if (source.bindings_ != null) {
            setBindings((XulBindings)source.getBindings().clone());
        }
        if (source.action_ != null) {
            setAction((XulAction)source.getAction().clone());
        }
    }

    /**
     * Initializes the <code>XulRuleModel</code> by the Stack <code>stack</code>
     * that contains Elements.
     * This constructor is supposed to be used internally
     * by the Relaxer system.
     *
     * @param stack
     */
    public void setup(RStack stack) {
        Element element = stack.getContextElement();
        setConditions(new XulConditions(stack));
        if (XulBindings.isMatch(stack)) {
            setBindings(new XulBindings(stack));
        }
        if (XulAction.isMatch(stack)) {
            setAction(new XulAction(stack));
        }
    }

    /**
     * @return Object
     */
    public Object clone() {
        return (new XulRuleModel((XulRuleModel)this));
    }

    /**
     * Creates a DOM representation of the object.
     * Result is appended to the Node <code>parent</code>.
     *
     * @param parent
     */
    public void makeElement(Node parent) {
        Document doc = parent.getOwnerDocument();
        Element element = (Element)parent;
        int size;
        this.conditions_.makeElement(element);
        if (this.bindings_ != null) {
            this.bindings_.makeElement(element);
        }
        if (this.action_ != null) {
            this.action_.makeElement(element);
        }
    }

    /**
     * Gets the RNSContext property <b>RNSContext</b>.
     *
     * @return RNSContext
     */
    public RNSContext rGetRNSContext() {
        return (rNSContext_);
    }

    /**
     * Sets the RNSContext property <b>RNSContext</b>.
     *
     * @param rNSContext
     */
    public void rSetRNSContext(RNSContext rNSContext) {
        this.rNSContext_ = rNSContext;
    }

    /**
     * Gets the XulConditions property <b>conditions</b>.
     *
     * @return XulConditions
     */
    public XulConditions getConditions() {
        return (conditions_);
    }

    /**
     * Sets the XulConditions property <b>conditions</b>.
     *
     * @param conditions
     */
    public void setConditions(XulConditions conditions) {
        this.conditions_ = conditions;
        if (conditions != null) {
            conditions.rSetParentRNode(this);
        }
    }

    /**
     * Gets the XulBindings property <b>bindings</b>.
     *
     * @return XulBindings
     */
    public XulBindings getBindings() {
        return (bindings_);
    }

    /**
     * Sets the XulBindings property <b>bindings</b>.
     *
     * @param bindings
     */
    public void setBindings(XulBindings bindings) {
        this.bindings_ = bindings;
        if (bindings != null) {
            bindings.rSetParentRNode(this);
        }
    }

    /**
     * Gets the XulAction property <b>action</b>.
     *
     * @return XulAction
     */
    public XulAction getAction() {
        return (action_);
    }

    /**
     * Sets the XulAction property <b>action</b>.
     *
     * @param action
     */
    public void setAction(XulAction action) {
        this.action_ = action;
        if (action != null) {
            action.rSetParentRNode(this);
        }
    }

    /**
     * Makes an XML text representation.
     *
     * @return String
     */
    public String makeTextDocument() {
        StringBuffer buffer = new StringBuffer();
        makeTextElement(buffer);
        return (new String(buffer));
    }

    /**
     * Makes an XML text representation.
     *
     * @param buffer
     */
    public void makeTextElement(StringBuffer buffer) {
        int size;
        String prefix = rNSContext_.getPrefixByUri("http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul");
        conditions_.makeTextElement(buffer);
        if (bindings_ != null) {
            bindings_.makeTextElement(buffer);
        }
        if (action_ != null) {
            action_.makeTextElement(buffer);
        }
    }

    /**
     * Makes an XML text representation.
     *
     * @param buffer
     * @exception IOException
     */
    public void makeTextElement(Writer buffer) throws IOException {
        int size;
        String prefix = rNSContext_.getPrefixByUri("http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul");
        conditions_.makeTextElement(buffer);
        if (bindings_ != null) {
            bindings_.makeTextElement(buffer);
        }
        if (action_ != null) {
            action_.makeTextElement(buffer);
        }
    }

    /**
     * Makes an XML text representation.
     *
     * @param buffer
     */
    public void makeTextElement(PrintWriter buffer) {
        int size;
        String prefix = rNSContext_.getPrefixByUri("http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul");
        conditions_.makeTextElement(buffer);
        if (bindings_ != null) {
            bindings_.makeTextElement(buffer);
        }
        if (action_ != null) {
            action_.makeTextElement(buffer);
        }
    }

    /**
     * Makes an XML text representation.
     *
     * @param buffer
     */
    public void makeTextAttribute(StringBuffer buffer) {
        int size;
    }

    /**
     * Makes an XML text representation.
     *
     * @param buffer
     * @exception IOException
     */
    public void makeTextAttribute(Writer buffer) throws IOException {
        int size;
    }

    /**
     * Makes an XML text representation.
     *
     * @param buffer
     */
    public void makeTextAttribute(PrintWriter buffer) {
        int size;
    }

    /**
     * Returns a String representation of this object.
     * While this method informs as XML format representaion, 
     *  it's purpose is just information, not making 
     * a rigid XML documentation.
     *
     * @return String
     */
    public String toString() {
        try {
            return (makeTextDocument());
        } catch (Exception e) {
            return (super.toString());
        }
    }

    /**
     * Accepts the Visitor for enter behavior.
     *
     * @param visitor
     * @return boolean
     */
    public boolean enter(IRVisitor visitor) {
        return (visitor.enter(this));
    }

    /**
     * Accepts the Visitor for leave behavior.
     *
     * @param visitor
     */
    public void leave(IRVisitor visitor) {
        visitor.leave(this);
    }

    /**
     * Gets PI by name.
     *
     * @param name
     * @return String
     */
    public String rGetPi(String name) {
        return ((String)xmlPIMap_.get(name));
    }

    /**
     * Gets PI map.
     *
     * @return java.util.Map
     */
    public java.util.Map rGetPiMap() {
        return (java.util.Collections.unmodifiableMap(xmlPIMap_));
    }

    /**
     * Gets the IRNode property <b>parentRNode</b>.
     *
     * @return IRNode
     */
    public IRNode rGetParentRNode() {
        return (parentRNode_);
    }

    /**
     * Sets the IRNode property <b>parentRNode</b>.
     *
     * @param parentRNode
     */
    public void rSetParentRNode(IRNode parentRNode) {
        this.parentRNode_ = parentRNode;
    }

    /**
     * Gets child RNodes.
     *
     * @return IRNode[]
     */
    public IRNode[] rGetRNodes() {
        java.util.List classNodes = new java.util.ArrayList();
        if (conditions_ != null) {
            classNodes.add(conditions_);
        }
        if (bindings_ != null) {
            classNodes.add(bindings_);
        }
        if (action_ != null) {
            classNodes.add(action_);
        }
        IRNode[] nodes = new IRNode[classNodes.size()];
        return ((IRNode[])classNodes.toArray(nodes));
    }

    /**
     * Tests if elements contained in a Stack <code>stack</code>
     * is valid for the <code>XulRuleModel</code>.
     * This mehtod is supposed to be used internally
     * by the Relaxer system.
     *
     * @param stack
     * @return boolean
     */
    public static boolean isMatch(RStack stack) {
        return (isMatchHungry(stack.makeClone()));
    }

    /**
     * Tests if elements contained in a Stack <code>stack</code>
     * is valid for the <code>XulRuleModel</code>.
     * This method consumes the stack contents during matching operation.
     * This mehtod is supposed to be used internally
     * by the Relaxer system.
     *
     * @param stack
     * @return boolean
     */
    public static boolean isMatchHungry(RStack stack) {
        RStack target = stack.makeClone();
        boolean $match$ = false;
        Element element = target.peekElement();
        Element child;
        if (!XulConditions.isMatchHungry(target)) {
            return (false);
        }
        $match$ = true;
        if (XulBindings.isMatchHungry(target)) {
        }
        if (XulAction.isMatchHungry(target)) {
        }
        if ($match$) {
            stack.eat(target);
        }
        return ($match$);
    }
}
