/*
 * The Relaxer artifact
 * Copyright (c) 2000-2004, ASAMI Tomoharu, All rights reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 *
 * - Redistributions of source code must retain the above copyright
 *   notice, this list of conditions and the following disclaimer. 
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
 * LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
 * OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 * WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */
package com.netpotlet.xul.element;

import java.io.*;
import java.io.IOException;
import java.io.PrintWriter;
import java.io.Writer;
import java.net.URL;
import javax.xml.parsers.*;
import org.w3c.dom.*;
import org.xml.sax.*;

/**
 * <b>XulTab</b> is generated from xul-jsf.rng by Relaxer.
 * This class is derived from:
 * 
 * <!-- for programmer
 * <element name="tab">
 *       <ref name="core.attlist"/>
 *       <ref name="afterselected.attrib"/>
 *       <ref name="beforeselected.attrib"/>
 *       <ref name="disabled.attrib"/>
 *       <ref name="linkedpanel.attrib"/>
 *       <ref name="selected.attrib"/>
 *       <ref name="validate.attrib"/>
 *     </element>-->
 * <!-- for javadoc -->
 * <pre> &lt;element name="tab"&gt;
 *       &lt;ref name="core.attlist"/&gt;
 *       &lt;ref name="afterselected.attrib"/&gt;
 *       &lt;ref name="beforeselected.attrib"/&gt;
 *       &lt;ref name="disabled.attrib"/&gt;
 *       &lt;ref name="linkedpanel.attrib"/&gt;
 *       &lt;ref name="selected.attrib"/&gt;
 *       &lt;ref name="validate.attrib"/&gt;
 *     &lt;/element&gt;</pre>
 *
 * @version xul-jsf.rng (Sat Sep 25 09:38:05 JST 2004)
 * @author  Relaxer 1.1b (http://www.relaxer.org)
 */
public class XulTab implements java.io.Serializable, Cloneable, IRNSContainer, IRVisitable, IRNode {
    public static final String CROP_START = "start";
    public static final String CROP_END = "end";
    public static final String CROP_CENTER = "center";
    public static final String CROP_NONE = "none";
    public static final String VALIDATE_ALWAYS = "always";
    public static final String VALIDATE_NEVER = "never";

    private RNSContext rNSContext_ = new RNSContext(this, "http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul");
    private String accesskey_;
    private String crop_;
    private String image_;
    private String label_;
    private Boolean afterselected_;
    private Boolean beforeselected_;
    private Boolean disabled_;
    private String linkedpanel_;
    private Boolean selected_;
    private String validate_;
    private java.util.Map xmlPIMap_;
    private IRNode parentRNode_;

    /**
     * Creates a <code>XulTab</code>.
     *
     */
    public XulTab() {
    }

    /**
     * Creates a <code>XulTab</code>.
     *
     * @param source
     */
    public XulTab(XulTab source) {
        setup(source);
    }

    /**
     * Creates a <code>XulTab</code> by the Stack <code>stack</code>
     * that contains Elements.
     * This constructor is supposed to be used internally
     * by the Relaxer system.
     *
     * @param stack
     */
    public XulTab(RStack stack) {
        setup(stack);
    }

    /**
     * Creates a <code>XulTab</code> by the Document <code>doc</code>.
     *
     * @param doc
     */
    public XulTab(Document doc) {
        setup(doc.getDocumentElement());
    }

    /**
     * Creates a <code>XulTab</code> by the Element <code>element</code>.
     *
     * @param element
     */
    public XulTab(Element element) {
        setup(element);
    }

    /**
     * Creates a <code>XulTab</code> by the File <code>file</code>.
     *
     * @param file
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public XulTab(File file) throws IOException, SAXException, ParserConfigurationException {
        setup(file);
    }

    /**
     * Creates a <code>XulTab</code>
     * by the String representation of URI <code>uri</code>.
     *
     * @param uri
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public XulTab(String uri) throws IOException, SAXException, ParserConfigurationException {
        setup(uri);
    }

    /**
     * Creates a <code>XulTab</code> by the URL <code>url</code>.
     *
     * @param url
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public XulTab(URL url) throws IOException, SAXException, ParserConfigurationException {
        setup(url);
    }

    /**
     * Creates a <code>XulTab</code> by the InputStream <code>in</code>.
     *
     * @param in
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public XulTab(InputStream in) throws IOException, SAXException, ParserConfigurationException {
        setup(in);
    }

    /**
     * Creates a <code>XulTab</code> by the InputSource <code>is</code>.
     *
     * @param is
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public XulTab(InputSource is) throws IOException, SAXException, ParserConfigurationException {
        setup(is);
    }

    /**
     * Creates a <code>XulTab</code> by the Reader <code>reader</code>.
     *
     * @param reader
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public XulTab(Reader reader) throws IOException, SAXException, ParserConfigurationException {
        setup(reader);
    }

    /**
     * Initializes the <code>XulTab</code> by the XulTab <code>source</code>.
     *
     * @param source
     */
    public void setup(XulTab source) {
        int size;
        accesskey_ = source.accesskey_;
        crop_ = source.crop_;
        image_ = source.image_;
        label_ = source.label_;
        afterselected_ = source.afterselected_;
        beforeselected_ = source.beforeselected_;
        disabled_ = source.disabled_;
        linkedpanel_ = source.linkedpanel_;
        selected_ = source.selected_;
        validate_ = source.validate_;
    }

    /**
     * Initializes the <code>XulTab</code> by the Document <code>doc</code>.
     *
     * @param doc
     */
    public void setup(Document doc) {
        setup(doc.getDocumentElement());
    }

    /**
     * Initializes the <code>XulTab</code> by the Element <code>element</code>.
     *
     * @param element
     */
    public void setup(Element element) {
        init(element);
    }

    /**
     * Initializes the <code>XulTab</code> by the Stack <code>stack</code>
     * that contains Elements.
     * This constructor is supposed to be used internally
     * by the Relaxer system.
     *
     * @param stack
     */
    public void setup(RStack stack) {
        init(stack.popElement());
    }

    /**
     * @param element
     */
    private void init(Element element) {
        RStack stack = new RStack(element);
        rNSContext_.declareNamespace(element);
        accesskey_ = URelaxer.getAttributePropertyAsString(element, "accesskey");
        crop_ = URelaxer.getAttributePropertyAsString(element, "crop");
        image_ = URelaxer.getAttributePropertyAsString(element, "image");
        label_ = URelaxer.getAttributePropertyAsString(element, "label");
        afterselected_ = URelaxer.getAttributePropertyAsBooleanObject(element, "afterselected");
        beforeselected_ = URelaxer.getAttributePropertyAsBooleanObject(element, "beforeselected");
        disabled_ = URelaxer.getAttributePropertyAsBooleanObject(element, "disabled");
        linkedpanel_ = URelaxer.getAttributePropertyAsString(element, "linkedpanel");
        selected_ = URelaxer.getAttributePropertyAsBooleanObject(element, "selected");
        validate_ = URelaxer.getAttributePropertyAsString(element, "validate");
        xmlPIMap_ = stack.getPIMap();
    }

    /**
     * @return Object
     */
    public Object clone() {
        return (new XulTab((XulTab)this));
    }

    /**
     * Creates a DOM representation of the object.
     * Result is appended to the Node <code>parent</code>.
     *
     * @param parent
     */
    public void makeElement(Node parent) {
        Document doc;
        if (parent instanceof Document) {
            doc = (Document)parent;
        } else {
            doc = parent.getOwnerDocument();
        }
        Element element = doc.createElementNS("http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul", "tab");
        rNSContext_.setupNamespace(element);
        int size;
        if (this.accesskey_ != null) {
            URelaxer.setAttributePropertyByString(element, "accesskey", this.accesskey_);
        }
        if (this.crop_ != null) {
            URelaxer.setAttributePropertyByString(element, "crop", this.crop_);
        }
        if (this.image_ != null) {
            URelaxer.setAttributePropertyByString(element, "image", this.image_);
        }
        if (this.label_ != null) {
            URelaxer.setAttributePropertyByString(element, "label", this.label_);
        }
        if (this.afterselected_ != null) {
            URelaxer.setAttributePropertyByBoolean(element, "afterselected", this.afterselected_);
        }
        if (this.beforeselected_ != null) {
            URelaxer.setAttributePropertyByBoolean(element, "beforeselected", this.beforeselected_);
        }
        if (this.disabled_ != null) {
            URelaxer.setAttributePropertyByBoolean(element, "disabled", this.disabled_);
        }
        if (this.linkedpanel_ != null) {
            URelaxer.setAttributePropertyByString(element, "linkedpanel", this.linkedpanel_);
        }
        if (this.selected_ != null) {
            URelaxer.setAttributePropertyByBoolean(element, "selected", this.selected_);
        }
        if (this.validate_ != null) {
            URelaxer.setAttributePropertyByString(element, "validate", this.validate_);
        }
        parent.appendChild(element);
    }

    /**
     * Initializes the <code>XulTab</code> by the File <code>file</code>.
     *
     * @param file
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public void setup(File file) throws IOException, SAXException, ParserConfigurationException {
        setup(file.toURL());
    }

    /**
     * Initializes the <code>XulTab</code>
     * by the String representation of URI <code>uri</code>.
     *
     * @param uri
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public void setup(String uri) throws IOException, SAXException, ParserConfigurationException {
        setup(UJAXP.getDocument(uri, UJAXP.FLAG_NAMESPACE_AWARE));
    }

    /**
     * Initializes the <code>XulTab</code> by the URL <code>url</code>.
     *
     * @param url
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public void setup(URL url) throws IOException, SAXException, ParserConfigurationException {
        setup(UJAXP.getDocument(url, UJAXP.FLAG_NAMESPACE_AWARE));
    }

    /**
     * Initializes the <code>XulTab</code> by the InputStream <code>in</code>.
     *
     * @param in
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public void setup(InputStream in) throws IOException, SAXException, ParserConfigurationException {
        setup(UJAXP.getDocument(in, UJAXP.FLAG_NAMESPACE_AWARE));
    }

    /**
     * Initializes the <code>XulTab</code> by the InputSource <code>is</code>.
     *
     * @param is
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public void setup(InputSource is) throws IOException, SAXException, ParserConfigurationException {
        setup(UJAXP.getDocument(is, UJAXP.FLAG_NAMESPACE_AWARE));
    }

    /**
     * Initializes the <code>XulTab</code> by the Reader <code>reader</code>.
     *
     * @param reader
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public void setup(Reader reader) throws IOException, SAXException, ParserConfigurationException {
        setup(UJAXP.getDocument(reader, UJAXP.FLAG_NAMESPACE_AWARE));
    }

    /**
     * Creates a DOM document representation of the object.
     *
     * @exception ParserConfigurationException
     * @return Document
     */
    public Document makeDocument() throws ParserConfigurationException {
        Document doc = UJAXP.makeDocument();
        makeElement(doc);
        return (doc);
    }

    /**
     * Gets the RNSContext property <b>RNSContext</b>.
     *
     * @return RNSContext
     */
    public RNSContext rGetRNSContext() {
        return (rNSContext_);
    }

    /**
     * Sets the RNSContext property <b>RNSContext</b>.
     *
     * @param rNSContext
     */
    public void rSetRNSContext(RNSContext rNSContext) {
        this.rNSContext_ = rNSContext;
    }

    /**
     * Gets the String property <b>accesskey</b>.
     *
     * @return String
     */
    public String getAccesskey() {
        return (accesskey_);
    }

    /**
     * Sets the String property <b>accesskey</b>.
     *
     * @param accesskey
     */
    public void setAccesskey(String accesskey) {
        this.accesskey_ = accesskey;
    }

    /**
     * Gets the String property <b>crop</b>.
     *
     * @return String
     */
    public String getCrop() {
        return (crop_);
    }

    /**
     * Sets the String property <b>crop</b>.
     *
     * @param crop
     */
    public void setCrop(String crop) {
        this.crop_ = crop;
    }

    /**
     * Gets the String property <b>image</b>.
     *
     * @return String
     */
    public String getImage() {
        return (image_);
    }

    /**
     * Sets the String property <b>image</b>.
     *
     * @param image
     */
    public void setImage(String image) {
        this.image_ = image;
    }

    /**
     * Gets the String property <b>label</b>.
     *
     * @return String
     */
    public String getLabel() {
        return (label_);
    }

    /**
     * Sets the String property <b>label</b>.
     *
     * @param label
     */
    public void setLabel(String label) {
        this.label_ = label;
    }

    /**
     * Gets the boolean property <b>afterselected</b>.
     *
     * @return boolean
     */
    public boolean getAfterselected() {
        if (afterselected_ == null) {
            return(false);
        }
        return (afterselected_.booleanValue());
    }

    /**
     * Gets the boolean property <b>afterselected</b>.
     *
     * @param afterselected
     * @return boolean
     */
    public boolean getAfterselected(boolean afterselected) {
        if (afterselected_ == null) {
            return(afterselected);
        }
        return (this.afterselected_.booleanValue());
    }

    /**
     * Gets the boolean property <b>afterselected</b>.
     *
     * @return Boolean
     */
    public Boolean getAfterselectedAsBoolean() {
        return (afterselected_);
    }

    /**
     * Check the boolean property <b>afterselected</b>.
     *
     * @return boolean
     */
    public boolean checkAfterselected() {
        return (afterselected_ != null);
    }

    /**
     * Sets the boolean property <b>afterselected</b>.
     *
     * @param afterselected
     */
    public void setAfterselected(boolean afterselected) {
        this.afterselected_ = new Boolean(afterselected);
    }

    /**
     * Sets the boolean property <b>afterselected</b>.
     *
     * @param afterselected
     */
    public void setAfterselected(Boolean afterselected) {
        this.afterselected_ = afterselected;
    }

    /**
     * Gets the boolean property <b>beforeselected</b>.
     *
     * @return boolean
     */
    public boolean getBeforeselected() {
        if (beforeselected_ == null) {
            return(false);
        }
        return (beforeselected_.booleanValue());
    }

    /**
     * Gets the boolean property <b>beforeselected</b>.
     *
     * @param beforeselected
     * @return boolean
     */
    public boolean getBeforeselected(boolean beforeselected) {
        if (beforeselected_ == null) {
            return(beforeselected);
        }
        return (this.beforeselected_.booleanValue());
    }

    /**
     * Gets the boolean property <b>beforeselected</b>.
     *
     * @return Boolean
     */
    public Boolean getBeforeselectedAsBoolean() {
        return (beforeselected_);
    }

    /**
     * Check the boolean property <b>beforeselected</b>.
     *
     * @return boolean
     */
    public boolean checkBeforeselected() {
        return (beforeselected_ != null);
    }

    /**
     * Sets the boolean property <b>beforeselected</b>.
     *
     * @param beforeselected
     */
    public void setBeforeselected(boolean beforeselected) {
        this.beforeselected_ = new Boolean(beforeselected);
    }

    /**
     * Sets the boolean property <b>beforeselected</b>.
     *
     * @param beforeselected
     */
    public void setBeforeselected(Boolean beforeselected) {
        this.beforeselected_ = beforeselected;
    }

    /**
     * Gets the boolean property <b>disabled</b>.
     *
     * @return boolean
     */
    public boolean getDisabled() {
        if (disabled_ == null) {
            return(false);
        }
        return (disabled_.booleanValue());
    }

    /**
     * Gets the boolean property <b>disabled</b>.
     *
     * @param disabled
     * @return boolean
     */
    public boolean getDisabled(boolean disabled) {
        if (disabled_ == null) {
            return(disabled);
        }
        return (this.disabled_.booleanValue());
    }

    /**
     * Gets the boolean property <b>disabled</b>.
     *
     * @return Boolean
     */
    public Boolean getDisabledAsBoolean() {
        return (disabled_);
    }

    /**
     * Check the boolean property <b>disabled</b>.
     *
     * @return boolean
     */
    public boolean checkDisabled() {
        return (disabled_ != null);
    }

    /**
     * Sets the boolean property <b>disabled</b>.
     *
     * @param disabled
     */
    public void setDisabled(boolean disabled) {
        this.disabled_ = new Boolean(disabled);
    }

    /**
     * Sets the boolean property <b>disabled</b>.
     *
     * @param disabled
     */
    public void setDisabled(Boolean disabled) {
        this.disabled_ = disabled;
    }

    /**
     * Gets the String property <b>linkedpanel</b>.
     *
     * @return String
     */
    public String getLinkedpanel() {
        return (linkedpanel_);
    }

    /**
     * Sets the String property <b>linkedpanel</b>.
     *
     * @param linkedpanel
     */
    public void setLinkedpanel(String linkedpanel) {
        this.linkedpanel_ = linkedpanel;
    }

    /**
     * Gets the boolean property <b>selected</b>.
     *
     * @return boolean
     */
    public boolean getSelected() {
        if (selected_ == null) {
            return(false);
        }
        return (selected_.booleanValue());
    }

    /**
     * Gets the boolean property <b>selected</b>.
     *
     * @param selected
     * @return boolean
     */
    public boolean getSelected(boolean selected) {
        if (selected_ == null) {
            return(selected);
        }
        return (this.selected_.booleanValue());
    }

    /**
     * Gets the boolean property <b>selected</b>.
     *
     * @return Boolean
     */
    public Boolean getSelectedAsBoolean() {
        return (selected_);
    }

    /**
     * Check the boolean property <b>selected</b>.
     *
     * @return boolean
     */
    public boolean checkSelected() {
        return (selected_ != null);
    }

    /**
     * Sets the boolean property <b>selected</b>.
     *
     * @param selected
     */
    public void setSelected(boolean selected) {
        this.selected_ = new Boolean(selected);
    }

    /**
     * Sets the boolean property <b>selected</b>.
     *
     * @param selected
     */
    public void setSelected(Boolean selected) {
        this.selected_ = selected;
    }

    /**
     * Gets the String property <b>validate</b>.
     *
     * @return String
     */
    public String getValidate() {
        return (validate_);
    }

    /**
     * Sets the String property <b>validate</b>.
     *
     * @param validate
     */
    public void setValidate(String validate) {
        this.validate_ = validate;
    }

    /**
     * Makes an XML text representation.
     *
     * @return String
     */
    public String makeTextDocument() {
        StringBuffer buffer = new StringBuffer();
        makeTextElement(buffer);
        return (new String(buffer));
    }

    /**
     * Makes an XML text representation.
     *
     * @param buffer
     */
    public void makeTextElement(StringBuffer buffer) {
        int size;
        String prefix = rNSContext_.getPrefixByUri("http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul");
        buffer.append("<");
        URelaxer.makeQName(prefix, "tab", buffer);
        rNSContext_.makeNSMappings(buffer);
        if (accesskey_ != null) {
            buffer.append(" ");
            rNSContext_.makeQName("", "accesskey", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.escapeAttrQuot(URelaxer.getString(getAccesskey())));
            buffer.append("\"");
        }
        if (crop_ != null) {
            buffer.append(" ");
            rNSContext_.makeQName("", "crop", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.escapeAttrQuot(URelaxer.getString(getCrop())));
            buffer.append("\"");
        }
        if (image_ != null) {
            buffer.append(" ");
            rNSContext_.makeQName("", "image", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.escapeAttrQuot(URelaxer.getString(getImage())));
            buffer.append("\"");
        }
        if (label_ != null) {
            buffer.append(" ");
            rNSContext_.makeQName("", "label", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.escapeAttrQuot(URelaxer.getString(getLabel())));
            buffer.append("\"");
        }
        if (afterselected_ != null) {
            buffer.append(" ");
            rNSContext_.makeQName("", "afterselected", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.getString(getAfterselected()));
            buffer.append("\"");
        }
        if (beforeselected_ != null) {
            buffer.append(" ");
            rNSContext_.makeQName("", "beforeselected", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.getString(getBeforeselected()));
            buffer.append("\"");
        }
        if (disabled_ != null) {
            buffer.append(" ");
            rNSContext_.makeQName("", "disabled", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.getString(getDisabled()));
            buffer.append("\"");
        }
        if (linkedpanel_ != null) {
            buffer.append(" ");
            rNSContext_.makeQName("", "linkedpanel", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.escapeAttrQuot(URelaxer.getString(getLinkedpanel())));
            buffer.append("\"");
        }
        if (selected_ != null) {
            buffer.append(" ");
            rNSContext_.makeQName("", "selected", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.getString(getSelected()));
            buffer.append("\"");
        }
        if (validate_ != null) {
            buffer.append(" ");
            rNSContext_.makeQName("", "validate", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.escapeAttrQuot(URelaxer.getString(getValidate())));
            buffer.append("\"");
        }
        buffer.append(">");
        buffer.append("</");
        URelaxer.makeQName(prefix, "tab", buffer);
        buffer.append(">");
    }

    /**
     * Makes an XML text representation.
     *
     * @param buffer
     * @exception IOException
     */
    public void makeTextElement(Writer buffer) throws IOException {
        int size;
        String prefix = rNSContext_.getPrefixByUri("http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul");
        buffer.write("<");
        URelaxer.makeQName(prefix, "tab", buffer);
        rNSContext_.makeNSMappings(buffer);
        if (accesskey_ != null) {
            buffer.write(" ");
            rNSContext_.makeQName("", "accesskey", buffer);
            buffer.write("=\"");
            buffer.write(URelaxer.escapeAttrQuot(URelaxer.getString(getAccesskey())));
            buffer.write("\"");
        }
        if (crop_ != null) {
            buffer.write(" ");
            rNSContext_.makeQName("", "crop", buffer);
            buffer.write("=\"");
            buffer.write(URelaxer.escapeAttrQuot(URelaxer.getString(getCrop())));
            buffer.write("\"");
        }
        if (image_ != null) {
            buffer.write(" ");
            rNSContext_.makeQName("", "image", buffer);
            buffer.write("=\"");
            buffer.write(URelaxer.escapeAttrQuot(URelaxer.getString(getImage())));
            buffer.write("\"");
        }
        if (label_ != null) {
            buffer.write(" ");
            rNSContext_.makeQName("", "label", buffer);
            buffer.write("=\"");
            buffer.write(URelaxer.escapeAttrQuot(URelaxer.getString(getLabel())));
            buffer.write("\"");
        }
        if (afterselected_ != null) {
            buffer.write(" ");
            rNSContext_.makeQName("", "afterselected", buffer);
            buffer.write("=\"");
            buffer.write(URelaxer.getString(getAfterselected()));
            buffer.write("\"");
        }
        if (beforeselected_ != null) {
            buffer.write(" ");
            rNSContext_.makeQName("", "beforeselected", buffer);
            buffer.write("=\"");
            buffer.write(URelaxer.getString(getBeforeselected()));
            buffer.write("\"");
        }
        if (disabled_ != null) {
            buffer.write(" ");
            rNSContext_.makeQName("", "disabled", buffer);
            buffer.write("=\"");
            buffer.write(URelaxer.getString(getDisabled()));
            buffer.write("\"");
        }
        if (linkedpanel_ != null) {
            buffer.write(" ");
            rNSContext_.makeQName("", "linkedpanel", buffer);
            buffer.write("=\"");
            buffer.write(URelaxer.escapeAttrQuot(URelaxer.getString(getLinkedpanel())));
            buffer.write("\"");
        }
        if (selected_ != null) {
            buffer.write(" ");
            rNSContext_.makeQName("", "selected", buffer);
            buffer.write("=\"");
            buffer.write(URelaxer.getString(getSelected()));
            buffer.write("\"");
        }
        if (validate_ != null) {
            buffer.write(" ");
            rNSContext_.makeQName("", "validate", buffer);
            buffer.write("=\"");
            buffer.write(URelaxer.escapeAttrQuot(URelaxer.getString(getValidate())));
            buffer.write("\"");
        }
        buffer.write(">");
        buffer.write("</");
        URelaxer.makeQName(prefix, "tab", buffer);
        buffer.write(">");
    }

    /**
     * Makes an XML text representation.
     *
     * @param buffer
     */
    public void makeTextElement(PrintWriter buffer) {
        int size;
        String prefix = rNSContext_.getPrefixByUri("http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul");
        buffer.print("<");
        URelaxer.makeQName(prefix, "tab", buffer);
        rNSContext_.makeNSMappings(buffer);
        if (accesskey_ != null) {
            buffer.print(" ");
            rNSContext_.makeQName("", "accesskey", buffer);
            buffer.print("=\"");
            buffer.print(URelaxer.escapeAttrQuot(URelaxer.getString(getAccesskey())));
            buffer.print("\"");
        }
        if (crop_ != null) {
            buffer.print(" ");
            rNSContext_.makeQName("", "crop", buffer);
            buffer.print("=\"");
            buffer.print(URelaxer.escapeAttrQuot(URelaxer.getString(getCrop())));
            buffer.print("\"");
        }
        if (image_ != null) {
            buffer.print(" ");
            rNSContext_.makeQName("", "image", buffer);
            buffer.print("=\"");
            buffer.print(URelaxer.escapeAttrQuot(URelaxer.getString(getImage())));
            buffer.print("\"");
        }
        if (label_ != null) {
            buffer.print(" ");
            rNSContext_.makeQName("", "label", buffer);
            buffer.print("=\"");
            buffer.print(URelaxer.escapeAttrQuot(URelaxer.getString(getLabel())));
            buffer.print("\"");
        }
        if (afterselected_ != null) {
            buffer.print(" ");
            rNSContext_.makeQName("", "afterselected", buffer);
            buffer.print("=\"");
            buffer.print(URelaxer.getString(getAfterselected()));
            buffer.print("\"");
        }
        if (beforeselected_ != null) {
            buffer.print(" ");
            rNSContext_.makeQName("", "beforeselected", buffer);
            buffer.print("=\"");
            buffer.print(URelaxer.getString(getBeforeselected()));
            buffer.print("\"");
        }
        if (disabled_ != null) {
            buffer.print(" ");
            rNSContext_.makeQName("", "disabled", buffer);
            buffer.print("=\"");
            buffer.print(URelaxer.getString(getDisabled()));
            buffer.print("\"");
        }
        if (linkedpanel_ != null) {
            buffer.print(" ");
            rNSContext_.makeQName("", "linkedpanel", buffer);
            buffer.print("=\"");
            buffer.print(URelaxer.escapeAttrQuot(URelaxer.getString(getLinkedpanel())));
            buffer.print("\"");
        }
        if (selected_ != null) {
            buffer.print(" ");
            rNSContext_.makeQName("", "selected", buffer);
            buffer.print("=\"");
            buffer.print(URelaxer.getString(getSelected()));
            buffer.print("\"");
        }
        if (validate_ != null) {
            buffer.print(" ");
            rNSContext_.makeQName("", "validate", buffer);
            buffer.print("=\"");
            buffer.print(URelaxer.escapeAttrQuot(URelaxer.getString(getValidate())));
            buffer.print("\"");
        }
        buffer.print(">");
        buffer.print("</");
        URelaxer.makeQName(prefix, "tab", buffer);
        buffer.print(">");
    }

    /**
     * Makes an XML text representation.
     *
     * @param buffer
     */
    public void makeTextAttribute(StringBuffer buffer) {
    }

    /**
     * Makes an XML text representation.
     *
     * @param buffer
     * @exception IOException
     */
    public void makeTextAttribute(Writer buffer) throws IOException {
    }

    /**
     * Makes an XML text representation.
     *
     * @param buffer
     */
    public void makeTextAttribute(PrintWriter buffer) {
    }

    /**
     * Gets the property value as String.
     *
     * @return String
     */
    public String getAccesskeyAsString() {
        return (URelaxer.getString(getAccesskey()));
    }

    /**
     * Gets the property value as String.
     *
     * @return String
     */
    public String getCropAsString() {
        return (URelaxer.getString(getCrop()));
    }

    /**
     * Gets the property value as String.
     *
     * @return String
     */
    public String getImageAsString() {
        return (URelaxer.getString(getImage()));
    }

    /**
     * Gets the property value as String.
     *
     * @return String
     */
    public String getLabelAsString() {
        return (URelaxer.getString(getLabel()));
    }

    /**
     * Gets the property value as String.
     *
     * @return String
     */
    public String getAfterselectedAsString() {
        return (URelaxer.getString(getAfterselected()));
    }

    /**
     * Gets the property value as String.
     *
     * @return String
     */
    public String getBeforeselectedAsString() {
        return (URelaxer.getString(getBeforeselected()));
    }

    /**
     * Gets the property value as String.
     *
     * @return String
     */
    public String getDisabledAsString() {
        return (URelaxer.getString(getDisabled()));
    }

    /**
     * Gets the property value as String.
     *
     * @return String
     */
    public String getLinkedpanelAsString() {
        return (URelaxer.getString(getLinkedpanel()));
    }

    /**
     * Gets the property value as String.
     *
     * @return String
     */
    public String getSelectedAsString() {
        return (URelaxer.getString(getSelected()));
    }

    /**
     * Gets the property value as String.
     *
     * @return String
     */
    public String getValidateAsString() {
        return (URelaxer.getString(getValidate()));
    }

    /**
     * Sets the property value by String.
     *
     * @param string
     */
    public void setAccesskeyByString(String string) {
        setAccesskey(string);
    }

    /**
     * Sets the property value by String.
     *
     * @param string
     */
    public void setCropByString(String string) {
        setCrop(string);
    }

    /**
     * Sets the property value by String.
     *
     * @param string
     */
    public void setImageByString(String string) {
        setImage(string);
    }

    /**
     * Sets the property value by String.
     *
     * @param string
     */
    public void setLabelByString(String string) {
        setLabel(string);
    }

    /**
     * Sets the property value by String.
     *
     * @param string
     */
    public void setAfterselectedByString(String string) {
        setAfterselected(new Boolean(string).booleanValue());
    }

    /**
     * Sets the property value by String.
     *
     * @param string
     */
    public void setBeforeselectedByString(String string) {
        setBeforeselected(new Boolean(string).booleanValue());
    }

    /**
     * Sets the property value by String.
     *
     * @param string
     */
    public void setDisabledByString(String string) {
        setDisabled(new Boolean(string).booleanValue());
    }

    /**
     * Sets the property value by String.
     *
     * @param string
     */
    public void setLinkedpanelByString(String string) {
        setLinkedpanel(string);
    }

    /**
     * Sets the property value by String.
     *
     * @param string
     */
    public void setSelectedByString(String string) {
        setSelected(new Boolean(string).booleanValue());
    }

    /**
     * Sets the property value by String.
     *
     * @param string
     */
    public void setValidateByString(String string) {
        setValidate(string);
    }

    /**
     * Returns a String representation of this object.
     * While this method informs as XML format representaion, 
     *  it's purpose is just information, not making 
     * a rigid XML documentation.
     *
     * @return String
     */
    public String toString() {
        try {
            return (makeTextDocument());
        } catch (Exception e) {
            return (super.toString());
        }
    }

    /**
     * Accepts the Visitor for enter behavior.
     *
     * @param visitor
     * @return boolean
     */
    public boolean enter(IRVisitor visitor) {
        return (visitor.enter(this));
    }

    /**
     * Accepts the Visitor for leave behavior.
     *
     * @param visitor
     */
    public void leave(IRVisitor visitor) {
        visitor.leave(this);
    }

    /**
     * Gets PI by name.
     *
     * @param name
     * @return String
     */
    public String rGetPi(String name) {
        return ((String)xmlPIMap_.get(name));
    }

    /**
     * Gets PI map.
     *
     * @return java.util.Map
     */
    public java.util.Map rGetPiMap() {
        return (java.util.Collections.unmodifiableMap(xmlPIMap_));
    }

    /**
     * Gets the IRNode property <b>parentRNode</b>.
     *
     * @return IRNode
     */
    public IRNode rGetParentRNode() {
        return (parentRNode_);
    }

    /**
     * Sets the IRNode property <b>parentRNode</b>.
     *
     * @param parentRNode
     */
    public void rSetParentRNode(IRNode parentRNode) {
        this.parentRNode_ = parentRNode;
    }

    /**
     * Gets child RNodes.
     *
     * @return IRNode[]
     */
    public IRNode[] rGetRNodes() {
        java.util.List classNodes = new java.util.ArrayList();
        IRNode[] nodes = new IRNode[classNodes.size()];
        return ((IRNode[])classNodes.toArray(nodes));
    }

    /**
     * Tests if a Element <code>element</code> is valid
     * for the <code>XulTab</code>.
     *
     * @param element
     * @return boolean
     */
    public static boolean isMatch(Element element) {
        if (!URelaxer2.isTargetElement(element, "http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul", "tab")) {
            return (false);
        }
        RStack target = new RStack(element);
        boolean $match$ = false;
        Element child;
        if (!target.isEmptyElement()) {
            return (false);
        }
        return (true);
    }

    /**
     * Tests if elements contained in a Stack <code>stack</code>
     * is valid for the <code>XulTab</code>.
     * This mehtod is supposed to be used internally
     * by the Relaxer system.
     *
     * @param stack
     * @return boolean
     */
    public static boolean isMatch(RStack stack) {
        Element element = stack.peekElement();
        if (element == null) {
            return (false);
        }
        return (isMatch(element));
    }

    /**
     * Tests if elements contained in a Stack <code>stack</code>
     * is valid for the <code>XulTab</code>.
     * This method consumes the stack contents during matching operation.
     * This mehtod is supposed to be used internally
     * by the Relaxer system.
     *
     * @param stack
     * @return boolean
     */
    public static boolean isMatchHungry(RStack stack) {
        Element element = stack.peekElement();
        if (element == null) {
            return (false);
        }
        if (isMatch(element)) {
            stack.popElement();
            return (true);
        } else {
            return (false);
        }
    }
}
