/*
 * The Relaxer artifact
 * Copyright (c) 2000-2004, ASAMI Tomoharu, All rights reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 *
 * - Redistributions of source code must retain the above copyright
 *   notice, this list of conditions and the following disclaimer. 
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
 * LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
 * OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 * WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */
package com.netpotlet.xul.element;

import java.io.*;
import java.io.IOException;
import java.io.PrintWriter;
import java.io.Writer;
import java.net.URL;
import javax.xml.parsers.*;
import org.w3c.dom.*;
import org.xml.sax.*;

/**
 * <b>XulWindow</b> is generated from xul-jsf.rng by Relaxer.
 * This class is derived from:
 * 
 * <!-- for programmer
 * <element name="window">
 *       <ref name="xul.attlist"/>
 *       <ref name="script.attlist"/>
 *       <ref name="window.attlist"/>
 *       <ref name="Block.model"/>
 *     </element>-->
 * <!-- for javadoc -->
 * <pre> &lt;element name="window"&gt;
 *       &lt;ref name="xul.attlist"/&gt;
 *       &lt;ref name="script.attlist"/&gt;
 *       &lt;ref name="window.attlist"/&gt;
 *       &lt;ref name="Block.model"/&gt;
 *     &lt;/element&gt;</pre>
 *
 * @version xul-jsf.rng (Sat Sep 25 09:37:59 JST 2004)
 * @author  Relaxer 1.1b (http://www.relaxer.org)
 */
public class XulWindow implements java.io.Serializable, Cloneable, IRNSContainer, IRVisitable, IRNode {
    public static final String ALIGN_START = "start";
    public static final String ALIGN_CENTER = "center";
    public static final String ALIGN_END = "end";
    public static final String ALIGN_BASELINE = "baseline";
    public static final String ALIGN_STRETCH = "stretch";
    public static final String ALIGN_LEFT = "left";
    public static final String DIR_LTR = "ltr";
    public static final String DIR_RTL = "rtl";
    public static final String EQUALSIZE_ALWAYS = "always";
    public static final String EQUALSIZE_NEVER = "never";
    public static final String FLAGS_DONT_TEST_EMPTY = "dont-test-empty";
    public static final String FLAGS_DONT_BUILD_CONTENT = "dont-build-content";
    public static final String ORIENT_HORIZONTAL = "horizontal";
    public static final String ORIENT_VERTICAL = "vertical";
    public static final String PACK_START = "start";
    public static final String PACK_CENTER = "center";
    public static final String PACK_END = "end";
    public static final String SORTDIRECTION_ASCENDING = "ascending";
    public static final String SORTDIRECTION_DESCENDING = "descending";
    public static final String SORTDIRECTION_NATURAL = "natural";
    public static final String SIZEMODE_MAXIMIZED = "maximized";
    public static final String SIZEMODE_MINIMIZED = "minimized";
    public static final String SIZEMODE_NORMAL = "normal";

    private RNSContext rNSContext_ = new RNSContext(this, "http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul");
    private String align_;
    private Boolean allowevents_;
    private Boolean allownegativeassertions_;
    private String classValue_;
    private Boolean coalesceduplicatearcs_;
    private Boolean collapsed_;
    private String container_;
    private String containment_;
    private String context_;
    private String contextmenu_;
    private String datasources_;
    private Boolean debug_;
    private String dir_;
    private Boolean empty_;
    private String equalsize_;
    private String flags_;
    private java.math.BigInteger flex_;
    private java.math.BigInteger height_;
    private Boolean hidden_;
    private String id_;
    private String insertafter_;
    private String insertbefore_;
    private java.math.BigInteger left_;
    private java.math.BigInteger maxheight_;
    private java.math.BigInteger maxwidth_;
    private String menu_;
    private java.math.BigInteger minheight_;
    private java.math.BigInteger minwidth_;
    private String observes_;
    private java.math.BigInteger ordinal_;
    private String orient_;
    private String pack_;
    private String persist_;
    private String popup_;
    private String position_;
    private String ref_;
    private String removeelement_;
    private String sortDirection_;
    private String sortResource_;
    private String sortResource2_;
    private String statustext_;
    private String style_;
    private String template_;
    private String tooltip_;
    private String tooltiptext_;
    private java.math.BigInteger top_;
    private String uri_;
    private java.math.BigInteger width_;
    private String onclick_;
    private String onmousedown_;
    private String onmouseup_;
    private String onmouseover_;
    private String onmousemove_;
    private String onmouseout_;
    private String oncommand_;
    private String onkeypress_;
    private String onkeydown_;
    private String onkeyup_;
    private String onfocus_;
    private String onblur_;
    private String onload_;
    private String onunload_;
    private Boolean hidechrome_;
    private java.math.BigInteger screenX_;
    private java.math.BigInteger screenY_;
    private String sizemode_;
    private String title_;
    private String windowtype_;
    // List<IXulWindowChoice>
    private java.util.List blockClass_ = new java.util.ArrayList();
    private java.util.Map xmlPIMap_;
    private IRNode parentRNode_;

    /**
     * Creates a <code>XulWindow</code>.
     *
     */
    public XulWindow() {
    }

    /**
     * Creates a <code>XulWindow</code>.
     *
     * @param source
     */
    public XulWindow(XulWindow source) {
        setup(source);
    }

    /**
     * Creates a <code>XulWindow</code> by the Stack <code>stack</code>
     * that contains Elements.
     * This constructor is supposed to be used internally
     * by the Relaxer system.
     *
     * @param stack
     */
    public XulWindow(RStack stack) {
        setup(stack);
    }

    /**
     * Creates a <code>XulWindow</code> by the Document <code>doc</code>.
     *
     * @param doc
     */
    public XulWindow(Document doc) {
        setup(doc.getDocumentElement());
    }

    /**
     * Creates a <code>XulWindow</code> by the Element <code>element</code>.
     *
     * @param element
     */
    public XulWindow(Element element) {
        setup(element);
    }

    /**
     * Creates a <code>XulWindow</code> by the File <code>file</code>.
     *
     * @param file
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public XulWindow(File file) throws IOException, SAXException, ParserConfigurationException {
        setup(file);
    }

    /**
     * Creates a <code>XulWindow</code>
     * by the String representation of URI <code>uri</code>.
     *
     * @param uri
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public XulWindow(String uri) throws IOException, SAXException, ParserConfigurationException {
        setup(uri);
    }

    /**
     * Creates a <code>XulWindow</code> by the URL <code>url</code>.
     *
     * @param url
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public XulWindow(URL url) throws IOException, SAXException, ParserConfigurationException {
        setup(url);
    }

    /**
     * Creates a <code>XulWindow</code> by the InputStream <code>in</code>.
     *
     * @param in
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public XulWindow(InputStream in) throws IOException, SAXException, ParserConfigurationException {
        setup(in);
    }

    /**
     * Creates a <code>XulWindow</code> by the InputSource <code>is</code>.
     *
     * @param is
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public XulWindow(InputSource is) throws IOException, SAXException, ParserConfigurationException {
        setup(is);
    }

    /**
     * Creates a <code>XulWindow</code> by the Reader <code>reader</code>.
     *
     * @param reader
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public XulWindow(Reader reader) throws IOException, SAXException, ParserConfigurationException {
        setup(reader);
    }

    /**
     * Initializes the <code>XulWindow</code> by the XulWindow <code>source</code>.
     *
     * @param source
     */
    public void setup(XulWindow source) {
        int size;
        align_ = source.align_;
        allowevents_ = source.allowevents_;
        allownegativeassertions_ = source.allownegativeassertions_;
        classValue_ = source.classValue_;
        coalesceduplicatearcs_ = source.coalesceduplicatearcs_;
        collapsed_ = source.collapsed_;
        container_ = source.container_;
        containment_ = source.containment_;
        context_ = source.context_;
        contextmenu_ = source.contextmenu_;
        datasources_ = source.datasources_;
        debug_ = source.debug_;
        dir_ = source.dir_;
        empty_ = source.empty_;
        equalsize_ = source.equalsize_;
        flags_ = source.flags_;
        flex_ = source.flex_;
        height_ = source.height_;
        hidden_ = source.hidden_;
        id_ = source.id_;
        insertafter_ = source.insertafter_;
        insertbefore_ = source.insertbefore_;
        left_ = source.left_;
        maxheight_ = source.maxheight_;
        maxwidth_ = source.maxwidth_;
        menu_ = source.menu_;
        minheight_ = source.minheight_;
        minwidth_ = source.minwidth_;
        observes_ = source.observes_;
        ordinal_ = source.ordinal_;
        orient_ = source.orient_;
        pack_ = source.pack_;
        persist_ = source.persist_;
        popup_ = source.popup_;
        position_ = source.position_;
        ref_ = source.ref_;
        removeelement_ = source.removeelement_;
        sortDirection_ = source.sortDirection_;
        sortResource_ = source.sortResource_;
        sortResource2_ = source.sortResource2_;
        statustext_ = source.statustext_;
        style_ = source.style_;
        template_ = source.template_;
        tooltip_ = source.tooltip_;
        tooltiptext_ = source.tooltiptext_;
        top_ = source.top_;
        uri_ = source.uri_;
        width_ = source.width_;
        onclick_ = source.onclick_;
        onmousedown_ = source.onmousedown_;
        onmouseup_ = source.onmouseup_;
        onmouseover_ = source.onmouseover_;
        onmousemove_ = source.onmousemove_;
        onmouseout_ = source.onmouseout_;
        oncommand_ = source.oncommand_;
        onkeypress_ = source.onkeypress_;
        onkeydown_ = source.onkeydown_;
        onkeyup_ = source.onkeyup_;
        onfocus_ = source.onfocus_;
        onblur_ = source.onblur_;
        onload_ = source.onload_;
        onunload_ = source.onunload_;
        hidechrome_ = source.hidechrome_;
        screenX_ = source.screenX_;
        screenY_ = source.screenY_;
        sizemode_ = source.sizemode_;
        title_ = source.title_;
        windowtype_ = source.windowtype_;
        this.blockClass_.clear();
        size = source.blockClass_.size();
        for (int i = 0;i < size;i++) {
            addBlockClass((IXulWindowChoice)source.getBlockClass(i).clone());
        }
    }

    /**
     * Initializes the <code>XulWindow</code> by the Document <code>doc</code>.
     *
     * @param doc
     */
    public void setup(Document doc) {
        setup(doc.getDocumentElement());
    }

    /**
     * Initializes the <code>XulWindow</code> by the Element <code>element</code>.
     *
     * @param element
     */
    public void setup(Element element) {
        init(element);
    }

    /**
     * Initializes the <code>XulWindow</code> by the Stack <code>stack</code>
     * that contains Elements.
     * This constructor is supposed to be used internally
     * by the Relaxer system.
     *
     * @param stack
     */
    public void setup(RStack stack) {
        init(stack.popElement());
    }

    /**
     * @param element
     */
    private void init(Element element) {
        RStack stack = new RStack(element);
        rNSContext_.declareNamespace(element);
        align_ = URelaxer.getAttributePropertyAsString(element, "align");
        allowevents_ = URelaxer.getAttributePropertyAsBooleanObject(element, "allowevents");
        allownegativeassertions_ = URelaxer.getAttributePropertyAsBooleanObject(element, "allownegativeassertions");
        classValue_ = URelaxer.getAttributePropertyAsString(element, "class");
        coalesceduplicatearcs_ = URelaxer.getAttributePropertyAsBooleanObject(element, "coalesceduplicatearcs");
        collapsed_ = URelaxer.getAttributePropertyAsBooleanObject(element, "collapsed");
        container_ = URelaxer.getAttributePropertyAsString(element, "container");
        containment_ = URelaxer.getAttributePropertyAsString(element, "containment");
        context_ = URelaxer.getAttributePropertyAsString(element, "context");
        contextmenu_ = URelaxer.getAttributePropertyAsString(element, "contextmenu");
        datasources_ = URelaxer.getAttributePropertyAsString(element, "datasources");
        debug_ = URelaxer.getAttributePropertyAsBooleanObject(element, "debug");
        dir_ = URelaxer.getAttributePropertyAsString(element, "dir");
        empty_ = URelaxer.getAttributePropertyAsBooleanObject(element, "empty");
        equalsize_ = URelaxer.getAttributePropertyAsString(element, "equalsize");
        flags_ = URelaxer.getAttributePropertyAsString(element, "flags");
        flex_ = URelaxer.getAttributePropertyAsBigInteger(element, "flex");
        height_ = URelaxer.getAttributePropertyAsBigInteger(element, "height");
        hidden_ = URelaxer.getAttributePropertyAsBooleanObject(element, "hidden");
        id_ = URelaxer.getAttributePropertyAsString(element, "id");
        insertafter_ = URelaxer.getAttributePropertyAsString(element, "insertafter");
        insertbefore_ = URelaxer.getAttributePropertyAsString(element, "insertbefore");
        left_ = URelaxer.getAttributePropertyAsBigInteger(element, "left");
        maxheight_ = URelaxer.getAttributePropertyAsBigInteger(element, "maxheight");
        maxwidth_ = URelaxer.getAttributePropertyAsBigInteger(element, "maxwidth");
        menu_ = URelaxer.getAttributePropertyAsString(element, "menu");
        minheight_ = URelaxer.getAttributePropertyAsBigInteger(element, "minheight");
        minwidth_ = URelaxer.getAttributePropertyAsBigInteger(element, "minwidth");
        observes_ = URelaxer.getAttributePropertyAsString(element, "observes");
        ordinal_ = URelaxer.getAttributePropertyAsBigInteger(element, "ordinal");
        orient_ = URelaxer.getAttributePropertyAsString(element, "orient");
        pack_ = URelaxer.getAttributePropertyAsString(element, "pack");
        persist_ = URelaxer.getAttributePropertyAsString(element, "persist");
        popup_ = URelaxer.getAttributePropertyAsString(element, "popup");
        position_ = URelaxer.getAttributePropertyAsString(element, "position");
        ref_ = URelaxer.getAttributePropertyAsString(element, "ref");
        removeelement_ = URelaxer.getAttributePropertyAsString(element, "removeelement");
        sortDirection_ = URelaxer.getAttributePropertyAsString(element, "sortDirection");
        sortResource_ = URelaxer.getAttributePropertyAsString(element, "sortResource");
        sortResource2_ = URelaxer.getAttributePropertyAsString(element, "sortResource2");
        statustext_ = URelaxer.getAttributePropertyAsString(element, "statustext");
        style_ = URelaxer.getAttributePropertyAsString(element, "style");
        template_ = URelaxer.getAttributePropertyAsString(element, "template");
        tooltip_ = URelaxer.getAttributePropertyAsString(element, "tooltip");
        tooltiptext_ = URelaxer.getAttributePropertyAsString(element, "tooltiptext");
        top_ = URelaxer.getAttributePropertyAsBigInteger(element, "top");
        uri_ = URelaxer.getAttributePropertyAsString(element, "uri");
        width_ = URelaxer.getAttributePropertyAsBigInteger(element, "width");
        onclick_ = URelaxer.getAttributePropertyAsString(element, "onclick");
        onmousedown_ = URelaxer.getAttributePropertyAsString(element, "onmousedown");
        onmouseup_ = URelaxer.getAttributePropertyAsString(element, "onmouseup");
        onmouseover_ = URelaxer.getAttributePropertyAsString(element, "onmouseover");
        onmousemove_ = URelaxer.getAttributePropertyAsString(element, "onmousemove");
        onmouseout_ = URelaxer.getAttributePropertyAsString(element, "onmouseout");
        oncommand_ = URelaxer.getAttributePropertyAsString(element, "oncommand");
        onkeypress_ = URelaxer.getAttributePropertyAsString(element, "onkeypress");
        onkeydown_ = URelaxer.getAttributePropertyAsString(element, "onkeydown");
        onkeyup_ = URelaxer.getAttributePropertyAsString(element, "onkeyup");
        onfocus_ = URelaxer.getAttributePropertyAsString(element, "onfocus");
        onblur_ = URelaxer.getAttributePropertyAsString(element, "onblur");
        onload_ = URelaxer.getAttributePropertyAsString(element, "onload");
        onunload_ = URelaxer.getAttributePropertyAsString(element, "onunload");
        hidechrome_ = URelaxer.getAttributePropertyAsBooleanObject(element, "hidechrome");
        screenX_ = URelaxer.getAttributePropertyAsBigInteger(element, "screenX");
        screenY_ = URelaxer.getAttributePropertyAsBigInteger(element, "screenY");
        sizemode_ = URelaxer.getAttributePropertyAsString(element, "sizemode");
        title_ = URelaxer.getAttributePropertyAsString(element, "title");
        windowtype_ = URelaxer.getAttributePropertyAsString(element, "windowtype");
        blockClass_.clear();
        while (true) {
            if (XulButton.isMatch(stack)) {
                addBlockClass(new XulButton(stack));
            } else if (XulTextbox.isMatch(stack)) {
                addBlockClass(new XulTextbox(stack));
            } else if (XulCommandset.isMatch(stack)) {
                addBlockClass(new XulCommandset(stack));
            } else if (XulLabel.isMatch(stack)) {
                addBlockClass(new XulLabel(stack));
            } else if (XulObserves.isMatch(stack)) {
                addBlockClass(new XulObserves(stack));
            } else if (XulListbox.isMatch(stack)) {
                addBlockClass(new XulListbox(stack));
            } else if (XulRadio.isMatch(stack)) {
                addBlockClass(new XulRadio(stack));
            } else if (XulCheckbox.isMatch(stack)) {
                addBlockClass(new XulCheckbox(stack));
            } else if (XulTree.isMatch(stack)) {
                addBlockClass(new XulTree(stack));
            } else if (XulMenupopup.isMatch(stack)) {
                addBlockClass(new XulMenupopup(stack));
            } else if (XulSplitter.isMatch(stack)) {
                addBlockClass(new XulSplitter(stack));
            } else if (XulTabbox.isMatch(stack)) {
                addBlockClass(new XulTabbox(stack));
            } else if (XulImage.isMatch(stack)) {
                addBlockClass(new XulImage(stack));
            } else if (XulCommand.isMatch(stack)) {
                addBlockClass(new XulCommand(stack));
            } else if (XulDescription.isMatch(stack)) {
                addBlockClass(new XulDescription(stack));
            } else if (XulGroupbox.isMatch(stack)) {
                addBlockClass(new XulGroupbox(stack));
            } else if (XulDeck.isMatch(stack)) {
                addBlockClass(new XulDeck(stack));
            } else if (XulScript.isMatch(stack)) {
                addBlockClass(new XulScript(stack));
            } else if (XulMenubar.isMatch(stack)) {
                addBlockClass(new XulMenubar(stack));
            } else if (XulArrowscrollbox.isMatch(stack)) {
                addBlockClass(new XulArrowscrollbox(stack));
            } else if (XulBox.isMatch(stack)) {
                addBlockClass(new XulBox(stack));
            } else if (XulHbox.isMatch(stack)) {
                addBlockClass(new XulHbox(stack));
            } else if (XulVbox.isMatch(stack)) {
                addBlockClass(new XulVbox(stack));
            } else if (XulBroadcasterset.isMatch(stack)) {
                addBlockClass(new XulBroadcasterset(stack));
            } else if (XulStack.isMatch(stack)) {
                addBlockClass(new XulStack(stack));
            } else if (XulGrid.isMatch(stack)) {
                addBlockClass(new XulGrid(stack));
            } else if (XulIframe.isMatch(stack)) {
                addBlockClass(new XulIframe(stack));
            } else if (XulKeyset.isMatch(stack)) {
                addBlockClass(new XulKeyset(stack));
            } else if (XulTemplate.isMatch(stack)) {
                addBlockClass(new XulTemplate(stack));
            } else if (XulToolbox.isMatch(stack)) {
                addBlockClass(new XulToolbox(stack));
            } else if (XulBroadcaster.isMatch(stack)) {
                addBlockClass(new XulBroadcaster(stack));
            } else if (XulSeparator.isMatch(stack)) {
                addBlockClass(new XulSeparator(stack));
            } else if (XulSpacer.isMatch(stack)) {
                addBlockClass(new XulSpacer(stack));
            } else if (XulMenulist.isMatch(stack)) {
                addBlockClass(new XulMenulist(stack));
            } else if (XulRadiogroup.isMatch(stack)) {
                addBlockClass(new XulRadiogroup(stack));
            } else if (XulPopupset.isMatch(stack)) {
                addBlockClass(new XulPopupset(stack));
            } else {
                break;
            }
        }
        xmlPIMap_ = stack.getPIMap();
    }

    /**
     * @return Object
     */
    public Object clone() {
        return (new XulWindow((XulWindow)this));
    }

    /**
     * Creates a DOM representation of the object.
     * Result is appended to the Node <code>parent</code>.
     *
     * @param parent
     */
    public void makeElement(Node parent) {
        Document doc;
        if (parent instanceof Document) {
            doc = (Document)parent;
        } else {
            doc = parent.getOwnerDocument();
        }
        Element element = doc.createElementNS("http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul", "window");
        rNSContext_.setupNamespace(element);
        int size;
        if (this.align_ != null) {
            URelaxer.setAttributePropertyByString(element, "align", this.align_);
        }
        if (this.allowevents_ != null) {
            URelaxer.setAttributePropertyByBoolean(element, "allowevents", this.allowevents_);
        }
        if (this.allownegativeassertions_ != null) {
            URelaxer.setAttributePropertyByBoolean(element, "allownegativeassertions", this.allownegativeassertions_);
        }
        if (this.classValue_ != null) {
            URelaxer.setAttributePropertyByString(element, "class", this.classValue_);
        }
        if (this.coalesceduplicatearcs_ != null) {
            URelaxer.setAttributePropertyByBoolean(element, "coalesceduplicatearcs", this.coalesceduplicatearcs_);
        }
        if (this.collapsed_ != null) {
            URelaxer.setAttributePropertyByBoolean(element, "collapsed", this.collapsed_);
        }
        if (this.container_ != null) {
            URelaxer.setAttributePropertyByString(element, "container", this.container_);
        }
        if (this.containment_ != null) {
            URelaxer.setAttributePropertyByString(element, "containment", this.containment_);
        }
        if (this.context_ != null) {
            URelaxer.setAttributePropertyByString(element, "context", this.context_);
        }
        if (this.contextmenu_ != null) {
            URelaxer.setAttributePropertyByString(element, "contextmenu", this.contextmenu_);
        }
        if (this.datasources_ != null) {
            URelaxer.setAttributePropertyByString(element, "datasources", this.datasources_);
        }
        if (this.debug_ != null) {
            URelaxer.setAttributePropertyByBoolean(element, "debug", this.debug_);
        }
        if (this.dir_ != null) {
            URelaxer.setAttributePropertyByString(element, "dir", this.dir_);
        }
        if (this.empty_ != null) {
            URelaxer.setAttributePropertyByBoolean(element, "empty", this.empty_);
        }
        if (this.equalsize_ != null) {
            URelaxer.setAttributePropertyByString(element, "equalsize", this.equalsize_);
        }
        if (this.flags_ != null) {
            URelaxer.setAttributePropertyByString(element, "flags", this.flags_);
        }
        if (this.flex_ != null) {
            URelaxer.setAttributePropertyByBigInteger(element, "flex", this.flex_);
        }
        if (this.height_ != null) {
            URelaxer.setAttributePropertyByBigInteger(element, "height", this.height_);
        }
        if (this.hidden_ != null) {
            URelaxer.setAttributePropertyByBoolean(element, "hidden", this.hidden_);
        }
        if (this.id_ != null) {
            URelaxer.setAttributePropertyByString(element, "id", this.id_);
        }
        if (this.insertafter_ != null) {
            URelaxer.setAttributePropertyByString(element, "insertafter", this.insertafter_);
        }
        if (this.insertbefore_ != null) {
            URelaxer.setAttributePropertyByString(element, "insertbefore", this.insertbefore_);
        }
        if (this.left_ != null) {
            URelaxer.setAttributePropertyByBigInteger(element, "left", this.left_);
        }
        if (this.maxheight_ != null) {
            URelaxer.setAttributePropertyByBigInteger(element, "maxheight", this.maxheight_);
        }
        if (this.maxwidth_ != null) {
            URelaxer.setAttributePropertyByBigInteger(element, "maxwidth", this.maxwidth_);
        }
        if (this.menu_ != null) {
            URelaxer.setAttributePropertyByString(element, "menu", this.menu_);
        }
        if (this.minheight_ != null) {
            URelaxer.setAttributePropertyByBigInteger(element, "minheight", this.minheight_);
        }
        if (this.minwidth_ != null) {
            URelaxer.setAttributePropertyByBigInteger(element, "minwidth", this.minwidth_);
        }
        if (this.observes_ != null) {
            URelaxer.setAttributePropertyByString(element, "observes", this.observes_);
        }
        if (this.ordinal_ != null) {
            URelaxer.setAttributePropertyByBigInteger(element, "ordinal", this.ordinal_);
        }
        if (this.orient_ != null) {
            URelaxer.setAttributePropertyByString(element, "orient", this.orient_);
        }
        if (this.pack_ != null) {
            URelaxer.setAttributePropertyByString(element, "pack", this.pack_);
        }
        if (this.persist_ != null) {
            URelaxer.setAttributePropertyByString(element, "persist", this.persist_);
        }
        if (this.popup_ != null) {
            URelaxer.setAttributePropertyByString(element, "popup", this.popup_);
        }
        if (this.position_ != null) {
            URelaxer.setAttributePropertyByString(element, "position", this.position_);
        }
        if (this.ref_ != null) {
            URelaxer.setAttributePropertyByString(element, "ref", this.ref_);
        }
        if (this.removeelement_ != null) {
            URelaxer.setAttributePropertyByString(element, "removeelement", this.removeelement_);
        }
        if (this.sortDirection_ != null) {
            URelaxer.setAttributePropertyByString(element, "sortDirection", this.sortDirection_);
        }
        if (this.sortResource_ != null) {
            URelaxer.setAttributePropertyByString(element, "sortResource", this.sortResource_);
        }
        if (this.sortResource2_ != null) {
            URelaxer.setAttributePropertyByString(element, "sortResource2", this.sortResource2_);
        }
        if (this.statustext_ != null) {
            URelaxer.setAttributePropertyByString(element, "statustext", this.statustext_);
        }
        if (this.style_ != null) {
            URelaxer.setAttributePropertyByString(element, "style", this.style_);
        }
        if (this.template_ != null) {
            URelaxer.setAttributePropertyByString(element, "template", this.template_);
        }
        if (this.tooltip_ != null) {
            URelaxer.setAttributePropertyByString(element, "tooltip", this.tooltip_);
        }
        if (this.tooltiptext_ != null) {
            URelaxer.setAttributePropertyByString(element, "tooltiptext", this.tooltiptext_);
        }
        if (this.top_ != null) {
            URelaxer.setAttributePropertyByBigInteger(element, "top", this.top_);
        }
        if (this.uri_ != null) {
            URelaxer.setAttributePropertyByString(element, "uri", this.uri_);
        }
        if (this.width_ != null) {
            URelaxer.setAttributePropertyByBigInteger(element, "width", this.width_);
        }
        if (this.onclick_ != null) {
            URelaxer.setAttributePropertyByString(element, "onclick", this.onclick_);
        }
        if (this.onmousedown_ != null) {
            URelaxer.setAttributePropertyByString(element, "onmousedown", this.onmousedown_);
        }
        if (this.onmouseup_ != null) {
            URelaxer.setAttributePropertyByString(element, "onmouseup", this.onmouseup_);
        }
        if (this.onmouseover_ != null) {
            URelaxer.setAttributePropertyByString(element, "onmouseover", this.onmouseover_);
        }
        if (this.onmousemove_ != null) {
            URelaxer.setAttributePropertyByString(element, "onmousemove", this.onmousemove_);
        }
        if (this.onmouseout_ != null) {
            URelaxer.setAttributePropertyByString(element, "onmouseout", this.onmouseout_);
        }
        if (this.oncommand_ != null) {
            URelaxer.setAttributePropertyByString(element, "oncommand", this.oncommand_);
        }
        if (this.onkeypress_ != null) {
            URelaxer.setAttributePropertyByString(element, "onkeypress", this.onkeypress_);
        }
        if (this.onkeydown_ != null) {
            URelaxer.setAttributePropertyByString(element, "onkeydown", this.onkeydown_);
        }
        if (this.onkeyup_ != null) {
            URelaxer.setAttributePropertyByString(element, "onkeyup", this.onkeyup_);
        }
        if (this.onfocus_ != null) {
            URelaxer.setAttributePropertyByString(element, "onfocus", this.onfocus_);
        }
        if (this.onblur_ != null) {
            URelaxer.setAttributePropertyByString(element, "onblur", this.onblur_);
        }
        if (this.onload_ != null) {
            URelaxer.setAttributePropertyByString(element, "onload", this.onload_);
        }
        if (this.onunload_ != null) {
            URelaxer.setAttributePropertyByString(element, "onunload", this.onunload_);
        }
        if (this.hidechrome_ != null) {
            URelaxer.setAttributePropertyByBoolean(element, "hidechrome", this.hidechrome_);
        }
        if (this.screenX_ != null) {
            URelaxer.setAttributePropertyByBigInteger(element, "screenX", this.screenX_);
        }
        if (this.screenY_ != null) {
            URelaxer.setAttributePropertyByBigInteger(element, "screenY", this.screenY_);
        }
        if (this.sizemode_ != null) {
            URelaxer.setAttributePropertyByString(element, "sizemode", this.sizemode_);
        }
        if (this.title_ != null) {
            URelaxer.setAttributePropertyByString(element, "title", this.title_);
        }
        if (this.windowtype_ != null) {
            URelaxer.setAttributePropertyByString(element, "windowtype", this.windowtype_);
        }
        size = this.blockClass_.size();
        for (int i = 0;i < size;i++) {
            IXulWindowChoice value = (IXulWindowChoice)this.blockClass_.get(i);
            value.makeElement(element);
        }
        parent.appendChild(element);
    }

    /**
     * Initializes the <code>XulWindow</code> by the File <code>file</code>.
     *
     * @param file
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public void setup(File file) throws IOException, SAXException, ParserConfigurationException {
        setup(file.toURL());
    }

    /**
     * Initializes the <code>XulWindow</code>
     * by the String representation of URI <code>uri</code>.
     *
     * @param uri
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public void setup(String uri) throws IOException, SAXException, ParserConfigurationException {
        setup(UJAXP.getDocument(uri, UJAXP.FLAG_NAMESPACE_AWARE));
    }

    /**
     * Initializes the <code>XulWindow</code> by the URL <code>url</code>.
     *
     * @param url
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public void setup(URL url) throws IOException, SAXException, ParserConfigurationException {
        setup(UJAXP.getDocument(url, UJAXP.FLAG_NAMESPACE_AWARE));
    }

    /**
     * Initializes the <code>XulWindow</code> by the InputStream <code>in</code>.
     *
     * @param in
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public void setup(InputStream in) throws IOException, SAXException, ParserConfigurationException {
        setup(UJAXP.getDocument(in, UJAXP.FLAG_NAMESPACE_AWARE));
    }

    /**
     * Initializes the <code>XulWindow</code> by the InputSource <code>is</code>.
     *
     * @param is
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public void setup(InputSource is) throws IOException, SAXException, ParserConfigurationException {
        setup(UJAXP.getDocument(is, UJAXP.FLAG_NAMESPACE_AWARE));
    }

    /**
     * Initializes the <code>XulWindow</code> by the Reader <code>reader</code>.
     *
     * @param reader
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public void setup(Reader reader) throws IOException, SAXException, ParserConfigurationException {
        setup(UJAXP.getDocument(reader, UJAXP.FLAG_NAMESPACE_AWARE));
    }

    /**
     * Creates a DOM document representation of the object.
     *
     * @exception ParserConfigurationException
     * @return Document
     */
    public Document makeDocument() throws ParserConfigurationException {
        Document doc = UJAXP.makeDocument();
        makeElement(doc);
        return (doc);
    }

    /**
     * Gets the RNSContext property <b>RNSContext</b>.
     *
     * @return RNSContext
     */
    public RNSContext rGetRNSContext() {
        return (rNSContext_);
    }

    /**
     * Sets the RNSContext property <b>RNSContext</b>.
     *
     * @param rNSContext
     */
    public void rSetRNSContext(RNSContext rNSContext) {
        this.rNSContext_ = rNSContext;
    }

    /**
     * Gets the String property <b>align</b>.
     *
     * @return String
     */
    public String getAlign() {
        return (align_);
    }

    /**
     * Sets the String property <b>align</b>.
     *
     * @param align
     */
    public void setAlign(String align) {
        this.align_ = align;
    }

    /**
     * Gets the boolean property <b>allowevents</b>.
     *
     * @return boolean
     */
    public boolean getAllowevents() {
        if (allowevents_ == null) {
            return(false);
        }
        return (allowevents_.booleanValue());
    }

    /**
     * Gets the boolean property <b>allowevents</b>.
     *
     * @param allowevents
     * @return boolean
     */
    public boolean getAllowevents(boolean allowevents) {
        if (allowevents_ == null) {
            return(allowevents);
        }
        return (this.allowevents_.booleanValue());
    }

    /**
     * Gets the boolean property <b>allowevents</b>.
     *
     * @return Boolean
     */
    public Boolean getAlloweventsAsBoolean() {
        return (allowevents_);
    }

    /**
     * Check the boolean property <b>allowevents</b>.
     *
     * @return boolean
     */
    public boolean checkAllowevents() {
        return (allowevents_ != null);
    }

    /**
     * Sets the boolean property <b>allowevents</b>.
     *
     * @param allowevents
     */
    public void setAllowevents(boolean allowevents) {
        this.allowevents_ = new Boolean(allowevents);
    }

    /**
     * Sets the boolean property <b>allowevents</b>.
     *
     * @param allowevents
     */
    public void setAllowevents(Boolean allowevents) {
        this.allowevents_ = allowevents;
    }

    /**
     * Gets the boolean property <b>allownegativeassertions</b>.
     *
     * @return boolean
     */
    public boolean getAllownegativeassertions() {
        if (allownegativeassertions_ == null) {
            return(false);
        }
        return (allownegativeassertions_.booleanValue());
    }

    /**
     * Gets the boolean property <b>allownegativeassertions</b>.
     *
     * @param allownegativeassertions
     * @return boolean
     */
    public boolean getAllownegativeassertions(boolean allownegativeassertions) {
        if (allownegativeassertions_ == null) {
            return(allownegativeassertions);
        }
        return (this.allownegativeassertions_.booleanValue());
    }

    /**
     * Gets the boolean property <b>allownegativeassertions</b>.
     *
     * @return Boolean
     */
    public Boolean getAllownegativeassertionsAsBoolean() {
        return (allownegativeassertions_);
    }

    /**
     * Check the boolean property <b>allownegativeassertions</b>.
     *
     * @return boolean
     */
    public boolean checkAllownegativeassertions() {
        return (allownegativeassertions_ != null);
    }

    /**
     * Sets the boolean property <b>allownegativeassertions</b>.
     *
     * @param allownegativeassertions
     */
    public void setAllownegativeassertions(boolean allownegativeassertions) {
        this.allownegativeassertions_ = new Boolean(allownegativeassertions);
    }

    /**
     * Sets the boolean property <b>allownegativeassertions</b>.
     *
     * @param allownegativeassertions
     */
    public void setAllownegativeassertions(Boolean allownegativeassertions) {
        this.allownegativeassertions_ = allownegativeassertions;
    }

    /**
     * Gets the String property <b>classValue</b>.
     *
     * @return String
     */
    public String getClassValue() {
        return (classValue_);
    }

    /**
     * Sets the String property <b>classValue</b>.
     *
     * @param classValue
     */
    public void setClassValue(String classValue) {
        this.classValue_ = classValue;
    }

    /**
     * Gets the boolean property <b>coalesceduplicatearcs</b>.
     *
     * @return boolean
     */
    public boolean getCoalesceduplicatearcs() {
        if (coalesceduplicatearcs_ == null) {
            return(false);
        }
        return (coalesceduplicatearcs_.booleanValue());
    }

    /**
     * Gets the boolean property <b>coalesceduplicatearcs</b>.
     *
     * @param coalesceduplicatearcs
     * @return boolean
     */
    public boolean getCoalesceduplicatearcs(boolean coalesceduplicatearcs) {
        if (coalesceduplicatearcs_ == null) {
            return(coalesceduplicatearcs);
        }
        return (this.coalesceduplicatearcs_.booleanValue());
    }

    /**
     * Gets the boolean property <b>coalesceduplicatearcs</b>.
     *
     * @return Boolean
     */
    public Boolean getCoalesceduplicatearcsAsBoolean() {
        return (coalesceduplicatearcs_);
    }

    /**
     * Check the boolean property <b>coalesceduplicatearcs</b>.
     *
     * @return boolean
     */
    public boolean checkCoalesceduplicatearcs() {
        return (coalesceduplicatearcs_ != null);
    }

    /**
     * Sets the boolean property <b>coalesceduplicatearcs</b>.
     *
     * @param coalesceduplicatearcs
     */
    public void setCoalesceduplicatearcs(boolean coalesceduplicatearcs) {
        this.coalesceduplicatearcs_ = new Boolean(coalesceduplicatearcs);
    }

    /**
     * Sets the boolean property <b>coalesceduplicatearcs</b>.
     *
     * @param coalesceduplicatearcs
     */
    public void setCoalesceduplicatearcs(Boolean coalesceduplicatearcs) {
        this.coalesceduplicatearcs_ = coalesceduplicatearcs;
    }

    /**
     * Gets the boolean property <b>collapsed</b>.
     *
     * @return boolean
     */
    public boolean getCollapsed() {
        if (collapsed_ == null) {
            return(false);
        }
        return (collapsed_.booleanValue());
    }

    /**
     * Gets the boolean property <b>collapsed</b>.
     *
     * @param collapsed
     * @return boolean
     */
    public boolean getCollapsed(boolean collapsed) {
        if (collapsed_ == null) {
            return(collapsed);
        }
        return (this.collapsed_.booleanValue());
    }

    /**
     * Gets the boolean property <b>collapsed</b>.
     *
     * @return Boolean
     */
    public Boolean getCollapsedAsBoolean() {
        return (collapsed_);
    }

    /**
     * Check the boolean property <b>collapsed</b>.
     *
     * @return boolean
     */
    public boolean checkCollapsed() {
        return (collapsed_ != null);
    }

    /**
     * Sets the boolean property <b>collapsed</b>.
     *
     * @param collapsed
     */
    public void setCollapsed(boolean collapsed) {
        this.collapsed_ = new Boolean(collapsed);
    }

    /**
     * Sets the boolean property <b>collapsed</b>.
     *
     * @param collapsed
     */
    public void setCollapsed(Boolean collapsed) {
        this.collapsed_ = collapsed;
    }

    /**
     * Gets the String property <b>container</b>.
     *
     * @return String
     */
    public String getContainer() {
        return (container_);
    }

    /**
     * Sets the String property <b>container</b>.
     *
     * @param container
     */
    public void setContainer(String container) {
        this.container_ = container;
    }

    /**
     * Gets the String property <b>containment</b>.
     *
     * @return String
     */
    public String getContainment() {
        return (containment_);
    }

    /**
     * Sets the String property <b>containment</b>.
     *
     * @param containment
     */
    public void setContainment(String containment) {
        this.containment_ = containment;
    }

    /**
     * Gets the String property <b>context</b>.
     *
     * @return String
     */
    public String getContext() {
        return (context_);
    }

    /**
     * Sets the String property <b>context</b>.
     *
     * @param context
     */
    public void setContext(String context) {
        this.context_ = context;
    }

    /**
     * Gets the String property <b>contextmenu</b>.
     *
     * @return String
     */
    public String getContextmenu() {
        return (contextmenu_);
    }

    /**
     * Sets the String property <b>contextmenu</b>.
     *
     * @param contextmenu
     */
    public void setContextmenu(String contextmenu) {
        this.contextmenu_ = contextmenu;
    }

    /**
     * Gets the String property <b>datasources</b>.
     *
     * @return String
     */
    public String getDatasources() {
        return (datasources_);
    }

    /**
     * Sets the String property <b>datasources</b>.
     *
     * @param datasources
     */
    public void setDatasources(String datasources) {
        this.datasources_ = datasources;
    }

    /**
     * Gets the boolean property <b>debug</b>.
     *
     * @return boolean
     */
    public boolean getDebug() {
        if (debug_ == null) {
            return(false);
        }
        return (debug_.booleanValue());
    }

    /**
     * Gets the boolean property <b>debug</b>.
     *
     * @param debug
     * @return boolean
     */
    public boolean getDebug(boolean debug) {
        if (debug_ == null) {
            return(debug);
        }
        return (this.debug_.booleanValue());
    }

    /**
     * Gets the boolean property <b>debug</b>.
     *
     * @return Boolean
     */
    public Boolean getDebugAsBoolean() {
        return (debug_);
    }

    /**
     * Check the boolean property <b>debug</b>.
     *
     * @return boolean
     */
    public boolean checkDebug() {
        return (debug_ != null);
    }

    /**
     * Sets the boolean property <b>debug</b>.
     *
     * @param debug
     */
    public void setDebug(boolean debug) {
        this.debug_ = new Boolean(debug);
    }

    /**
     * Sets the boolean property <b>debug</b>.
     *
     * @param debug
     */
    public void setDebug(Boolean debug) {
        this.debug_ = debug;
    }

    /**
     * Gets the String property <b>dir</b>.
     *
     * @return String
     */
    public String getDir() {
        return (dir_);
    }

    /**
     * Sets the String property <b>dir</b>.
     *
     * @param dir
     */
    public void setDir(String dir) {
        this.dir_ = dir;
    }

    /**
     * Gets the boolean property <b>empty</b>.
     *
     * @return boolean
     */
    public boolean getEmpty() {
        if (empty_ == null) {
            return(false);
        }
        return (empty_.booleanValue());
    }

    /**
     * Gets the boolean property <b>empty</b>.
     *
     * @param empty
     * @return boolean
     */
    public boolean getEmpty(boolean empty) {
        if (empty_ == null) {
            return(empty);
        }
        return (this.empty_.booleanValue());
    }

    /**
     * Gets the boolean property <b>empty</b>.
     *
     * @return Boolean
     */
    public Boolean getEmptyAsBoolean() {
        return (empty_);
    }

    /**
     * Check the boolean property <b>empty</b>.
     *
     * @return boolean
     */
    public boolean checkEmpty() {
        return (empty_ != null);
    }

    /**
     * Sets the boolean property <b>empty</b>.
     *
     * @param empty
     */
    public void setEmpty(boolean empty) {
        this.empty_ = new Boolean(empty);
    }

    /**
     * Sets the boolean property <b>empty</b>.
     *
     * @param empty
     */
    public void setEmpty(Boolean empty) {
        this.empty_ = empty;
    }

    /**
     * Gets the String property <b>equalsize</b>.
     *
     * @return String
     */
    public String getEqualsize() {
        return (equalsize_);
    }

    /**
     * Sets the String property <b>equalsize</b>.
     *
     * @param equalsize
     */
    public void setEqualsize(String equalsize) {
        this.equalsize_ = equalsize;
    }

    /**
     * Gets the String property <b>flags</b>.
     *
     * @return String
     */
    public String getFlags() {
        return (flags_);
    }

    /**
     * Sets the String property <b>flags</b>.
     *
     * @param flags
     */
    public void setFlags(String flags) {
        this.flags_ = flags;
    }

    /**
     * Gets the java.math.BigInteger property <b>flex</b>.
     *
     * @return java.math.BigInteger
     */
    public java.math.BigInteger getFlex() {
        return (flex_);
    }

    /**
     * Sets the java.math.BigInteger property <b>flex</b>.
     *
     * @param flex
     */
    public void setFlex(java.math.BigInteger flex) {
        this.flex_ = flex;
    }

    /**
     * Gets the java.math.BigInteger property <b>height</b>.
     *
     * @return java.math.BigInteger
     */
    public java.math.BigInteger getHeight() {
        return (height_);
    }

    /**
     * Sets the java.math.BigInteger property <b>height</b>.
     *
     * @param height
     */
    public void setHeight(java.math.BigInteger height) {
        this.height_ = height;
    }

    /**
     * Gets the boolean property <b>hidden</b>.
     *
     * @return boolean
     */
    public boolean getHidden() {
        if (hidden_ == null) {
            return(false);
        }
        return (hidden_.booleanValue());
    }

    /**
     * Gets the boolean property <b>hidden</b>.
     *
     * @param hidden
     * @return boolean
     */
    public boolean getHidden(boolean hidden) {
        if (hidden_ == null) {
            return(hidden);
        }
        return (this.hidden_.booleanValue());
    }

    /**
     * Gets the boolean property <b>hidden</b>.
     *
     * @return Boolean
     */
    public Boolean getHiddenAsBoolean() {
        return (hidden_);
    }

    /**
     * Check the boolean property <b>hidden</b>.
     *
     * @return boolean
     */
    public boolean checkHidden() {
        return (hidden_ != null);
    }

    /**
     * Sets the boolean property <b>hidden</b>.
     *
     * @param hidden
     */
    public void setHidden(boolean hidden) {
        this.hidden_ = new Boolean(hidden);
    }

    /**
     * Sets the boolean property <b>hidden</b>.
     *
     * @param hidden
     */
    public void setHidden(Boolean hidden) {
        this.hidden_ = hidden;
    }

    /**
     * Gets the String property <b>id</b>.
     *
     * @return String
     */
    public String getId() {
        return (id_);
    }

    /**
     * Sets the String property <b>id</b>.
     *
     * @param id
     */
    public void setId(String id) {
        this.id_ = id;
    }

    /**
     * Gets the String property <b>insertafter</b>.
     *
     * @return String
     */
    public String getInsertafter() {
        return (insertafter_);
    }

    /**
     * Sets the String property <b>insertafter</b>.
     *
     * @param insertafter
     */
    public void setInsertafter(String insertafter) {
        this.insertafter_ = insertafter;
    }

    /**
     * Gets the String property <b>insertbefore</b>.
     *
     * @return String
     */
    public String getInsertbefore() {
        return (insertbefore_);
    }

    /**
     * Sets the String property <b>insertbefore</b>.
     *
     * @param insertbefore
     */
    public void setInsertbefore(String insertbefore) {
        this.insertbefore_ = insertbefore;
    }

    /**
     * Gets the java.math.BigInteger property <b>left</b>.
     *
     * @return java.math.BigInteger
     */
    public java.math.BigInteger getLeft() {
        return (left_);
    }

    /**
     * Sets the java.math.BigInteger property <b>left</b>.
     *
     * @param left
     */
    public void setLeft(java.math.BigInteger left) {
        this.left_ = left;
    }

    /**
     * Gets the java.math.BigInteger property <b>maxheight</b>.
     *
     * @return java.math.BigInteger
     */
    public java.math.BigInteger getMaxheight() {
        return (maxheight_);
    }

    /**
     * Sets the java.math.BigInteger property <b>maxheight</b>.
     *
     * @param maxheight
     */
    public void setMaxheight(java.math.BigInteger maxheight) {
        this.maxheight_ = maxheight;
    }

    /**
     * Gets the java.math.BigInteger property <b>maxwidth</b>.
     *
     * @return java.math.BigInteger
     */
    public java.math.BigInteger getMaxwidth() {
        return (maxwidth_);
    }

    /**
     * Sets the java.math.BigInteger property <b>maxwidth</b>.
     *
     * @param maxwidth
     */
    public void setMaxwidth(java.math.BigInteger maxwidth) {
        this.maxwidth_ = maxwidth;
    }

    /**
     * Gets the String property <b>menu</b>.
     *
     * @return String
     */
    public String getMenu() {
        return (menu_);
    }

    /**
     * Sets the String property <b>menu</b>.
     *
     * @param menu
     */
    public void setMenu(String menu) {
        this.menu_ = menu;
    }

    /**
     * Gets the java.math.BigInteger property <b>minheight</b>.
     *
     * @return java.math.BigInteger
     */
    public java.math.BigInteger getMinheight() {
        return (minheight_);
    }

    /**
     * Sets the java.math.BigInteger property <b>minheight</b>.
     *
     * @param minheight
     */
    public void setMinheight(java.math.BigInteger minheight) {
        this.minheight_ = minheight;
    }

    /**
     * Gets the java.math.BigInteger property <b>minwidth</b>.
     *
     * @return java.math.BigInteger
     */
    public java.math.BigInteger getMinwidth() {
        return (minwidth_);
    }

    /**
     * Sets the java.math.BigInteger property <b>minwidth</b>.
     *
     * @param minwidth
     */
    public void setMinwidth(java.math.BigInteger minwidth) {
        this.minwidth_ = minwidth;
    }

    /**
     * Gets the String property <b>observes</b>.
     *
     * @return String
     */
    public String getObserves() {
        return (observes_);
    }

    /**
     * Sets the String property <b>observes</b>.
     *
     * @param observes
     */
    public void setObserves(String observes) {
        this.observes_ = observes;
    }

    /**
     * Gets the java.math.BigInteger property <b>ordinal</b>.
     *
     * @return java.math.BigInteger
     */
    public java.math.BigInteger getOrdinal() {
        return (ordinal_);
    }

    /**
     * Sets the java.math.BigInteger property <b>ordinal</b>.
     *
     * @param ordinal
     */
    public void setOrdinal(java.math.BigInteger ordinal) {
        this.ordinal_ = ordinal;
    }

    /**
     * Gets the String property <b>orient</b>.
     *
     * @return String
     */
    public String getOrient() {
        return (orient_);
    }

    /**
     * Sets the String property <b>orient</b>.
     *
     * @param orient
     */
    public void setOrient(String orient) {
        this.orient_ = orient;
    }

    /**
     * Gets the String property <b>pack</b>.
     *
     * @return String
     */
    public String getPack() {
        return (pack_);
    }

    /**
     * Sets the String property <b>pack</b>.
     *
     * @param pack
     */
    public void setPack(String pack) {
        this.pack_ = pack;
    }

    /**
     * Gets the String property <b>persist</b>.
     *
     * @return String
     */
    public String getPersist() {
        return (persist_);
    }

    /**
     * Sets the String property <b>persist</b>.
     *
     * @param persist
     */
    public void setPersist(String persist) {
        this.persist_ = persist;
    }

    /**
     * Gets the String property <b>popup</b>.
     *
     * @return String
     */
    public String getPopup() {
        return (popup_);
    }

    /**
     * Sets the String property <b>popup</b>.
     *
     * @param popup
     */
    public void setPopup(String popup) {
        this.popup_ = popup;
    }

    /**
     * Gets the String property <b>position</b>.
     *
     * @return String
     */
    public String getPosition() {
        return (position_);
    }

    /**
     * Sets the String property <b>position</b>.
     *
     * @param position
     */
    public void setPosition(String position) {
        this.position_ = position;
    }

    /**
     * Gets the String property <b>ref</b>.
     *
     * @return String
     */
    public String getRef() {
        return (ref_);
    }

    /**
     * Sets the String property <b>ref</b>.
     *
     * @param ref
     */
    public void setRef(String ref) {
        this.ref_ = ref;
    }

    /**
     * Gets the String property <b>removeelement</b>.
     *
     * @return String
     */
    public String getRemoveelement() {
        return (removeelement_);
    }

    /**
     * Sets the String property <b>removeelement</b>.
     *
     * @param removeelement
     */
    public void setRemoveelement(String removeelement) {
        this.removeelement_ = removeelement;
    }

    /**
     * Gets the String property <b>sortDirection</b>.
     *
     * @return String
     */
    public String getSortDirection() {
        return (sortDirection_);
    }

    /**
     * Sets the String property <b>sortDirection</b>.
     *
     * @param sortDirection
     */
    public void setSortDirection(String sortDirection) {
        this.sortDirection_ = sortDirection;
    }

    /**
     * Gets the String property <b>sortResource</b>.
     *
     * @return String
     */
    public String getSortResource() {
        return (sortResource_);
    }

    /**
     * Sets the String property <b>sortResource</b>.
     *
     * @param sortResource
     */
    public void setSortResource(String sortResource) {
        this.sortResource_ = sortResource;
    }

    /**
     * Gets the String property <b>sortResource2</b>.
     *
     * @return String
     */
    public String getSortResource2() {
        return (sortResource2_);
    }

    /**
     * Sets the String property <b>sortResource2</b>.
     *
     * @param sortResource2
     */
    public void setSortResource2(String sortResource2) {
        this.sortResource2_ = sortResource2;
    }

    /**
     * Gets the String property <b>statustext</b>.
     *
     * @return String
     */
    public String getStatustext() {
        return (statustext_);
    }

    /**
     * Sets the String property <b>statustext</b>.
     *
     * @param statustext
     */
    public void setStatustext(String statustext) {
        this.statustext_ = statustext;
    }

    /**
     * Gets the String property <b>style</b>.
     *
     * @return String
     */
    public String getStyle() {
        return (style_);
    }

    /**
     * Sets the String property <b>style</b>.
     *
     * @param style
     */
    public void setStyle(String style) {
        this.style_ = style;
    }

    /**
     * Gets the String property <b>template</b>.
     *
     * @return String
     */
    public String getTemplate() {
        return (template_);
    }

    /**
     * Sets the String property <b>template</b>.
     *
     * @param template
     */
    public void setTemplate(String template) {
        this.template_ = template;
    }

    /**
     * Gets the String property <b>tooltip</b>.
     *
     * @return String
     */
    public String getTooltip() {
        return (tooltip_);
    }

    /**
     * Sets the String property <b>tooltip</b>.
     *
     * @param tooltip
     */
    public void setTooltip(String tooltip) {
        this.tooltip_ = tooltip;
    }

    /**
     * Gets the String property <b>tooltiptext</b>.
     *
     * @return String
     */
    public String getTooltiptext() {
        return (tooltiptext_);
    }

    /**
     * Sets the String property <b>tooltiptext</b>.
     *
     * @param tooltiptext
     */
    public void setTooltiptext(String tooltiptext) {
        this.tooltiptext_ = tooltiptext;
    }

    /**
     * Gets the java.math.BigInteger property <b>top</b>.
     *
     * @return java.math.BigInteger
     */
    public java.math.BigInteger getTop() {
        return (top_);
    }

    /**
     * Sets the java.math.BigInteger property <b>top</b>.
     *
     * @param top
     */
    public void setTop(java.math.BigInteger top) {
        this.top_ = top;
    }

    /**
     * Gets the String property <b>uri</b>.
     *
     * @return String
     */
    public String getUri() {
        return (uri_);
    }

    /**
     * Sets the String property <b>uri</b>.
     *
     * @param uri
     */
    public void setUri(String uri) {
        this.uri_ = uri;
    }

    /**
     * Gets the java.math.BigInteger property <b>width</b>.
     *
     * @return java.math.BigInteger
     */
    public java.math.BigInteger getWidth() {
        return (width_);
    }

    /**
     * Sets the java.math.BigInteger property <b>width</b>.
     *
     * @param width
     */
    public void setWidth(java.math.BigInteger width) {
        this.width_ = width;
    }

    /**
     * Gets the String property <b>onclick</b>.
     *
     * @return String
     */
    public String getOnclick() {
        return (onclick_);
    }

    /**
     * Sets the String property <b>onclick</b>.
     *
     * @param onclick
     */
    public void setOnclick(String onclick) {
        this.onclick_ = onclick;
    }

    /**
     * Gets the String property <b>onmousedown</b>.
     *
     * @return String
     */
    public String getOnmousedown() {
        return (onmousedown_);
    }

    /**
     * Sets the String property <b>onmousedown</b>.
     *
     * @param onmousedown
     */
    public void setOnmousedown(String onmousedown) {
        this.onmousedown_ = onmousedown;
    }

    /**
     * Gets the String property <b>onmouseup</b>.
     *
     * @return String
     */
    public String getOnmouseup() {
        return (onmouseup_);
    }

    /**
     * Sets the String property <b>onmouseup</b>.
     *
     * @param onmouseup
     */
    public void setOnmouseup(String onmouseup) {
        this.onmouseup_ = onmouseup;
    }

    /**
     * Gets the String property <b>onmouseover</b>.
     *
     * @return String
     */
    public String getOnmouseover() {
        return (onmouseover_);
    }

    /**
     * Sets the String property <b>onmouseover</b>.
     *
     * @param onmouseover
     */
    public void setOnmouseover(String onmouseover) {
        this.onmouseover_ = onmouseover;
    }

    /**
     * Gets the String property <b>onmousemove</b>.
     *
     * @return String
     */
    public String getOnmousemove() {
        return (onmousemove_);
    }

    /**
     * Sets the String property <b>onmousemove</b>.
     *
     * @param onmousemove
     */
    public void setOnmousemove(String onmousemove) {
        this.onmousemove_ = onmousemove;
    }

    /**
     * Gets the String property <b>onmouseout</b>.
     *
     * @return String
     */
    public String getOnmouseout() {
        return (onmouseout_);
    }

    /**
     * Sets the String property <b>onmouseout</b>.
     *
     * @param onmouseout
     */
    public void setOnmouseout(String onmouseout) {
        this.onmouseout_ = onmouseout;
    }

    /**
     * Gets the String property <b>oncommand</b>.
     *
     * @return String
     */
    public String getOncommand() {
        return (oncommand_);
    }

    /**
     * Sets the String property <b>oncommand</b>.
     *
     * @param oncommand
     */
    public void setOncommand(String oncommand) {
        this.oncommand_ = oncommand;
    }

    /**
     * Gets the String property <b>onkeypress</b>.
     *
     * @return String
     */
    public String getOnkeypress() {
        return (onkeypress_);
    }

    /**
     * Sets the String property <b>onkeypress</b>.
     *
     * @param onkeypress
     */
    public void setOnkeypress(String onkeypress) {
        this.onkeypress_ = onkeypress;
    }

    /**
     * Gets the String property <b>onkeydown</b>.
     *
     * @return String
     */
    public String getOnkeydown() {
        return (onkeydown_);
    }

    /**
     * Sets the String property <b>onkeydown</b>.
     *
     * @param onkeydown
     */
    public void setOnkeydown(String onkeydown) {
        this.onkeydown_ = onkeydown;
    }

    /**
     * Gets the String property <b>onkeyup</b>.
     *
     * @return String
     */
    public String getOnkeyup() {
        return (onkeyup_);
    }

    /**
     * Sets the String property <b>onkeyup</b>.
     *
     * @param onkeyup
     */
    public void setOnkeyup(String onkeyup) {
        this.onkeyup_ = onkeyup;
    }

    /**
     * Gets the String property <b>onfocus</b>.
     *
     * @return String
     */
    public String getOnfocus() {
        return (onfocus_);
    }

    /**
     * Sets the String property <b>onfocus</b>.
     *
     * @param onfocus
     */
    public void setOnfocus(String onfocus) {
        this.onfocus_ = onfocus;
    }

    /**
     * Gets the String property <b>onblur</b>.
     *
     * @return String
     */
    public String getOnblur() {
        return (onblur_);
    }

    /**
     * Sets the String property <b>onblur</b>.
     *
     * @param onblur
     */
    public void setOnblur(String onblur) {
        this.onblur_ = onblur;
    }

    /**
     * Gets the String property <b>onload</b>.
     *
     * @return String
     */
    public String getOnload() {
        return (onload_);
    }

    /**
     * Sets the String property <b>onload</b>.
     *
     * @param onload
     */
    public void setOnload(String onload) {
        this.onload_ = onload;
    }

    /**
     * Gets the String property <b>onunload</b>.
     *
     * @return String
     */
    public String getOnunload() {
        return (onunload_);
    }

    /**
     * Sets the String property <b>onunload</b>.
     *
     * @param onunload
     */
    public void setOnunload(String onunload) {
        this.onunload_ = onunload;
    }

    /**
     * Gets the boolean property <b>hidechrome</b>.
     *
     * @return boolean
     */
    public boolean getHidechrome() {
        if (hidechrome_ == null) {
            return(false);
        }
        return (hidechrome_.booleanValue());
    }

    /**
     * Gets the boolean property <b>hidechrome</b>.
     *
     * @param hidechrome
     * @return boolean
     */
    public boolean getHidechrome(boolean hidechrome) {
        if (hidechrome_ == null) {
            return(hidechrome);
        }
        return (this.hidechrome_.booleanValue());
    }

    /**
     * Gets the boolean property <b>hidechrome</b>.
     *
     * @return Boolean
     */
    public Boolean getHidechromeAsBoolean() {
        return (hidechrome_);
    }

    /**
     * Check the boolean property <b>hidechrome</b>.
     *
     * @return boolean
     */
    public boolean checkHidechrome() {
        return (hidechrome_ != null);
    }

    /**
     * Sets the boolean property <b>hidechrome</b>.
     *
     * @param hidechrome
     */
    public void setHidechrome(boolean hidechrome) {
        this.hidechrome_ = new Boolean(hidechrome);
    }

    /**
     * Sets the boolean property <b>hidechrome</b>.
     *
     * @param hidechrome
     */
    public void setHidechrome(Boolean hidechrome) {
        this.hidechrome_ = hidechrome;
    }

    /**
     * Gets the java.math.BigInteger property <b>screenX</b>.
     *
     * @return java.math.BigInteger
     */
    public java.math.BigInteger getScreenX() {
        return (screenX_);
    }

    /**
     * Sets the java.math.BigInteger property <b>screenX</b>.
     *
     * @param screenX
     */
    public void setScreenX(java.math.BigInteger screenX) {
        this.screenX_ = screenX;
    }

    /**
     * Gets the java.math.BigInteger property <b>screenY</b>.
     *
     * @return java.math.BigInteger
     */
    public java.math.BigInteger getScreenY() {
        return (screenY_);
    }

    /**
     * Sets the java.math.BigInteger property <b>screenY</b>.
     *
     * @param screenY
     */
    public void setScreenY(java.math.BigInteger screenY) {
        this.screenY_ = screenY;
    }

    /**
     * Gets the String property <b>sizemode</b>.
     *
     * @return String
     */
    public String getSizemode() {
        return (sizemode_);
    }

    /**
     * Sets the String property <b>sizemode</b>.
     *
     * @param sizemode
     */
    public void setSizemode(String sizemode) {
        this.sizemode_ = sizemode;
    }

    /**
     * Gets the String property <b>title</b>.
     *
     * @return String
     */
    public String getTitle() {
        return (title_);
    }

    /**
     * Sets the String property <b>title</b>.
     *
     * @param title
     */
    public void setTitle(String title) {
        this.title_ = title;
    }

    /**
     * Gets the String property <b>windowtype</b>.
     *
     * @return String
     */
    public String getWindowtype() {
        return (windowtype_);
    }

    /**
     * Sets the String property <b>windowtype</b>.
     *
     * @param windowtype
     */
    public void setWindowtype(String windowtype) {
        this.windowtype_ = windowtype;
    }

    /**
     * Gets the IXulWindowChoice property <b>BlockClass</b>.
     *
     * @return IXulWindowChoice[]
     */
    public IXulWindowChoice[] getBlockClass() {
        IXulWindowChoice[] array = new IXulWindowChoice[blockClass_.size()];
        return ((IXulWindowChoice[])blockClass_.toArray(array));
    }

    /**
     * Sets the IXulWindowChoice property <b>BlockClass</b>.
     *
     * @param blockClass
     */
    public void setBlockClass(IXulWindowChoice[] blockClass) {
        this.blockClass_.clear();
        for (int i = 0;i < blockClass.length;i++) {
            addBlockClass(blockClass[i]);
        }
        for (int i = 0;i < blockClass.length;i++) {
            blockClass[i].rSetParentRNode(this);
        }
    }

    /**
     * Sets the IXulWindowChoice property <b>BlockClass</b>.
     *
     * @param blockClass
     */
    public void setBlockClass(IXulWindowChoice blockClass) {
        this.blockClass_.clear();
        addBlockClass(blockClass);
        if (blockClass != null) {
            blockClass.rSetParentRNode(this);
        }
    }

    /**
     * Adds the IXulWindowChoice property <b>BlockClass</b>.
     *
     * @param blockClass
     */
    public void addBlockClass(IXulWindowChoice blockClass) {
        this.blockClass_.add(blockClass);
        if (blockClass != null) {
            blockClass.rSetParentRNode(this);
        }
    }

    /**
     * Adds the IXulWindowChoice property <b>BlockClass</b>.
     *
     * @param blockClass
     */
    public void addBlockClass(IXulWindowChoice[] blockClass) {
        for (int i = 0;i < blockClass.length;i++) {
            addBlockClass(blockClass[i]);
        }
        for (int i = 0;i < blockClass.length;i++) {
            blockClass[i].rSetParentRNode(this);
        }
    }

    /**
     * Gets number of the IXulWindowChoice property <b>BlockClass</b>.
     *
     * @return int
     */
    public int sizeBlockClass() {
        return (blockClass_.size());
    }

    /**
     * Gets the IXulWindowChoice property <b>BlockClass</b> by index.
     *
     * @param index
     * @return IXulWindowChoice
     */
    public IXulWindowChoice getBlockClass(int index) {
        return ((IXulWindowChoice)blockClass_.get(index));
    }

    /**
     * Sets the IXulWindowChoice property <b>BlockClass</b> by index.
     *
     * @param index
     * @param blockClass
     */
    public void setBlockClass(int index, IXulWindowChoice blockClass) {
        this.blockClass_.set(index, blockClass);
        if (blockClass != null) {
            blockClass.rSetParentRNode(this);
        }
    }

    /**
     * Adds the IXulWindowChoice property <b>BlockClass</b> by index.
     *
     * @param index
     * @param blockClass
     */
    public void addBlockClass(int index, IXulWindowChoice blockClass) {
        this.blockClass_.add(index, blockClass);
        if (blockClass != null) {
            blockClass.rSetParentRNode(this);
        }
    }

    /**
     * Remove the IXulWindowChoice property <b>BlockClass</b> by index.
     *
     * @param index
     */
    public void removeBlockClass(int index) {
        this.blockClass_.remove(index);
    }

    /**
     * Remove the IXulWindowChoice property <b>BlockClass</b> by object.
     *
     * @param blockClass
     */
    public void removeBlockClass(IXulWindowChoice blockClass) {
        this.blockClass_.remove(blockClass);
    }

    /**
     * Clear the IXulWindowChoice property <b>BlockClass</b>.
     *
     */
    public void clearBlockClass() {
        this.blockClass_.clear();
    }

    /**
     * Makes an XML text representation.
     *
     * @return String
     */
    public String makeTextDocument() {
        StringBuffer buffer = new StringBuffer();
        makeTextElement(buffer);
        return (new String(buffer));
    }

    /**
     * Makes an XML text representation.
     *
     * @param buffer
     */
    public void makeTextElement(StringBuffer buffer) {
        int size;
        String prefix = rNSContext_.getPrefixByUri("http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul");
        buffer.append("<");
        URelaxer.makeQName(prefix, "window", buffer);
        rNSContext_.makeNSMappings(buffer);
        if (align_ != null) {
            buffer.append(" ");
            rNSContext_.makeQName("", "align", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.escapeAttrQuot(URelaxer.getString(getAlign())));
            buffer.append("\"");
        }
        if (allowevents_ != null) {
            buffer.append(" ");
            rNSContext_.makeQName("", "allowevents", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.getString(getAllowevents()));
            buffer.append("\"");
        }
        if (allownegativeassertions_ != null) {
            buffer.append(" ");
            rNSContext_.makeQName("", "allownegativeassertions", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.getString(getAllownegativeassertions()));
            buffer.append("\"");
        }
        if (classValue_ != null) {
            buffer.append(" ");
            rNSContext_.makeQName("", "class", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.escapeAttrQuot(URelaxer.getString(getClassValue())));
            buffer.append("\"");
        }
        if (coalesceduplicatearcs_ != null) {
            buffer.append(" ");
            rNSContext_.makeQName("", "coalesceduplicatearcs", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.getString(getCoalesceduplicatearcs()));
            buffer.append("\"");
        }
        if (collapsed_ != null) {
            buffer.append(" ");
            rNSContext_.makeQName("", "collapsed", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.getString(getCollapsed()));
            buffer.append("\"");
        }
        if (container_ != null) {
            buffer.append(" ");
            rNSContext_.makeQName("", "container", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.escapeAttrQuot(URelaxer.getString(getContainer())));
            buffer.append("\"");
        }
        if (containment_ != null) {
            buffer.append(" ");
            rNSContext_.makeQName("", "containment", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.escapeAttrQuot(URelaxer.getString(getContainment())));
            buffer.append("\"");
        }
        if (context_ != null) {
            buffer.append(" ");
            rNSContext_.makeQName("", "context", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.escapeAttrQuot(URelaxer.getString(getContext())));
            buffer.append("\"");
        }
        if (contextmenu_ != null) {
            buffer.append(" ");
            rNSContext_.makeQName("", "contextmenu", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.escapeAttrQuot(URelaxer.getString(getContextmenu())));
            buffer.append("\"");
        }
        if (datasources_ != null) {
            buffer.append(" ");
            rNSContext_.makeQName("", "datasources", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.escapeAttrQuot(URelaxer.getString(getDatasources())));
            buffer.append("\"");
        }
        if (debug_ != null) {
            buffer.append(" ");
            rNSContext_.makeQName("", "debug", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.getString(getDebug()));
            buffer.append("\"");
        }
        if (dir_ != null) {
            buffer.append(" ");
            rNSContext_.makeQName("", "dir", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.escapeAttrQuot(URelaxer.getString(getDir())));
            buffer.append("\"");
        }
        if (empty_ != null) {
            buffer.append(" ");
            rNSContext_.makeQName("", "empty", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.getString(getEmpty()));
            buffer.append("\"");
        }
        if (equalsize_ != null) {
            buffer.append(" ");
            rNSContext_.makeQName("", "equalsize", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.escapeAttrQuot(URelaxer.getString(getEqualsize())));
            buffer.append("\"");
        }
        if (flags_ != null) {
            buffer.append(" ");
            rNSContext_.makeQName("", "flags", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.escapeAttrQuot(URelaxer.getString(getFlags())));
            buffer.append("\"");
        }
        if (flex_ != null) {
            buffer.append(" ");
            rNSContext_.makeQName("", "flex", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.getString(getFlex()));
            buffer.append("\"");
        }
        if (height_ != null) {
            buffer.append(" ");
            rNSContext_.makeQName("", "height", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.getString(getHeight()));
            buffer.append("\"");
        }
        if (hidden_ != null) {
            buffer.append(" ");
            rNSContext_.makeQName("", "hidden", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.getString(getHidden()));
            buffer.append("\"");
        }
        if (id_ != null) {
            buffer.append(" ");
            rNSContext_.makeQName("", "id", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.escapeAttrQuot(URelaxer.getString(getId())));
            buffer.append("\"");
        }
        if (insertafter_ != null) {
            buffer.append(" ");
            rNSContext_.makeQName("", "insertafter", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.escapeAttrQuot(URelaxer.getString(getInsertafter())));
            buffer.append("\"");
        }
        if (insertbefore_ != null) {
            buffer.append(" ");
            rNSContext_.makeQName("", "insertbefore", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.escapeAttrQuot(URelaxer.getString(getInsertbefore())));
            buffer.append("\"");
        }
        if (left_ != null) {
            buffer.append(" ");
            rNSContext_.makeQName("", "left", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.getString(getLeft()));
            buffer.append("\"");
        }
        if (maxheight_ != null) {
            buffer.append(" ");
            rNSContext_.makeQName("", "maxheight", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.getString(getMaxheight()));
            buffer.append("\"");
        }
        if (maxwidth_ != null) {
            buffer.append(" ");
            rNSContext_.makeQName("", "maxwidth", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.getString(getMaxwidth()));
            buffer.append("\"");
        }
        if (menu_ != null) {
            buffer.append(" ");
            rNSContext_.makeQName("", "menu", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.escapeAttrQuot(URelaxer.getString(getMenu())));
            buffer.append("\"");
        }
        if (minheight_ != null) {
            buffer.append(" ");
            rNSContext_.makeQName("", "minheight", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.getString(getMinheight()));
            buffer.append("\"");
        }
        if (minwidth_ != null) {
            buffer.append(" ");
            rNSContext_.makeQName("", "minwidth", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.getString(getMinwidth()));
            buffer.append("\"");
        }
        if (observes_ != null) {
            buffer.append(" ");
            rNSContext_.makeQName("", "observes", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.escapeAttrQuot(URelaxer.getString(getObserves())));
            buffer.append("\"");
        }
        if (ordinal_ != null) {
            buffer.append(" ");
            rNSContext_.makeQName("", "ordinal", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.getString(getOrdinal()));
            buffer.append("\"");
        }
        if (orient_ != null) {
            buffer.append(" ");
            rNSContext_.makeQName("", "orient", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.escapeAttrQuot(URelaxer.getString(getOrient())));
            buffer.append("\"");
        }
        if (pack_ != null) {
            buffer.append(" ");
            rNSContext_.makeQName("", "pack", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.escapeAttrQuot(URelaxer.getString(getPack())));
            buffer.append("\"");
        }
        if (persist_ != null) {
            buffer.append(" ");
            rNSContext_.makeQName("", "persist", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.escapeAttrQuot(URelaxer.getString(getPersist())));
            buffer.append("\"");
        }
        if (popup_ != null) {
            buffer.append(" ");
            rNSContext_.makeQName("", "popup", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.escapeAttrQuot(URelaxer.getString(getPopup())));
            buffer.append("\"");
        }
        if (position_ != null) {
            buffer.append(" ");
            rNSContext_.makeQName("", "position", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.escapeAttrQuot(URelaxer.getString(getPosition())));
            buffer.append("\"");
        }
        if (ref_ != null) {
            buffer.append(" ");
            rNSContext_.makeQName("", "ref", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.escapeAttrQuot(URelaxer.getString(getRef())));
            buffer.append("\"");
        }
        if (removeelement_ != null) {
            buffer.append(" ");
            rNSContext_.makeQName("", "removeelement", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.escapeAttrQuot(URelaxer.getString(getRemoveelement())));
            buffer.append("\"");
        }
        if (sortDirection_ != null) {
            buffer.append(" ");
            rNSContext_.makeQName("", "sortDirection", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.escapeAttrQuot(URelaxer.getString(getSortDirection())));
            buffer.append("\"");
        }
        if (sortResource_ != null) {
            buffer.append(" ");
            rNSContext_.makeQName("", "sortResource", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.escapeAttrQuot(URelaxer.getString(getSortResource())));
            buffer.append("\"");
        }
        if (sortResource2_ != null) {
            buffer.append(" ");
            rNSContext_.makeQName("", "sortResource2", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.escapeAttrQuot(URelaxer.getString(getSortResource2())));
            buffer.append("\"");
        }
        if (statustext_ != null) {
            buffer.append(" ");
            rNSContext_.makeQName("", "statustext", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.escapeAttrQuot(URelaxer.getString(getStatustext())));
            buffer.append("\"");
        }
        if (style_ != null) {
            buffer.append(" ");
            rNSContext_.makeQName("", "style", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.escapeAttrQuot(URelaxer.getString(getStyle())));
            buffer.append("\"");
        }
        if (template_ != null) {
            buffer.append(" ");
            rNSContext_.makeQName("", "template", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.escapeAttrQuot(URelaxer.getString(getTemplate())));
            buffer.append("\"");
        }
        if (tooltip_ != null) {
            buffer.append(" ");
            rNSContext_.makeQName("", "tooltip", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.escapeAttrQuot(URelaxer.getString(getTooltip())));
            buffer.append("\"");
        }
        if (tooltiptext_ != null) {
            buffer.append(" ");
            rNSContext_.makeQName("", "tooltiptext", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.escapeAttrQuot(URelaxer.getString(getTooltiptext())));
            buffer.append("\"");
        }
        if (top_ != null) {
            buffer.append(" ");
            rNSContext_.makeQName("", "top", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.getString(getTop()));
            buffer.append("\"");
        }
        if (uri_ != null) {
            buffer.append(" ");
            rNSContext_.makeQName("", "uri", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.escapeAttrQuot(URelaxer.getString(getUri())));
            buffer.append("\"");
        }
        if (width_ != null) {
            buffer.append(" ");
            rNSContext_.makeQName("", "width", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.getString(getWidth()));
            buffer.append("\"");
        }
        if (onclick_ != null) {
            buffer.append(" ");
            rNSContext_.makeQName("", "onclick", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.escapeAttrQuot(URelaxer.getString(getOnclick())));
            buffer.append("\"");
        }
        if (onmousedown_ != null) {
            buffer.append(" ");
            rNSContext_.makeQName("", "onmousedown", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.escapeAttrQuot(URelaxer.getString(getOnmousedown())));
            buffer.append("\"");
        }
        if (onmouseup_ != null) {
            buffer.append(" ");
            rNSContext_.makeQName("", "onmouseup", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.escapeAttrQuot(URelaxer.getString(getOnmouseup())));
            buffer.append("\"");
        }
        if (onmouseover_ != null) {
            buffer.append(" ");
            rNSContext_.makeQName("", "onmouseover", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.escapeAttrQuot(URelaxer.getString(getOnmouseover())));
            buffer.append("\"");
        }
        if (onmousemove_ != null) {
            buffer.append(" ");
            rNSContext_.makeQName("", "onmousemove", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.escapeAttrQuot(URelaxer.getString(getOnmousemove())));
            buffer.append("\"");
        }
        if (onmouseout_ != null) {
            buffer.append(" ");
            rNSContext_.makeQName("", "onmouseout", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.escapeAttrQuot(URelaxer.getString(getOnmouseout())));
            buffer.append("\"");
        }
        if (oncommand_ != null) {
            buffer.append(" ");
            rNSContext_.makeQName("", "oncommand", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.escapeAttrQuot(URelaxer.getString(getOncommand())));
            buffer.append("\"");
        }
        if (onkeypress_ != null) {
            buffer.append(" ");
            rNSContext_.makeQName("", "onkeypress", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.escapeAttrQuot(URelaxer.getString(getOnkeypress())));
            buffer.append("\"");
        }
        if (onkeydown_ != null) {
            buffer.append(" ");
            rNSContext_.makeQName("", "onkeydown", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.escapeAttrQuot(URelaxer.getString(getOnkeydown())));
            buffer.append("\"");
        }
        if (onkeyup_ != null) {
            buffer.append(" ");
            rNSContext_.makeQName("", "onkeyup", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.escapeAttrQuot(URelaxer.getString(getOnkeyup())));
            buffer.append("\"");
        }
        if (onfocus_ != null) {
            buffer.append(" ");
            rNSContext_.makeQName("", "onfocus", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.escapeAttrQuot(URelaxer.getString(getOnfocus())));
            buffer.append("\"");
        }
        if (onblur_ != null) {
            buffer.append(" ");
            rNSContext_.makeQName("", "onblur", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.escapeAttrQuot(URelaxer.getString(getOnblur())));
            buffer.append("\"");
        }
        if (onload_ != null) {
            buffer.append(" ");
            rNSContext_.makeQName("", "onload", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.escapeAttrQuot(URelaxer.getString(getOnload())));
            buffer.append("\"");
        }
        if (onunload_ != null) {
            buffer.append(" ");
            rNSContext_.makeQName("", "onunload", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.escapeAttrQuot(URelaxer.getString(getOnunload())));
            buffer.append("\"");
        }
        if (hidechrome_ != null) {
            buffer.append(" ");
            rNSContext_.makeQName("", "hidechrome", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.getString(getHidechrome()));
            buffer.append("\"");
        }
        if (screenX_ != null) {
            buffer.append(" ");
            rNSContext_.makeQName("", "screenX", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.getString(getScreenX()));
            buffer.append("\"");
        }
        if (screenY_ != null) {
            buffer.append(" ");
            rNSContext_.makeQName("", "screenY", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.getString(getScreenY()));
            buffer.append("\"");
        }
        if (sizemode_ != null) {
            buffer.append(" ");
            rNSContext_.makeQName("", "sizemode", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.escapeAttrQuot(URelaxer.getString(getSizemode())));
            buffer.append("\"");
        }
        if (title_ != null) {
            buffer.append(" ");
            rNSContext_.makeQName("", "title", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.escapeAttrQuot(URelaxer.getString(getTitle())));
            buffer.append("\"");
        }
        if (windowtype_ != null) {
            buffer.append(" ");
            rNSContext_.makeQName("", "windowtype", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.escapeAttrQuot(URelaxer.getString(getWindowtype())));
            buffer.append("\"");
        }
        size = this.blockClass_.size();
        for (int i = 0;i < size;i++) {
            IXulWindowChoice value = (IXulWindowChoice)this.blockClass_.get(i);
            value.makeTextAttribute(buffer);
        }
        buffer.append(">");
        size = this.blockClass_.size();
        for (int i = 0;i < size;i++) {
            IXulWindowChoice value = (IXulWindowChoice)this.blockClass_.get(i);
            value.makeTextElement(buffer);
        }
        buffer.append("</");
        URelaxer.makeQName(prefix, "window", buffer);
        buffer.append(">");
    }

    /**
     * Makes an XML text representation.
     *
     * @param buffer
     * @exception IOException
     */
    public void makeTextElement(Writer buffer) throws IOException {
        int size;
        String prefix = rNSContext_.getPrefixByUri("http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul");
        buffer.write("<");
        URelaxer.makeQName(prefix, "window", buffer);
        rNSContext_.makeNSMappings(buffer);
        if (align_ != null) {
            buffer.write(" ");
            rNSContext_.makeQName("", "align", buffer);
            buffer.write("=\"");
            buffer.write(URelaxer.escapeAttrQuot(URelaxer.getString(getAlign())));
            buffer.write("\"");
        }
        if (allowevents_ != null) {
            buffer.write(" ");
            rNSContext_.makeQName("", "allowevents", buffer);
            buffer.write("=\"");
            buffer.write(URelaxer.getString(getAllowevents()));
            buffer.write("\"");
        }
        if (allownegativeassertions_ != null) {
            buffer.write(" ");
            rNSContext_.makeQName("", "allownegativeassertions", buffer);
            buffer.write("=\"");
            buffer.write(URelaxer.getString(getAllownegativeassertions()));
            buffer.write("\"");
        }
        if (classValue_ != null) {
            buffer.write(" ");
            rNSContext_.makeQName("", "class", buffer);
            buffer.write("=\"");
            buffer.write(URelaxer.escapeAttrQuot(URelaxer.getString(getClassValue())));
            buffer.write("\"");
        }
        if (coalesceduplicatearcs_ != null) {
            buffer.write(" ");
            rNSContext_.makeQName("", "coalesceduplicatearcs", buffer);
            buffer.write("=\"");
            buffer.write(URelaxer.getString(getCoalesceduplicatearcs()));
            buffer.write("\"");
        }
        if (collapsed_ != null) {
            buffer.write(" ");
            rNSContext_.makeQName("", "collapsed", buffer);
            buffer.write("=\"");
            buffer.write(URelaxer.getString(getCollapsed()));
            buffer.write("\"");
        }
        if (container_ != null) {
            buffer.write(" ");
            rNSContext_.makeQName("", "container", buffer);
            buffer.write("=\"");
            buffer.write(URelaxer.escapeAttrQuot(URelaxer.getString(getContainer())));
            buffer.write("\"");
        }
        if (containment_ != null) {
            buffer.write(" ");
            rNSContext_.makeQName("", "containment", buffer);
            buffer.write("=\"");
            buffer.write(URelaxer.escapeAttrQuot(URelaxer.getString(getContainment())));
            buffer.write("\"");
        }
        if (context_ != null) {
            buffer.write(" ");
            rNSContext_.makeQName("", "context", buffer);
            buffer.write("=\"");
            buffer.write(URelaxer.escapeAttrQuot(URelaxer.getString(getContext())));
            buffer.write("\"");
        }
        if (contextmenu_ != null) {
            buffer.write(" ");
            rNSContext_.makeQName("", "contextmenu", buffer);
            buffer.write("=\"");
            buffer.write(URelaxer.escapeAttrQuot(URelaxer.getString(getContextmenu())));
            buffer.write("\"");
        }
        if (datasources_ != null) {
            buffer.write(" ");
            rNSContext_.makeQName("", "datasources", buffer);
            buffer.write("=\"");
            buffer.write(URelaxer.escapeAttrQuot(URelaxer.getString(getDatasources())));
            buffer.write("\"");
        }
        if (debug_ != null) {
            buffer.write(" ");
            rNSContext_.makeQName("", "debug", buffer);
            buffer.write("=\"");
            buffer.write(URelaxer.getString(getDebug()));
            buffer.write("\"");
        }
        if (dir_ != null) {
            buffer.write(" ");
            rNSContext_.makeQName("", "dir", buffer);
            buffer.write("=\"");
            buffer.write(URelaxer.escapeAttrQuot(URelaxer.getString(getDir())));
            buffer.write("\"");
        }
        if (empty_ != null) {
            buffer.write(" ");
            rNSContext_.makeQName("", "empty", buffer);
            buffer.write("=\"");
            buffer.write(URelaxer.getString(getEmpty()));
            buffer.write("\"");
        }
        if (equalsize_ != null) {
            buffer.write(" ");
            rNSContext_.makeQName("", "equalsize", buffer);
            buffer.write("=\"");
            buffer.write(URelaxer.escapeAttrQuot(URelaxer.getString(getEqualsize())));
            buffer.write("\"");
        }
        if (flags_ != null) {
            buffer.write(" ");
            rNSContext_.makeQName("", "flags", buffer);
            buffer.write("=\"");
            buffer.write(URelaxer.escapeAttrQuot(URelaxer.getString(getFlags())));
            buffer.write("\"");
        }
        if (flex_ != null) {
            buffer.write(" ");
            rNSContext_.makeQName("", "flex", buffer);
            buffer.write("=\"");
            buffer.write(URelaxer.getString(getFlex()));
            buffer.write("\"");
        }
        if (height_ != null) {
            buffer.write(" ");
            rNSContext_.makeQName("", "height", buffer);
            buffer.write("=\"");
            buffer.write(URelaxer.getString(getHeight()));
            buffer.write("\"");
        }
        if (hidden_ != null) {
            buffer.write(" ");
            rNSContext_.makeQName("", "hidden", buffer);
            buffer.write("=\"");
            buffer.write(URelaxer.getString(getHidden()));
            buffer.write("\"");
        }
        if (id_ != null) {
            buffer.write(" ");
            rNSContext_.makeQName("", "id", buffer);
            buffer.write("=\"");
            buffer.write(URelaxer.escapeAttrQuot(URelaxer.getString(getId())));
            buffer.write("\"");
        }
        if (insertafter_ != null) {
            buffer.write(" ");
            rNSContext_.makeQName("", "insertafter", buffer);
            buffer.write("=\"");
            buffer.write(URelaxer.escapeAttrQuot(URelaxer.getString(getInsertafter())));
            buffer.write("\"");
        }
        if (insertbefore_ != null) {
            buffer.write(" ");
            rNSContext_.makeQName("", "insertbefore", buffer);
            buffer.write("=\"");
            buffer.write(URelaxer.escapeAttrQuot(URelaxer.getString(getInsertbefore())));
            buffer.write("\"");
        }
        if (left_ != null) {
            buffer.write(" ");
            rNSContext_.makeQName("", "left", buffer);
            buffer.write("=\"");
            buffer.write(URelaxer.getString(getLeft()));
            buffer.write("\"");
        }
        if (maxheight_ != null) {
            buffer.write(" ");
            rNSContext_.makeQName("", "maxheight", buffer);
            buffer.write("=\"");
            buffer.write(URelaxer.getString(getMaxheight()));
            buffer.write("\"");
        }
        if (maxwidth_ != null) {
            buffer.write(" ");
            rNSContext_.makeQName("", "maxwidth", buffer);
            buffer.write("=\"");
            buffer.write(URelaxer.getString(getMaxwidth()));
            buffer.write("\"");
        }
        if (menu_ != null) {
            buffer.write(" ");
            rNSContext_.makeQName("", "menu", buffer);
            buffer.write("=\"");
            buffer.write(URelaxer.escapeAttrQuot(URelaxer.getString(getMenu())));
            buffer.write("\"");
        }
        if (minheight_ != null) {
            buffer.write(" ");
            rNSContext_.makeQName("", "minheight", buffer);
            buffer.write("=\"");
            buffer.write(URelaxer.getString(getMinheight()));
            buffer.write("\"");
        }
        if (minwidth_ != null) {
            buffer.write(" ");
            rNSContext_.makeQName("", "minwidth", buffer);
            buffer.write("=\"");
            buffer.write(URelaxer.getString(getMinwidth()));
            buffer.write("\"");
        }
        if (observes_ != null) {
            buffer.write(" ");
            rNSContext_.makeQName("", "observes", buffer);
            buffer.write("=\"");
            buffer.write(URelaxer.escapeAttrQuot(URelaxer.getString(getObserves())));
            buffer.write("\"");
        }
        if (ordinal_ != null) {
            buffer.write(" ");
            rNSContext_.makeQName("", "ordinal", buffer);
            buffer.write("=\"");
            buffer.write(URelaxer.getString(getOrdinal()));
            buffer.write("\"");
        }
        if (orient_ != null) {
            buffer.write(" ");
            rNSContext_.makeQName("", "orient", buffer);
            buffer.write("=\"");
            buffer.write(URelaxer.escapeAttrQuot(URelaxer.getString(getOrient())));
            buffer.write("\"");
        }
        if (pack_ != null) {
            buffer.write(" ");
            rNSContext_.makeQName("", "pack", buffer);
            buffer.write("=\"");
            buffer.write(URelaxer.escapeAttrQuot(URelaxer.getString(getPack())));
            buffer.write("\"");
        }
        if (persist_ != null) {
            buffer.write(" ");
            rNSContext_.makeQName("", "persist", buffer);
            buffer.write("=\"");
            buffer.write(URelaxer.escapeAttrQuot(URelaxer.getString(getPersist())));
            buffer.write("\"");
        }
        if (popup_ != null) {
            buffer.write(" ");
            rNSContext_.makeQName("", "popup", buffer);
            buffer.write("=\"");
            buffer.write(URelaxer.escapeAttrQuot(URelaxer.getString(getPopup())));
            buffer.write("\"");
        }
        if (position_ != null) {
            buffer.write(" ");
            rNSContext_.makeQName("", "position", buffer);
            buffer.write("=\"");
            buffer.write(URelaxer.escapeAttrQuot(URelaxer.getString(getPosition())));
            buffer.write("\"");
        }
        if (ref_ != null) {
            buffer.write(" ");
            rNSContext_.makeQName("", "ref", buffer);
            buffer.write("=\"");
            buffer.write(URelaxer.escapeAttrQuot(URelaxer.getString(getRef())));
            buffer.write("\"");
        }
        if (removeelement_ != null) {
            buffer.write(" ");
            rNSContext_.makeQName("", "removeelement", buffer);
            buffer.write("=\"");
            buffer.write(URelaxer.escapeAttrQuot(URelaxer.getString(getRemoveelement())));
            buffer.write("\"");
        }
        if (sortDirection_ != null) {
            buffer.write(" ");
            rNSContext_.makeQName("", "sortDirection", buffer);
            buffer.write("=\"");
            buffer.write(URelaxer.escapeAttrQuot(URelaxer.getString(getSortDirection())));
            buffer.write("\"");
        }
        if (sortResource_ != null) {
            buffer.write(" ");
            rNSContext_.makeQName("", "sortResource", buffer);
            buffer.write("=\"");
            buffer.write(URelaxer.escapeAttrQuot(URelaxer.getString(getSortResource())));
            buffer.write("\"");
        }
        if (sortResource2_ != null) {
            buffer.write(" ");
            rNSContext_.makeQName("", "sortResource2", buffer);
            buffer.write("=\"");
            buffer.write(URelaxer.escapeAttrQuot(URelaxer.getString(getSortResource2())));
            buffer.write("\"");
        }
        if (statustext_ != null) {
            buffer.write(" ");
            rNSContext_.makeQName("", "statustext", buffer);
            buffer.write("=\"");
            buffer.write(URelaxer.escapeAttrQuot(URelaxer.getString(getStatustext())));
            buffer.write("\"");
        }
        if (style_ != null) {
            buffer.write(" ");
            rNSContext_.makeQName("", "style", buffer);
            buffer.write("=\"");
            buffer.write(URelaxer.escapeAttrQuot(URelaxer.getString(getStyle())));
            buffer.write("\"");
        }
        if (template_ != null) {
            buffer.write(" ");
            rNSContext_.makeQName("", "template", buffer);
            buffer.write("=\"");
            buffer.write(URelaxer.escapeAttrQuot(URelaxer.getString(getTemplate())));
            buffer.write("\"");
        }
        if (tooltip_ != null) {
            buffer.write(" ");
            rNSContext_.makeQName("", "tooltip", buffer);
            buffer.write("=\"");
            buffer.write(URelaxer.escapeAttrQuot(URelaxer.getString(getTooltip())));
            buffer.write("\"");
        }
        if (tooltiptext_ != null) {
            buffer.write(" ");
            rNSContext_.makeQName("", "tooltiptext", buffer);
            buffer.write("=\"");
            buffer.write(URelaxer.escapeAttrQuot(URelaxer.getString(getTooltiptext())));
            buffer.write("\"");
        }
        if (top_ != null) {
            buffer.write(" ");
            rNSContext_.makeQName("", "top", buffer);
            buffer.write("=\"");
            buffer.write(URelaxer.getString(getTop()));
            buffer.write("\"");
        }
        if (uri_ != null) {
            buffer.write(" ");
            rNSContext_.makeQName("", "uri", buffer);
            buffer.write("=\"");
            buffer.write(URelaxer.escapeAttrQuot(URelaxer.getString(getUri())));
            buffer.write("\"");
        }
        if (width_ != null) {
            buffer.write(" ");
            rNSContext_.makeQName("", "width", buffer);
            buffer.write("=\"");
            buffer.write(URelaxer.getString(getWidth()));
            buffer.write("\"");
        }
        if (onclick_ != null) {
            buffer.write(" ");
            rNSContext_.makeQName("", "onclick", buffer);
            buffer.write("=\"");
            buffer.write(URelaxer.escapeAttrQuot(URelaxer.getString(getOnclick())));
            buffer.write("\"");
        }
        if (onmousedown_ != null) {
            buffer.write(" ");
            rNSContext_.makeQName("", "onmousedown", buffer);
            buffer.write("=\"");
            buffer.write(URelaxer.escapeAttrQuot(URelaxer.getString(getOnmousedown())));
            buffer.write("\"");
        }
        if (onmouseup_ != null) {
            buffer.write(" ");
            rNSContext_.makeQName("", "onmouseup", buffer);
            buffer.write("=\"");
            buffer.write(URelaxer.escapeAttrQuot(URelaxer.getString(getOnmouseup())));
            buffer.write("\"");
        }
        if (onmouseover_ != null) {
            buffer.write(" ");
            rNSContext_.makeQName("", "onmouseover", buffer);
            buffer.write("=\"");
            buffer.write(URelaxer.escapeAttrQuot(URelaxer.getString(getOnmouseover())));
            buffer.write("\"");
        }
        if (onmousemove_ != null) {
            buffer.write(" ");
            rNSContext_.makeQName("", "onmousemove", buffer);
            buffer.write("=\"");
            buffer.write(URelaxer.escapeAttrQuot(URelaxer.getString(getOnmousemove())));
            buffer.write("\"");
        }
        if (onmouseout_ != null) {
            buffer.write(" ");
            rNSContext_.makeQName("", "onmouseout", buffer);
            buffer.write("=\"");
            buffer.write(URelaxer.escapeAttrQuot(URelaxer.getString(getOnmouseout())));
            buffer.write("\"");
        }
        if (oncommand_ != null) {
            buffer.write(" ");
            rNSContext_.makeQName("", "oncommand", buffer);
            buffer.write("=\"");
            buffer.write(URelaxer.escapeAttrQuot(URelaxer.getString(getOncommand())));
            buffer.write("\"");
        }
        if (onkeypress_ != null) {
            buffer.write(" ");
            rNSContext_.makeQName("", "onkeypress", buffer);
            buffer.write("=\"");
            buffer.write(URelaxer.escapeAttrQuot(URelaxer.getString(getOnkeypress())));
            buffer.write("\"");
        }
        if (onkeydown_ != null) {
            buffer.write(" ");
            rNSContext_.makeQName("", "onkeydown", buffer);
            buffer.write("=\"");
            buffer.write(URelaxer.escapeAttrQuot(URelaxer.getString(getOnkeydown())));
            buffer.write("\"");
        }
        if (onkeyup_ != null) {
            buffer.write(" ");
            rNSContext_.makeQName("", "onkeyup", buffer);
            buffer.write("=\"");
            buffer.write(URelaxer.escapeAttrQuot(URelaxer.getString(getOnkeyup())));
            buffer.write("\"");
        }
        if (onfocus_ != null) {
            buffer.write(" ");
            rNSContext_.makeQName("", "onfocus", buffer);
            buffer.write("=\"");
            buffer.write(URelaxer.escapeAttrQuot(URelaxer.getString(getOnfocus())));
            buffer.write("\"");
        }
        if (onblur_ != null) {
            buffer.write(" ");
            rNSContext_.makeQName("", "onblur", buffer);
            buffer.write("=\"");
            buffer.write(URelaxer.escapeAttrQuot(URelaxer.getString(getOnblur())));
            buffer.write("\"");
        }
        if (onload_ != null) {
            buffer.write(" ");
            rNSContext_.makeQName("", "onload", buffer);
            buffer.write("=\"");
            buffer.write(URelaxer.escapeAttrQuot(URelaxer.getString(getOnload())));
            buffer.write("\"");
        }
        if (onunload_ != null) {
            buffer.write(" ");
            rNSContext_.makeQName("", "onunload", buffer);
            buffer.write("=\"");
            buffer.write(URelaxer.escapeAttrQuot(URelaxer.getString(getOnunload())));
            buffer.write("\"");
        }
        if (hidechrome_ != null) {
            buffer.write(" ");
            rNSContext_.makeQName("", "hidechrome", buffer);
            buffer.write("=\"");
            buffer.write(URelaxer.getString(getHidechrome()));
            buffer.write("\"");
        }
        if (screenX_ != null) {
            buffer.write(" ");
            rNSContext_.makeQName("", "screenX", buffer);
            buffer.write("=\"");
            buffer.write(URelaxer.getString(getScreenX()));
            buffer.write("\"");
        }
        if (screenY_ != null) {
            buffer.write(" ");
            rNSContext_.makeQName("", "screenY", buffer);
            buffer.write("=\"");
            buffer.write(URelaxer.getString(getScreenY()));
            buffer.write("\"");
        }
        if (sizemode_ != null) {
            buffer.write(" ");
            rNSContext_.makeQName("", "sizemode", buffer);
            buffer.write("=\"");
            buffer.write(URelaxer.escapeAttrQuot(URelaxer.getString(getSizemode())));
            buffer.write("\"");
        }
        if (title_ != null) {
            buffer.write(" ");
            rNSContext_.makeQName("", "title", buffer);
            buffer.write("=\"");
            buffer.write(URelaxer.escapeAttrQuot(URelaxer.getString(getTitle())));
            buffer.write("\"");
        }
        if (windowtype_ != null) {
            buffer.write(" ");
            rNSContext_.makeQName("", "windowtype", buffer);
            buffer.write("=\"");
            buffer.write(URelaxer.escapeAttrQuot(URelaxer.getString(getWindowtype())));
            buffer.write("\"");
        }
        size = this.blockClass_.size();
        for (int i = 0;i < size;i++) {
            IXulWindowChoice value = (IXulWindowChoice)this.blockClass_.get(i);
            value.makeTextAttribute(buffer);
        }
        buffer.write(">");
        size = this.blockClass_.size();
        for (int i = 0;i < size;i++) {
            IXulWindowChoice value = (IXulWindowChoice)this.blockClass_.get(i);
            value.makeTextElement(buffer);
        }
        buffer.write("</");
        URelaxer.makeQName(prefix, "window", buffer);
        buffer.write(">");
    }

    /**
     * Makes an XML text representation.
     *
     * @param buffer
     */
    public void makeTextElement(PrintWriter buffer) {
        int size;
        String prefix = rNSContext_.getPrefixByUri("http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul");
        buffer.print("<");
        URelaxer.makeQName(prefix, "window", buffer);
        rNSContext_.makeNSMappings(buffer);
        if (align_ != null) {
            buffer.print(" ");
            rNSContext_.makeQName("", "align", buffer);
            buffer.print("=\"");
            buffer.print(URelaxer.escapeAttrQuot(URelaxer.getString(getAlign())));
            buffer.print("\"");
        }
        if (allowevents_ != null) {
            buffer.print(" ");
            rNSContext_.makeQName("", "allowevents", buffer);
            buffer.print("=\"");
            buffer.print(URelaxer.getString(getAllowevents()));
            buffer.print("\"");
        }
        if (allownegativeassertions_ != null) {
            buffer.print(" ");
            rNSContext_.makeQName("", "allownegativeassertions", buffer);
            buffer.print("=\"");
            buffer.print(URelaxer.getString(getAllownegativeassertions()));
            buffer.print("\"");
        }
        if (classValue_ != null) {
            buffer.print(" ");
            rNSContext_.makeQName("", "class", buffer);
            buffer.print("=\"");
            buffer.print(URelaxer.escapeAttrQuot(URelaxer.getString(getClassValue())));
            buffer.print("\"");
        }
        if (coalesceduplicatearcs_ != null) {
            buffer.print(" ");
            rNSContext_.makeQName("", "coalesceduplicatearcs", buffer);
            buffer.print("=\"");
            buffer.print(URelaxer.getString(getCoalesceduplicatearcs()));
            buffer.print("\"");
        }
        if (collapsed_ != null) {
            buffer.print(" ");
            rNSContext_.makeQName("", "collapsed", buffer);
            buffer.print("=\"");
            buffer.print(URelaxer.getString(getCollapsed()));
            buffer.print("\"");
        }
        if (container_ != null) {
            buffer.print(" ");
            rNSContext_.makeQName("", "container", buffer);
            buffer.print("=\"");
            buffer.print(URelaxer.escapeAttrQuot(URelaxer.getString(getContainer())));
            buffer.print("\"");
        }
        if (containment_ != null) {
            buffer.print(" ");
            rNSContext_.makeQName("", "containment", buffer);
            buffer.print("=\"");
            buffer.print(URelaxer.escapeAttrQuot(URelaxer.getString(getContainment())));
            buffer.print("\"");
        }
        if (context_ != null) {
            buffer.print(" ");
            rNSContext_.makeQName("", "context", buffer);
            buffer.print("=\"");
            buffer.print(URelaxer.escapeAttrQuot(URelaxer.getString(getContext())));
            buffer.print("\"");
        }
        if (contextmenu_ != null) {
            buffer.print(" ");
            rNSContext_.makeQName("", "contextmenu", buffer);
            buffer.print("=\"");
            buffer.print(URelaxer.escapeAttrQuot(URelaxer.getString(getContextmenu())));
            buffer.print("\"");
        }
        if (datasources_ != null) {
            buffer.print(" ");
            rNSContext_.makeQName("", "datasources", buffer);
            buffer.print("=\"");
            buffer.print(URelaxer.escapeAttrQuot(URelaxer.getString(getDatasources())));
            buffer.print("\"");
        }
        if (debug_ != null) {
            buffer.print(" ");
            rNSContext_.makeQName("", "debug", buffer);
            buffer.print("=\"");
            buffer.print(URelaxer.getString(getDebug()));
            buffer.print("\"");
        }
        if (dir_ != null) {
            buffer.print(" ");
            rNSContext_.makeQName("", "dir", buffer);
            buffer.print("=\"");
            buffer.print(URelaxer.escapeAttrQuot(URelaxer.getString(getDir())));
            buffer.print("\"");
        }
        if (empty_ != null) {
            buffer.print(" ");
            rNSContext_.makeQName("", "empty", buffer);
            buffer.print("=\"");
            buffer.print(URelaxer.getString(getEmpty()));
            buffer.print("\"");
        }
        if (equalsize_ != null) {
            buffer.print(" ");
            rNSContext_.makeQName("", "equalsize", buffer);
            buffer.print("=\"");
            buffer.print(URelaxer.escapeAttrQuot(URelaxer.getString(getEqualsize())));
            buffer.print("\"");
        }
        if (flags_ != null) {
            buffer.print(" ");
            rNSContext_.makeQName("", "flags", buffer);
            buffer.print("=\"");
            buffer.print(URelaxer.escapeAttrQuot(URelaxer.getString(getFlags())));
            buffer.print("\"");
        }
        if (flex_ != null) {
            buffer.print(" ");
            rNSContext_.makeQName("", "flex", buffer);
            buffer.print("=\"");
            buffer.print(URelaxer.getString(getFlex()));
            buffer.print("\"");
        }
        if (height_ != null) {
            buffer.print(" ");
            rNSContext_.makeQName("", "height", buffer);
            buffer.print("=\"");
            buffer.print(URelaxer.getString(getHeight()));
            buffer.print("\"");
        }
        if (hidden_ != null) {
            buffer.print(" ");
            rNSContext_.makeQName("", "hidden", buffer);
            buffer.print("=\"");
            buffer.print(URelaxer.getString(getHidden()));
            buffer.print("\"");
        }
        if (id_ != null) {
            buffer.print(" ");
            rNSContext_.makeQName("", "id", buffer);
            buffer.print("=\"");
            buffer.print(URelaxer.escapeAttrQuot(URelaxer.getString(getId())));
            buffer.print("\"");
        }
        if (insertafter_ != null) {
            buffer.print(" ");
            rNSContext_.makeQName("", "insertafter", buffer);
            buffer.print("=\"");
            buffer.print(URelaxer.escapeAttrQuot(URelaxer.getString(getInsertafter())));
            buffer.print("\"");
        }
        if (insertbefore_ != null) {
            buffer.print(" ");
            rNSContext_.makeQName("", "insertbefore", buffer);
            buffer.print("=\"");
            buffer.print(URelaxer.escapeAttrQuot(URelaxer.getString(getInsertbefore())));
            buffer.print("\"");
        }
        if (left_ != null) {
            buffer.print(" ");
            rNSContext_.makeQName("", "left", buffer);
            buffer.print("=\"");
            buffer.print(URelaxer.getString(getLeft()));
            buffer.print("\"");
        }
        if (maxheight_ != null) {
            buffer.print(" ");
            rNSContext_.makeQName("", "maxheight", buffer);
            buffer.print("=\"");
            buffer.print(URelaxer.getString(getMaxheight()));
            buffer.print("\"");
        }
        if (maxwidth_ != null) {
            buffer.print(" ");
            rNSContext_.makeQName("", "maxwidth", buffer);
            buffer.print("=\"");
            buffer.print(URelaxer.getString(getMaxwidth()));
            buffer.print("\"");
        }
        if (menu_ != null) {
            buffer.print(" ");
            rNSContext_.makeQName("", "menu", buffer);
            buffer.print("=\"");
            buffer.print(URelaxer.escapeAttrQuot(URelaxer.getString(getMenu())));
            buffer.print("\"");
        }
        if (minheight_ != null) {
            buffer.print(" ");
            rNSContext_.makeQName("", "minheight", buffer);
            buffer.print("=\"");
            buffer.print(URelaxer.getString(getMinheight()));
            buffer.print("\"");
        }
        if (minwidth_ != null) {
            buffer.print(" ");
            rNSContext_.makeQName("", "minwidth", buffer);
            buffer.print("=\"");
            buffer.print(URelaxer.getString(getMinwidth()));
            buffer.print("\"");
        }
        if (observes_ != null) {
            buffer.print(" ");
            rNSContext_.makeQName("", "observes", buffer);
            buffer.print("=\"");
            buffer.print(URelaxer.escapeAttrQuot(URelaxer.getString(getObserves())));
            buffer.print("\"");
        }
        if (ordinal_ != null) {
            buffer.print(" ");
            rNSContext_.makeQName("", "ordinal", buffer);
            buffer.print("=\"");
            buffer.print(URelaxer.getString(getOrdinal()));
            buffer.print("\"");
        }
        if (orient_ != null) {
            buffer.print(" ");
            rNSContext_.makeQName("", "orient", buffer);
            buffer.print("=\"");
            buffer.print(URelaxer.escapeAttrQuot(URelaxer.getString(getOrient())));
            buffer.print("\"");
        }
        if (pack_ != null) {
            buffer.print(" ");
            rNSContext_.makeQName("", "pack", buffer);
            buffer.print("=\"");
            buffer.print(URelaxer.escapeAttrQuot(URelaxer.getString(getPack())));
            buffer.print("\"");
        }
        if (persist_ != null) {
            buffer.print(" ");
            rNSContext_.makeQName("", "persist", buffer);
            buffer.print("=\"");
            buffer.print(URelaxer.escapeAttrQuot(URelaxer.getString(getPersist())));
            buffer.print("\"");
        }
        if (popup_ != null) {
            buffer.print(" ");
            rNSContext_.makeQName("", "popup", buffer);
            buffer.print("=\"");
            buffer.print(URelaxer.escapeAttrQuot(URelaxer.getString(getPopup())));
            buffer.print("\"");
        }
        if (position_ != null) {
            buffer.print(" ");
            rNSContext_.makeQName("", "position", buffer);
            buffer.print("=\"");
            buffer.print(URelaxer.escapeAttrQuot(URelaxer.getString(getPosition())));
            buffer.print("\"");
        }
        if (ref_ != null) {
            buffer.print(" ");
            rNSContext_.makeQName("", "ref", buffer);
            buffer.print("=\"");
            buffer.print(URelaxer.escapeAttrQuot(URelaxer.getString(getRef())));
            buffer.print("\"");
        }
        if (removeelement_ != null) {
            buffer.print(" ");
            rNSContext_.makeQName("", "removeelement", buffer);
            buffer.print("=\"");
            buffer.print(URelaxer.escapeAttrQuot(URelaxer.getString(getRemoveelement())));
            buffer.print("\"");
        }
        if (sortDirection_ != null) {
            buffer.print(" ");
            rNSContext_.makeQName("", "sortDirection", buffer);
            buffer.print("=\"");
            buffer.print(URelaxer.escapeAttrQuot(URelaxer.getString(getSortDirection())));
            buffer.print("\"");
        }
        if (sortResource_ != null) {
            buffer.print(" ");
            rNSContext_.makeQName("", "sortResource", buffer);
            buffer.print("=\"");
            buffer.print(URelaxer.escapeAttrQuot(URelaxer.getString(getSortResource())));
            buffer.print("\"");
        }
        if (sortResource2_ != null) {
            buffer.print(" ");
            rNSContext_.makeQName("", "sortResource2", buffer);
            buffer.print("=\"");
            buffer.print(URelaxer.escapeAttrQuot(URelaxer.getString(getSortResource2())));
            buffer.print("\"");
        }
        if (statustext_ != null) {
            buffer.print(" ");
            rNSContext_.makeQName("", "statustext", buffer);
            buffer.print("=\"");
            buffer.print(URelaxer.escapeAttrQuot(URelaxer.getString(getStatustext())));
            buffer.print("\"");
        }
        if (style_ != null) {
            buffer.print(" ");
            rNSContext_.makeQName("", "style", buffer);
            buffer.print("=\"");
            buffer.print(URelaxer.escapeAttrQuot(URelaxer.getString(getStyle())));
            buffer.print("\"");
        }
        if (template_ != null) {
            buffer.print(" ");
            rNSContext_.makeQName("", "template", buffer);
            buffer.print("=\"");
            buffer.print(URelaxer.escapeAttrQuot(URelaxer.getString(getTemplate())));
            buffer.print("\"");
        }
        if (tooltip_ != null) {
            buffer.print(" ");
            rNSContext_.makeQName("", "tooltip", buffer);
            buffer.print("=\"");
            buffer.print(URelaxer.escapeAttrQuot(URelaxer.getString(getTooltip())));
            buffer.print("\"");
        }
        if (tooltiptext_ != null) {
            buffer.print(" ");
            rNSContext_.makeQName("", "tooltiptext", buffer);
            buffer.print("=\"");
            buffer.print(URelaxer.escapeAttrQuot(URelaxer.getString(getTooltiptext())));
            buffer.print("\"");
        }
        if (top_ != null) {
            buffer.print(" ");
            rNSContext_.makeQName("", "top", buffer);
            buffer.print("=\"");
            buffer.print(URelaxer.getString(getTop()));
            buffer.print("\"");
        }
        if (uri_ != null) {
            buffer.print(" ");
            rNSContext_.makeQName("", "uri", buffer);
            buffer.print("=\"");
            buffer.print(URelaxer.escapeAttrQuot(URelaxer.getString(getUri())));
            buffer.print("\"");
        }
        if (width_ != null) {
            buffer.print(" ");
            rNSContext_.makeQName("", "width", buffer);
            buffer.print("=\"");
            buffer.print(URelaxer.getString(getWidth()));
            buffer.print("\"");
        }
        if (onclick_ != null) {
            buffer.print(" ");
            rNSContext_.makeQName("", "onclick", buffer);
            buffer.print("=\"");
            buffer.print(URelaxer.escapeAttrQuot(URelaxer.getString(getOnclick())));
            buffer.print("\"");
        }
        if (onmousedown_ != null) {
            buffer.print(" ");
            rNSContext_.makeQName("", "onmousedown", buffer);
            buffer.print("=\"");
            buffer.print(URelaxer.escapeAttrQuot(URelaxer.getString(getOnmousedown())));
            buffer.print("\"");
        }
        if (onmouseup_ != null) {
            buffer.print(" ");
            rNSContext_.makeQName("", "onmouseup", buffer);
            buffer.print("=\"");
            buffer.print(URelaxer.escapeAttrQuot(URelaxer.getString(getOnmouseup())));
            buffer.print("\"");
        }
        if (onmouseover_ != null) {
            buffer.print(" ");
            rNSContext_.makeQName("", "onmouseover", buffer);
            buffer.print("=\"");
            buffer.print(URelaxer.escapeAttrQuot(URelaxer.getString(getOnmouseover())));
            buffer.print("\"");
        }
        if (onmousemove_ != null) {
            buffer.print(" ");
            rNSContext_.makeQName("", "onmousemove", buffer);
            buffer.print("=\"");
            buffer.print(URelaxer.escapeAttrQuot(URelaxer.getString(getOnmousemove())));
            buffer.print("\"");
        }
        if (onmouseout_ != null) {
            buffer.print(" ");
            rNSContext_.makeQName("", "onmouseout", buffer);
            buffer.print("=\"");
            buffer.print(URelaxer.escapeAttrQuot(URelaxer.getString(getOnmouseout())));
            buffer.print("\"");
        }
        if (oncommand_ != null) {
            buffer.print(" ");
            rNSContext_.makeQName("", "oncommand", buffer);
            buffer.print("=\"");
            buffer.print(URelaxer.escapeAttrQuot(URelaxer.getString(getOncommand())));
            buffer.print("\"");
        }
        if (onkeypress_ != null) {
            buffer.print(" ");
            rNSContext_.makeQName("", "onkeypress", buffer);
            buffer.print("=\"");
            buffer.print(URelaxer.escapeAttrQuot(URelaxer.getString(getOnkeypress())));
            buffer.print("\"");
        }
        if (onkeydown_ != null) {
            buffer.print(" ");
            rNSContext_.makeQName("", "onkeydown", buffer);
            buffer.print("=\"");
            buffer.print(URelaxer.escapeAttrQuot(URelaxer.getString(getOnkeydown())));
            buffer.print("\"");
        }
        if (onkeyup_ != null) {
            buffer.print(" ");
            rNSContext_.makeQName("", "onkeyup", buffer);
            buffer.print("=\"");
            buffer.print(URelaxer.escapeAttrQuot(URelaxer.getString(getOnkeyup())));
            buffer.print("\"");
        }
        if (onfocus_ != null) {
            buffer.print(" ");
            rNSContext_.makeQName("", "onfocus", buffer);
            buffer.print("=\"");
            buffer.print(URelaxer.escapeAttrQuot(URelaxer.getString(getOnfocus())));
            buffer.print("\"");
        }
        if (onblur_ != null) {
            buffer.print(" ");
            rNSContext_.makeQName("", "onblur", buffer);
            buffer.print("=\"");
            buffer.print(URelaxer.escapeAttrQuot(URelaxer.getString(getOnblur())));
            buffer.print("\"");
        }
        if (onload_ != null) {
            buffer.print(" ");
            rNSContext_.makeQName("", "onload", buffer);
            buffer.print("=\"");
            buffer.print(URelaxer.escapeAttrQuot(URelaxer.getString(getOnload())));
            buffer.print("\"");
        }
        if (onunload_ != null) {
            buffer.print(" ");
            rNSContext_.makeQName("", "onunload", buffer);
            buffer.print("=\"");
            buffer.print(URelaxer.escapeAttrQuot(URelaxer.getString(getOnunload())));
            buffer.print("\"");
        }
        if (hidechrome_ != null) {
            buffer.print(" ");
            rNSContext_.makeQName("", "hidechrome", buffer);
            buffer.print("=\"");
            buffer.print(URelaxer.getString(getHidechrome()));
            buffer.print("\"");
        }
        if (screenX_ != null) {
            buffer.print(" ");
            rNSContext_.makeQName("", "screenX", buffer);
            buffer.print("=\"");
            buffer.print(URelaxer.getString(getScreenX()));
            buffer.print("\"");
        }
        if (screenY_ != null) {
            buffer.print(" ");
            rNSContext_.makeQName("", "screenY", buffer);
            buffer.print("=\"");
            buffer.print(URelaxer.getString(getScreenY()));
            buffer.print("\"");
        }
        if (sizemode_ != null) {
            buffer.print(" ");
            rNSContext_.makeQName("", "sizemode", buffer);
            buffer.print("=\"");
            buffer.print(URelaxer.escapeAttrQuot(URelaxer.getString(getSizemode())));
            buffer.print("\"");
        }
        if (title_ != null) {
            buffer.print(" ");
            rNSContext_.makeQName("", "title", buffer);
            buffer.print("=\"");
            buffer.print(URelaxer.escapeAttrQuot(URelaxer.getString(getTitle())));
            buffer.print("\"");
        }
        if (windowtype_ != null) {
            buffer.print(" ");
            rNSContext_.makeQName("", "windowtype", buffer);
            buffer.print("=\"");
            buffer.print(URelaxer.escapeAttrQuot(URelaxer.getString(getWindowtype())));
            buffer.print("\"");
        }
        size = this.blockClass_.size();
        for (int i = 0;i < size;i++) {
            IXulWindowChoice value = (IXulWindowChoice)this.blockClass_.get(i);
            value.makeTextAttribute(buffer);
        }
        buffer.print(">");
        size = this.blockClass_.size();
        for (int i = 0;i < size;i++) {
            IXulWindowChoice value = (IXulWindowChoice)this.blockClass_.get(i);
            value.makeTextElement(buffer);
        }
        buffer.print("</");
        URelaxer.makeQName(prefix, "window", buffer);
        buffer.print(">");
    }

    /**
     * Makes an XML text representation.
     *
     * @param buffer
     */
    public void makeTextAttribute(StringBuffer buffer) {
    }

    /**
     * Makes an XML text representation.
     *
     * @param buffer
     * @exception IOException
     */
    public void makeTextAttribute(Writer buffer) throws IOException {
    }

    /**
     * Makes an XML text representation.
     *
     * @param buffer
     */
    public void makeTextAttribute(PrintWriter buffer) {
    }

    /**
     * Gets the property value as String.
     *
     * @return String
     */
    public String getAlignAsString() {
        return (URelaxer.getString(getAlign()));
    }

    /**
     * Gets the property value as String.
     *
     * @return String
     */
    public String getAlloweventsAsString() {
        return (URelaxer.getString(getAllowevents()));
    }

    /**
     * Gets the property value as String.
     *
     * @return String
     */
    public String getAllownegativeassertionsAsString() {
        return (URelaxer.getString(getAllownegativeassertions()));
    }

    /**
     * Gets the property value as String.
     *
     * @return String
     */
    public String getClassValueAsString() {
        return (URelaxer.getString(getClassValue()));
    }

    /**
     * Gets the property value as String.
     *
     * @return String
     */
    public String getCoalesceduplicatearcsAsString() {
        return (URelaxer.getString(getCoalesceduplicatearcs()));
    }

    /**
     * Gets the property value as String.
     *
     * @return String
     */
    public String getCollapsedAsString() {
        return (URelaxer.getString(getCollapsed()));
    }

    /**
     * Gets the property value as String.
     *
     * @return String
     */
    public String getContainerAsString() {
        return (URelaxer.getString(getContainer()));
    }

    /**
     * Gets the property value as String.
     *
     * @return String
     */
    public String getContainmentAsString() {
        return (URelaxer.getString(getContainment()));
    }

    /**
     * Gets the property value as String.
     *
     * @return String
     */
    public String getContextAsString() {
        return (URelaxer.getString(getContext()));
    }

    /**
     * Gets the property value as String.
     *
     * @return String
     */
    public String getContextmenuAsString() {
        return (URelaxer.getString(getContextmenu()));
    }

    /**
     * Gets the property value as String.
     *
     * @return String
     */
    public String getDatasourcesAsString() {
        return (URelaxer.getString(getDatasources()));
    }

    /**
     * Gets the property value as String.
     *
     * @return String
     */
    public String getDebugAsString() {
        return (URelaxer.getString(getDebug()));
    }

    /**
     * Gets the property value as String.
     *
     * @return String
     */
    public String getDirAsString() {
        return (URelaxer.getString(getDir()));
    }

    /**
     * Gets the property value as String.
     *
     * @return String
     */
    public String getEmptyAsString() {
        return (URelaxer.getString(getEmpty()));
    }

    /**
     * Gets the property value as String.
     *
     * @return String
     */
    public String getEqualsizeAsString() {
        return (URelaxer.getString(getEqualsize()));
    }

    /**
     * Gets the property value as String.
     *
     * @return String
     */
    public String getFlagsAsString() {
        return (URelaxer.getString(getFlags()));
    }

    /**
     * Gets the property value as String.
     *
     * @return String
     */
    public String getFlexAsString() {
        return (URelaxer.getString(getFlex()));
    }

    /**
     * Gets the property value as String.
     *
     * @return String
     */
    public String getHeightAsString() {
        return (URelaxer.getString(getHeight()));
    }

    /**
     * Gets the property value as String.
     *
     * @return String
     */
    public String getHiddenAsString() {
        return (URelaxer.getString(getHidden()));
    }

    /**
     * Gets the property value as String.
     *
     * @return String
     */
    public String getIdAsString() {
        return (URelaxer.getString(getId()));
    }

    /**
     * Gets the property value as String.
     *
     * @return String
     */
    public String getInsertafterAsString() {
        return (URelaxer.getString(getInsertafter()));
    }

    /**
     * Gets the property value as String.
     *
     * @return String
     */
    public String getInsertbeforeAsString() {
        return (URelaxer.getString(getInsertbefore()));
    }

    /**
     * Gets the property value as String.
     *
     * @return String
     */
    public String getLeftAsString() {
        return (URelaxer.getString(getLeft()));
    }

    /**
     * Gets the property value as String.
     *
     * @return String
     */
    public String getMaxheightAsString() {
        return (URelaxer.getString(getMaxheight()));
    }

    /**
     * Gets the property value as String.
     *
     * @return String
     */
    public String getMaxwidthAsString() {
        return (URelaxer.getString(getMaxwidth()));
    }

    /**
     * Gets the property value as String.
     *
     * @return String
     */
    public String getMenuAsString() {
        return (URelaxer.getString(getMenu()));
    }

    /**
     * Gets the property value as String.
     *
     * @return String
     */
    public String getMinheightAsString() {
        return (URelaxer.getString(getMinheight()));
    }

    /**
     * Gets the property value as String.
     *
     * @return String
     */
    public String getMinwidthAsString() {
        return (URelaxer.getString(getMinwidth()));
    }

    /**
     * Gets the property value as String.
     *
     * @return String
     */
    public String getObservesAsString() {
        return (URelaxer.getString(getObserves()));
    }

    /**
     * Gets the property value as String.
     *
     * @return String
     */
    public String getOrdinalAsString() {
        return (URelaxer.getString(getOrdinal()));
    }

    /**
     * Gets the property value as String.
     *
     * @return String
     */
    public String getOrientAsString() {
        return (URelaxer.getString(getOrient()));
    }

    /**
     * Gets the property value as String.
     *
     * @return String
     */
    public String getPackAsString() {
        return (URelaxer.getString(getPack()));
    }

    /**
     * Gets the property value as String.
     *
     * @return String
     */
    public String getPersistAsString() {
        return (URelaxer.getString(getPersist()));
    }

    /**
     * Gets the property value as String.
     *
     * @return String
     */
    public String getPopupAsString() {
        return (URelaxer.getString(getPopup()));
    }

    /**
     * Gets the property value as String.
     *
     * @return String
     */
    public String getPositionAsString() {
        return (URelaxer.getString(getPosition()));
    }

    /**
     * Gets the property value as String.
     *
     * @return String
     */
    public String getRefAsString() {
        return (URelaxer.getString(getRef()));
    }

    /**
     * Gets the property value as String.
     *
     * @return String
     */
    public String getRemoveelementAsString() {
        return (URelaxer.getString(getRemoveelement()));
    }

    /**
     * Gets the property value as String.
     *
     * @return String
     */
    public String getSortDirectionAsString() {
        return (URelaxer.getString(getSortDirection()));
    }

    /**
     * Gets the property value as String.
     *
     * @return String
     */
    public String getSortResourceAsString() {
        return (URelaxer.getString(getSortResource()));
    }

    /**
     * Gets the property value as String.
     *
     * @return String
     */
    public String getSortResource2AsString() {
        return (URelaxer.getString(getSortResource2()));
    }

    /**
     * Gets the property value as String.
     *
     * @return String
     */
    public String getStatustextAsString() {
        return (URelaxer.getString(getStatustext()));
    }

    /**
     * Gets the property value as String.
     *
     * @return String
     */
    public String getStyleAsString() {
        return (URelaxer.getString(getStyle()));
    }

    /**
     * Gets the property value as String.
     *
     * @return String
     */
    public String getTemplateAsString() {
        return (URelaxer.getString(getTemplate()));
    }

    /**
     * Gets the property value as String.
     *
     * @return String
     */
    public String getTooltipAsString() {
        return (URelaxer.getString(getTooltip()));
    }

    /**
     * Gets the property value as String.
     *
     * @return String
     */
    public String getTooltiptextAsString() {
        return (URelaxer.getString(getTooltiptext()));
    }

    /**
     * Gets the property value as String.
     *
     * @return String
     */
    public String getTopAsString() {
        return (URelaxer.getString(getTop()));
    }

    /**
     * Gets the property value as String.
     *
     * @return String
     */
    public String getUriAsString() {
        return (URelaxer.getString(getUri()));
    }

    /**
     * Gets the property value as String.
     *
     * @return String
     */
    public String getWidthAsString() {
        return (URelaxer.getString(getWidth()));
    }

    /**
     * Gets the property value as String.
     *
     * @return String
     */
    public String getOnclickAsString() {
        return (URelaxer.getString(getOnclick()));
    }

    /**
     * Gets the property value as String.
     *
     * @return String
     */
    public String getOnmousedownAsString() {
        return (URelaxer.getString(getOnmousedown()));
    }

    /**
     * Gets the property value as String.
     *
     * @return String
     */
    public String getOnmouseupAsString() {
        return (URelaxer.getString(getOnmouseup()));
    }

    /**
     * Gets the property value as String.
     *
     * @return String
     */
    public String getOnmouseoverAsString() {
        return (URelaxer.getString(getOnmouseover()));
    }

    /**
     * Gets the property value as String.
     *
     * @return String
     */
    public String getOnmousemoveAsString() {
        return (URelaxer.getString(getOnmousemove()));
    }

    /**
     * Gets the property value as String.
     *
     * @return String
     */
    public String getOnmouseoutAsString() {
        return (URelaxer.getString(getOnmouseout()));
    }

    /**
     * Gets the property value as String.
     *
     * @return String
     */
    public String getOncommandAsString() {
        return (URelaxer.getString(getOncommand()));
    }

    /**
     * Gets the property value as String.
     *
     * @return String
     */
    public String getOnkeypressAsString() {
        return (URelaxer.getString(getOnkeypress()));
    }

    /**
     * Gets the property value as String.
     *
     * @return String
     */
    public String getOnkeydownAsString() {
        return (URelaxer.getString(getOnkeydown()));
    }

    /**
     * Gets the property value as String.
     *
     * @return String
     */
    public String getOnkeyupAsString() {
        return (URelaxer.getString(getOnkeyup()));
    }

    /**
     * Gets the property value as String.
     *
     * @return String
     */
    public String getOnfocusAsString() {
        return (URelaxer.getString(getOnfocus()));
    }

    /**
     * Gets the property value as String.
     *
     * @return String
     */
    public String getOnblurAsString() {
        return (URelaxer.getString(getOnblur()));
    }

    /**
     * Gets the property value as String.
     *
     * @return String
     */
    public String getOnloadAsString() {
        return (URelaxer.getString(getOnload()));
    }

    /**
     * Gets the property value as String.
     *
     * @return String
     */
    public String getOnunloadAsString() {
        return (URelaxer.getString(getOnunload()));
    }

    /**
     * Gets the property value as String.
     *
     * @return String
     */
    public String getHidechromeAsString() {
        return (URelaxer.getString(getHidechrome()));
    }

    /**
     * Gets the property value as String.
     *
     * @return String
     */
    public String getScreenXAsString() {
        return (URelaxer.getString(getScreenX()));
    }

    /**
     * Gets the property value as String.
     *
     * @return String
     */
    public String getScreenYAsString() {
        return (URelaxer.getString(getScreenY()));
    }

    /**
     * Gets the property value as String.
     *
     * @return String
     */
    public String getSizemodeAsString() {
        return (URelaxer.getString(getSizemode()));
    }

    /**
     * Gets the property value as String.
     *
     * @return String
     */
    public String getTitleAsString() {
        return (URelaxer.getString(getTitle()));
    }

    /**
     * Gets the property value as String.
     *
     * @return String
     */
    public String getWindowtypeAsString() {
        return (URelaxer.getString(getWindowtype()));
    }

    /**
     * Sets the property value by String.
     *
     * @param string
     */
    public void setAlignByString(String string) {
        setAlign(string);
    }

    /**
     * Sets the property value by String.
     *
     * @param string
     */
    public void setAlloweventsByString(String string) {
        setAllowevents(new Boolean(string).booleanValue());
    }

    /**
     * Sets the property value by String.
     *
     * @param string
     */
    public void setAllownegativeassertionsByString(String string) {
        setAllownegativeassertions(new Boolean(string).booleanValue());
    }

    /**
     * Sets the property value by String.
     *
     * @param string
     */
    public void setClassValueByString(String string) {
        setClassValue(string);
    }

    /**
     * Sets the property value by String.
     *
     * @param string
     */
    public void setCoalesceduplicatearcsByString(String string) {
        setCoalesceduplicatearcs(new Boolean(string).booleanValue());
    }

    /**
     * Sets the property value by String.
     *
     * @param string
     */
    public void setCollapsedByString(String string) {
        setCollapsed(new Boolean(string).booleanValue());
    }

    /**
     * Sets the property value by String.
     *
     * @param string
     */
    public void setContainerByString(String string) {
        setContainer(string);
    }

    /**
     * Sets the property value by String.
     *
     * @param string
     */
    public void setContainmentByString(String string) {
        setContainment(string);
    }

    /**
     * Sets the property value by String.
     *
     * @param string
     */
    public void setContextByString(String string) {
        setContext(string);
    }

    /**
     * Sets the property value by String.
     *
     * @param string
     */
    public void setContextmenuByString(String string) {
        setContextmenu(string);
    }

    /**
     * Sets the property value by String.
     *
     * @param string
     */
    public void setDatasourcesByString(String string) {
        setDatasources(string);
    }

    /**
     * Sets the property value by String.
     *
     * @param string
     */
    public void setDebugByString(String string) {
        setDebug(new Boolean(string).booleanValue());
    }

    /**
     * Sets the property value by String.
     *
     * @param string
     */
    public void setDirByString(String string) {
        setDir(string);
    }

    /**
     * Sets the property value by String.
     *
     * @param string
     */
    public void setEmptyByString(String string) {
        setEmpty(new Boolean(string).booleanValue());
    }

    /**
     * Sets the property value by String.
     *
     * @param string
     */
    public void setEqualsizeByString(String string) {
        setEqualsize(string);
    }

    /**
     * Sets the property value by String.
     *
     * @param string
     */
    public void setFlagsByString(String string) {
        setFlags(string);
    }

    /**
     * Sets the property value by String.
     *
     * @param string
     */
    public void setFlexByString(String string) {
        setFlex(new java.math.BigInteger(string));
    }

    /**
     * Sets the property value by String.
     *
     * @param string
     */
    public void setHeightByString(String string) {
        setHeight(new java.math.BigInteger(string));
    }

    /**
     * Sets the property value by String.
     *
     * @param string
     */
    public void setHiddenByString(String string) {
        setHidden(new Boolean(string).booleanValue());
    }

    /**
     * Sets the property value by String.
     *
     * @param string
     */
    public void setIdByString(String string) {
        setId(string);
    }

    /**
     * Sets the property value by String.
     *
     * @param string
     */
    public void setInsertafterByString(String string) {
        setInsertafter(string);
    }

    /**
     * Sets the property value by String.
     *
     * @param string
     */
    public void setInsertbeforeByString(String string) {
        setInsertbefore(string);
    }

    /**
     * Sets the property value by String.
     *
     * @param string
     */
    public void setLeftByString(String string) {
        setLeft(new java.math.BigInteger(string));
    }

    /**
     * Sets the property value by String.
     *
     * @param string
     */
    public void setMaxheightByString(String string) {
        setMaxheight(new java.math.BigInteger(string));
    }

    /**
     * Sets the property value by String.
     *
     * @param string
     */
    public void setMaxwidthByString(String string) {
        setMaxwidth(new java.math.BigInteger(string));
    }

    /**
     * Sets the property value by String.
     *
     * @param string
     */
    public void setMenuByString(String string) {
        setMenu(string);
    }

    /**
     * Sets the property value by String.
     *
     * @param string
     */
    public void setMinheightByString(String string) {
        setMinheight(new java.math.BigInteger(string));
    }

    /**
     * Sets the property value by String.
     *
     * @param string
     */
    public void setMinwidthByString(String string) {
        setMinwidth(new java.math.BigInteger(string));
    }

    /**
     * Sets the property value by String.
     *
     * @param string
     */
    public void setObservesByString(String string) {
        setObserves(string);
    }

    /**
     * Sets the property value by String.
     *
     * @param string
     */
    public void setOrdinalByString(String string) {
        setOrdinal(new java.math.BigInteger(string));
    }

    /**
     * Sets the property value by String.
     *
     * @param string
     */
    public void setOrientByString(String string) {
        setOrient(string);
    }

    /**
     * Sets the property value by String.
     *
     * @param string
     */
    public void setPackByString(String string) {
        setPack(string);
    }

    /**
     * Sets the property value by String.
     *
     * @param string
     */
    public void setPersistByString(String string) {
        setPersist(string);
    }

    /**
     * Sets the property value by String.
     *
     * @param string
     */
    public void setPopupByString(String string) {
        setPopup(string);
    }

    /**
     * Sets the property value by String.
     *
     * @param string
     */
    public void setPositionByString(String string) {
        setPosition(string);
    }

    /**
     * Sets the property value by String.
     *
     * @param string
     */
    public void setRefByString(String string) {
        setRef(string);
    }

    /**
     * Sets the property value by String.
     *
     * @param string
     */
    public void setRemoveelementByString(String string) {
        setRemoveelement(string);
    }

    /**
     * Sets the property value by String.
     *
     * @param string
     */
    public void setSortDirectionByString(String string) {
        setSortDirection(string);
    }

    /**
     * Sets the property value by String.
     *
     * @param string
     */
    public void setSortResourceByString(String string) {
        setSortResource(string);
    }

    /**
     * Sets the property value by String.
     *
     * @param string
     */
    public void setSortResource2ByString(String string) {
        setSortResource2(string);
    }

    /**
     * Sets the property value by String.
     *
     * @param string
     */
    public void setStatustextByString(String string) {
        setStatustext(string);
    }

    /**
     * Sets the property value by String.
     *
     * @param string
     */
    public void setStyleByString(String string) {
        setStyle(string);
    }

    /**
     * Sets the property value by String.
     *
     * @param string
     */
    public void setTemplateByString(String string) {
        setTemplate(string);
    }

    /**
     * Sets the property value by String.
     *
     * @param string
     */
    public void setTooltipByString(String string) {
        setTooltip(string);
    }

    /**
     * Sets the property value by String.
     *
     * @param string
     */
    public void setTooltiptextByString(String string) {
        setTooltiptext(string);
    }

    /**
     * Sets the property value by String.
     *
     * @param string
     */
    public void setTopByString(String string) {
        setTop(new java.math.BigInteger(string));
    }

    /**
     * Sets the property value by String.
     *
     * @param string
     */
    public void setUriByString(String string) {
        setUri(string);
    }

    /**
     * Sets the property value by String.
     *
     * @param string
     */
    public void setWidthByString(String string) {
        setWidth(new java.math.BigInteger(string));
    }

    /**
     * Sets the property value by String.
     *
     * @param string
     */
    public void setOnclickByString(String string) {
        setOnclick(string);
    }

    /**
     * Sets the property value by String.
     *
     * @param string
     */
    public void setOnmousedownByString(String string) {
        setOnmousedown(string);
    }

    /**
     * Sets the property value by String.
     *
     * @param string
     */
    public void setOnmouseupByString(String string) {
        setOnmouseup(string);
    }

    /**
     * Sets the property value by String.
     *
     * @param string
     */
    public void setOnmouseoverByString(String string) {
        setOnmouseover(string);
    }

    /**
     * Sets the property value by String.
     *
     * @param string
     */
    public void setOnmousemoveByString(String string) {
        setOnmousemove(string);
    }

    /**
     * Sets the property value by String.
     *
     * @param string
     */
    public void setOnmouseoutByString(String string) {
        setOnmouseout(string);
    }

    /**
     * Sets the property value by String.
     *
     * @param string
     */
    public void setOncommandByString(String string) {
        setOncommand(string);
    }

    /**
     * Sets the property value by String.
     *
     * @param string
     */
    public void setOnkeypressByString(String string) {
        setOnkeypress(string);
    }

    /**
     * Sets the property value by String.
     *
     * @param string
     */
    public void setOnkeydownByString(String string) {
        setOnkeydown(string);
    }

    /**
     * Sets the property value by String.
     *
     * @param string
     */
    public void setOnkeyupByString(String string) {
        setOnkeyup(string);
    }

    /**
     * Sets the property value by String.
     *
     * @param string
     */
    public void setOnfocusByString(String string) {
        setOnfocus(string);
    }

    /**
     * Sets the property value by String.
     *
     * @param string
     */
    public void setOnblurByString(String string) {
        setOnblur(string);
    }

    /**
     * Sets the property value by String.
     *
     * @param string
     */
    public void setOnloadByString(String string) {
        setOnload(string);
    }

    /**
     * Sets the property value by String.
     *
     * @param string
     */
    public void setOnunloadByString(String string) {
        setOnunload(string);
    }

    /**
     * Sets the property value by String.
     *
     * @param string
     */
    public void setHidechromeByString(String string) {
        setHidechrome(new Boolean(string).booleanValue());
    }

    /**
     * Sets the property value by String.
     *
     * @param string
     */
    public void setScreenXByString(String string) {
        setScreenX(new java.math.BigInteger(string));
    }

    /**
     * Sets the property value by String.
     *
     * @param string
     */
    public void setScreenYByString(String string) {
        setScreenY(new java.math.BigInteger(string));
    }

    /**
     * Sets the property value by String.
     *
     * @param string
     */
    public void setSizemodeByString(String string) {
        setSizemode(string);
    }

    /**
     * Sets the property value by String.
     *
     * @param string
     */
    public void setTitleByString(String string) {
        setTitle(string);
    }

    /**
     * Sets the property value by String.
     *
     * @param string
     */
    public void setWindowtypeByString(String string) {
        setWindowtype(string);
    }

    /**
     * Returns a String representation of this object.
     * While this method informs as XML format representaion, 
     *  it's purpose is just information, not making 
     * a rigid XML documentation.
     *
     * @return String
     */
    public String toString() {
        try {
            return (makeTextDocument());
        } catch (Exception e) {
            return (super.toString());
        }
    }

    /**
     * Accepts the Visitor for enter behavior.
     *
     * @param visitor
     * @return boolean
     */
    public boolean enter(IRVisitor visitor) {
        return (visitor.enter(this));
    }

    /**
     * Accepts the Visitor for leave behavior.
     *
     * @param visitor
     */
    public void leave(IRVisitor visitor) {
        visitor.leave(this);
    }

    /**
     * Gets PI by name.
     *
     * @param name
     * @return String
     */
    public String rGetPi(String name) {
        return ((String)xmlPIMap_.get(name));
    }

    /**
     * Gets PI map.
     *
     * @return java.util.Map
     */
    public java.util.Map rGetPiMap() {
        return (java.util.Collections.unmodifiableMap(xmlPIMap_));
    }

    /**
     * Gets the IRNode property <b>parentRNode</b>.
     *
     * @return IRNode
     */
    public IRNode rGetParentRNode() {
        return (parentRNode_);
    }

    /**
     * Sets the IRNode property <b>parentRNode</b>.
     *
     * @param parentRNode
     */
    public void rSetParentRNode(IRNode parentRNode) {
        this.parentRNode_ = parentRNode;
    }

    /**
     * Gets child RNodes.
     *
     * @return IRNode[]
     */
    public IRNode[] rGetRNodes() {
        java.util.List classNodes = new java.util.ArrayList();
        classNodes.addAll(blockClass_);
        IRNode[] nodes = new IRNode[classNodes.size()];
        return ((IRNode[])classNodes.toArray(nodes));
    }

    /**
     * Tests if a Element <code>element</code> is valid
     * for the <code>XulWindow</code>.
     *
     * @param element
     * @return boolean
     */
    public static boolean isMatch(Element element) {
        if (!URelaxer2.isTargetElement(element, "http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul", "window")) {
            return (false);
        }
        RStack target = new RStack(element);
        boolean $match$ = false;
        Element child;
        while (true) {
            if (XulButton.isMatchHungry(target)) {
                $match$ = true;
            } else if (XulTextbox.isMatchHungry(target)) {
                $match$ = true;
            } else if (XulCommandset.isMatchHungry(target)) {
                $match$ = true;
            } else if (XulLabel.isMatchHungry(target)) {
                $match$ = true;
            } else if (XulObserves.isMatchHungry(target)) {
                $match$ = true;
            } else if (XulListbox.isMatchHungry(target)) {
                $match$ = true;
            } else if (XulRadio.isMatchHungry(target)) {
                $match$ = true;
            } else if (XulCheckbox.isMatchHungry(target)) {
                $match$ = true;
            } else if (XulTree.isMatchHungry(target)) {
                $match$ = true;
            } else if (XulMenupopup.isMatchHungry(target)) {
                $match$ = true;
            } else if (XulSplitter.isMatchHungry(target)) {
                $match$ = true;
            } else if (XulTabbox.isMatchHungry(target)) {
                $match$ = true;
            } else if (XulImage.isMatchHungry(target)) {
                $match$ = true;
            } else if (XulCommand.isMatchHungry(target)) {
                $match$ = true;
            } else if (XulDescription.isMatchHungry(target)) {
                $match$ = true;
            } else if (XulGroupbox.isMatchHungry(target)) {
                $match$ = true;
            } else if (XulDeck.isMatchHungry(target)) {
                $match$ = true;
            } else if (XulScript.isMatchHungry(target)) {
                $match$ = true;
            } else if (XulMenubar.isMatchHungry(target)) {
                $match$ = true;
            } else if (XulArrowscrollbox.isMatchHungry(target)) {
                $match$ = true;
            } else if (XulBox.isMatchHungry(target)) {
                $match$ = true;
            } else if (XulHbox.isMatchHungry(target)) {
                $match$ = true;
            } else if (XulVbox.isMatchHungry(target)) {
                $match$ = true;
            } else if (XulBroadcasterset.isMatchHungry(target)) {
                $match$ = true;
            } else if (XulStack.isMatchHungry(target)) {
                $match$ = true;
            } else if (XulGrid.isMatchHungry(target)) {
                $match$ = true;
            } else if (XulIframe.isMatchHungry(target)) {
                $match$ = true;
            } else if (XulKeyset.isMatchHungry(target)) {
                $match$ = true;
            } else if (XulTemplate.isMatchHungry(target)) {
                $match$ = true;
            } else if (XulToolbox.isMatchHungry(target)) {
                $match$ = true;
            } else if (XulBroadcaster.isMatchHungry(target)) {
                $match$ = true;
            } else if (XulSeparator.isMatchHungry(target)) {
                $match$ = true;
            } else if (XulSpacer.isMatchHungry(target)) {
                $match$ = true;
            } else if (XulMenulist.isMatchHungry(target)) {
                $match$ = true;
            } else if (XulRadiogroup.isMatchHungry(target)) {
                $match$ = true;
            } else if (XulPopupset.isMatchHungry(target)) {
                $match$ = true;
            } else {
                break;
            }
        }
        if (!target.isEmptyElement()) {
            return (false);
        }
        return (true);
    }

    /**
     * Tests if elements contained in a Stack <code>stack</code>
     * is valid for the <code>XulWindow</code>.
     * This mehtod is supposed to be used internally
     * by the Relaxer system.
     *
     * @param stack
     * @return boolean
     */
    public static boolean isMatch(RStack stack) {
        Element element = stack.peekElement();
        if (element == null) {
            return (false);
        }
        return (isMatch(element));
    }

    /**
     * Tests if elements contained in a Stack <code>stack</code>
     * is valid for the <code>XulWindow</code>.
     * This method consumes the stack contents during matching operation.
     * This mehtod is supposed to be used internally
     * by the Relaxer system.
     *
     * @param stack
     * @return boolean
     */
    public static boolean isMatchHungry(RStack stack) {
        Element element = stack.peekElement();
        if (element == null) {
            return (false);
        }
        if (isMatch(element)) {
            stack.popElement();
            return (true);
        } else {
            return (false);
        }
    }
}
