package com.netpotlet.xul.renderkit;

import java.io.IOException;
import java.util.Map;

import javax.faces.FacesException;
import javax.faces.component.UIComponent;
import javax.faces.context.FacesContext;
import javax.faces.context.ResponseWriter;
import javax.faces.event.ActionEvent;
import javax.faces.render.Renderer;

import org.apache.commons.el.parser.ParseException;

import com.netpotlet.cove.el.CoveElResolver;
import com.netpotlet.xul.component.ButtonComponent;
import com.netpotlet.xul.element.RNSContext;
import com.netpotlet.xul.element.URelaxer;

public class ButtonRenderer extends Renderer {

    public ButtonRenderer() {
        super();
    }
    
    public void decode(FacesContext context, UIComponent component) {
        if ((context == null) || (component == null)) {
            throw new NullPointerException();
        }
        ButtonComponent button = (ButtonComponent) component;
        String clientId = button.getClientId(context); 
        Map requestParameterMap =
            (Map)context.getExternalContext().getRequestParameterMap();
        if (!requestParameterMap.containsKey("id")) return;
        String id = (String)requestParameterMap.get("id");
        if (id.equals(clientId)) {
            button.queueEvent(new ActionEvent(button));
        }
    }

    public void encodeBegin(FacesContext context, UIComponent component)
        throws IOException {
        if ((context == null) || (component == null)) {
            throw new NullPointerException();
        }
        ButtonComponent button = (ButtonComponent) component;
        try {
            char[] encoded = getEncodedBeginString(button);
            ResponseWriter writer = context.getResponseWriter();
            writer.writeText(encoded, 0, encoded.length);
        } catch (ParseException e) {
            throw new FacesException(e);
        }
    }

    private char[] getEncodedBeginString(ButtonComponent button)
        throws ParseException {
        RNSContext rNSContext_ = button.xulButton.rGetRNSContext();
        String mixed;
        String resolvedString;
        StringBuffer buffer = new StringBuffer();
        String prefix =
            rNSContext_.getPrefixByUri(
                "http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul");

        buffer.append("<");
        URelaxer.makeQName(prefix, "button", buffer);
        rNSContext_.makeNSMappings(buffer);
        if (button.xulButton.getAlign() != null) {
            buffer.append(" ");
            rNSContext_.makeQName("", "align", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.escapeAttrQuot(URelaxer.getString(button.xulButton.getAlign())));
            buffer.append("\"");
        }
        if (button.xulButton.getAllowevents() != false) {
            buffer.append(" ");
            rNSContext_.makeQName("", "allowevents", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.getString(new Boolean(button.xulButton.getAllowevents())));
            buffer.append("\"");
        }
        if (button.xulButton.getAllownegativeassertions() != false) {
            buffer.append(" ");
            rNSContext_.makeQName("", "allownegativeassertions", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.getString(button.xulButton.getAllownegativeassertions()));
            buffer.append("\"");
        }
        if (button.xulButton.getClassValue() != null) {
            buffer.append(" ");
            rNSContext_.makeQName("", "class", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.escapeAttrQuot(URelaxer.getString(button.xulButton.getClassValue())));
            buffer.append("\"");
        }
        if (button.xulButton.getCoalesceduplicatearcs() != false) {
            buffer.append(" ");
            rNSContext_.makeQName("", "coalesceduplicatearcs", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.getString(button.xulButton.getCoalesceduplicatearcs()));
            buffer.append("\"");
        }
        if (button.xulButton.getCollapsed() != false) {
            buffer.append(" ");
            rNSContext_.makeQName("", "collapsed", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.getString(button.xulButton.getCollapsed()));
            buffer.append("\"");
        }
        if (button.xulButton.getContainer() != null) {
            buffer.append(" ");
            rNSContext_.makeQName("", "container", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.escapeAttrQuot(URelaxer.getString(button.xulButton.getContainer())));
            buffer.append("\"");
        }
        if (button.xulButton.getContainment() != null) {
            buffer.append(" ");
            rNSContext_.makeQName("", "containment", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.escapeAttrQuot(URelaxer.getString(button.xulButton.getContainment())));
            buffer.append("\"");
        }
        if (button.xulButton.getContext() != null) {
            buffer.append(" ");
            rNSContext_.makeQName("", "context", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.escapeAttrQuot(URelaxer.getString(button.xulButton.getContext())));
            buffer.append("\"");
        }
        if (button.xulButton.getContextmenu() != null) {
            buffer.append(" ");
            rNSContext_.makeQName("", "contextmenu", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.escapeAttrQuot(URelaxer.getString(button.xulButton.getContextmenu())));
            buffer.append("\"");
        }
        if (button.xulButton.getDatasources() != null) {
            buffer.append(" ");
            rNSContext_.makeQName("", "datasources", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.escapeAttrQuot(URelaxer.getString(button.xulButton.getDatasources())));
            buffer.append("\"");
        }
        if (button.xulButton.getDebug() != false) {
            buffer.append(" ");
            rNSContext_.makeQName("", "debug", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.getString(button.xulButton.getDebug()));
            buffer.append("\"");
        }
        if (button.xulButton.getDir() != null) {
            buffer.append(" ");
            rNSContext_.makeQName("", "dir", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.escapeAttrQuot(URelaxer.getString(button.xulButton.getDir())));
            buffer.append("\"");
        }
        if (button.xulButton.getEmpty() != false) {
            buffer.append(" ");
            rNSContext_.makeQName("", "empty", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.getString(button.xulButton.getEmpty()));
            buffer.append("\"");
        }
        if (button.xulButton.getEqualsize() != null) {
            buffer.append(" ");
            rNSContext_.makeQName("", "equalsize", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.escapeAttrQuot(URelaxer.getString(button.xulButton.getEqualsize())));
            buffer.append("\"");
        }
        if (button.xulButton.getFlags() != null) {
            buffer.append(" ");
            rNSContext_.makeQName("", "flags", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.escapeAttrQuot(URelaxer.getString(button.xulButton.getFlags())));
            buffer.append("\"");
        }
        if (button.xulButton.getFlex() != null) {
            buffer.append(" ");
            rNSContext_.makeQName("", "flex", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.getString(button.xulButton.getFlex()));
            buffer.append("\"");
        }
        if (button.xulButton.getHeight() != null) {
            buffer.append(" ");
            rNSContext_.makeQName("", "height", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.getString(button.xulButton.getHeight()));
            buffer.append("\"");
        }
        if (button.xulButton.getHidden() != false) {
            buffer.append(" ");
            rNSContext_.makeQName("", "hidden", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.getString(button.xulButton.getHidden()));
            buffer.append("\"");
        }
        if (button.xulButton.getId() != null) {
            buffer.append(" ");
            rNSContext_.makeQName("", "id", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.escapeAttrQuot(URelaxer.getString(button.xulButton.getId())));
            buffer.append("\"");
        }
        if (button.xulButton.getInsertafter() != null) {
            buffer.append(" ");
            rNSContext_.makeQName("", "insertafter", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.escapeAttrQuot(URelaxer.getString(button.xulButton.getInsertafter())));
            buffer.append("\"");
        }
        if (button.xulButton.getInsertbefore() != null) {
            buffer.append(" ");
            rNSContext_.makeQName("", "insertbefore", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.escapeAttrQuot(URelaxer.getString(button.xulButton.getInsertbefore())));
            buffer.append("\"");
        }
        if (button.xulButton.getLeft() != null) {
            buffer.append(" ");
            rNSContext_.makeQName("", "left", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.getString(button.xulButton.getLeft()));
            buffer.append("\"");
        }
        if (button.xulButton.getMaxheight() != null) {
            buffer.append(" ");
            rNSContext_.makeQName("", "maxheight", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.getString(button.xulButton.getMaxheight()));
            buffer.append("\"");
        }
        if (button.xulButton.getMaxwidth() != null) {
            buffer.append(" ");
            rNSContext_.makeQName("", "maxwidth", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.getString(button.xulButton.getMaxwidth()));
            buffer.append("\"");
        }
        if (button.xulButton.getMenu() != null) {
            buffer.append(" ");
            rNSContext_.makeQName("", "menu", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.escapeAttrQuot(URelaxer.getString(button.xulButton.getMenu())));
            buffer.append("\"");
        }
        if (button.xulButton.getMinheight() != null) {
            buffer.append(" ");
            rNSContext_.makeQName("", "minheight", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.getString(button.xulButton.getMinheight()));
            buffer.append("\"");
        }
        if (button.xulButton.getMinwidth() != null) {
            buffer.append(" ");
            rNSContext_.makeQName("", "minwidth", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.getString(button.xulButton.getMinwidth()));
            buffer.append("\"");
        }
        if (button.xulButton.getObserves() != null) {
            buffer.append(" ");
            rNSContext_.makeQName("", "observes", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.escapeAttrQuot(URelaxer.getString(button.xulButton.getObserves())));
            buffer.append("\"");
        }
        if (button.xulButton.getOrdinal() != null) {
            buffer.append(" ");
            rNSContext_.makeQName("", "ordinal", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.getString(button.xulButton.getOrdinal()));
            buffer.append("\"");
        }
        if (button.xulButton.getOrient() != null) {
            buffer.append(" ");
            rNSContext_.makeQName("", "orient", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.escapeAttrQuot(URelaxer.getString(button.xulButton.getOrient())));
            buffer.append("\"");
        }
        if (button.xulButton.getPack() != null) {
            buffer.append(" ");
            rNSContext_.makeQName("", "pack", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.escapeAttrQuot(URelaxer.getString(button.xulButton.getPack())));
            buffer.append("\"");
        }
        if (button.xulButton.getPersist() != null) {
            buffer.append(" ");
            rNSContext_.makeQName("", "persist", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.escapeAttrQuot(URelaxer.getString(button.xulButton.getPersist())));
            buffer.append("\"");
        }
        if (button.xulButton.getPopup() != null) {
            buffer.append(" ");
            rNSContext_.makeQName("", "popup", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.escapeAttrQuot(URelaxer.getString(button.xulButton.getPopup())));
            buffer.append("\"");
        }
        if (button.xulButton.getPosition() != null) {
            buffer.append(" ");
            rNSContext_.makeQName("", "position", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.escapeAttrQuot(URelaxer.getString(button.xulButton.getPosition())));
            buffer.append("\"");
        }
        if (button.xulButton.getRef() != null) {
            buffer.append(" ");
            rNSContext_.makeQName("", "ref", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.escapeAttrQuot(URelaxer.getString(button.xulButton.getRef())));
            buffer.append("\"");
        }
        if (button.xulButton.getRemoveelement() != null) {
            buffer.append(" ");
            rNSContext_.makeQName("", "removeelement", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.escapeAttrQuot(URelaxer.getString(button.xulButton.getRemoveelement())));
            buffer.append("\"");
        }
        if (button.xulButton.getSortDirection() != null) {
            buffer.append(" ");
            rNSContext_.makeQName("", "sortDirection", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.escapeAttrQuot(URelaxer.getString(button.xulButton.getSortDirection())));
            buffer.append("\"");
        }
        if (button.xulButton.getSortResource() != null) {
            buffer.append(" ");
            rNSContext_.makeQName("", "sortResource", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.escapeAttrQuot(URelaxer.getString(button.xulButton.getSortResource())));
            buffer.append("\"");
        }
        if (button.xulButton.getSortResource2() != null) {
            buffer.append(" ");
            rNSContext_.makeQName("", "sortResource2", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.escapeAttrQuot(URelaxer.getString(button.xulButton.getSortResource2())));
            buffer.append("\"");
        }
        if (button.xulButton.getStatustext() != null) {
            buffer.append(" ");
            rNSContext_.makeQName("", "statustext", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.escapeAttrQuot(URelaxer.getString(button.xulButton.getStatustext())));
            buffer.append("\"");
        }
        if (button.xulButton.getStyle() != null) {
            buffer.append(" ");
            rNSContext_.makeQName("", "style", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.escapeAttrQuot(URelaxer.getString(button.xulButton.getStyle())));
            buffer.append("\"");
        }
        if (button.xulButton.getTemplate() != null) {
            buffer.append(" ");
            rNSContext_.makeQName("", "template", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.escapeAttrQuot(URelaxer.getString(button.xulButton.getTemplate())));
            buffer.append("\"");
        }
        if (button.xulButton.getTooltip() != null) {
            buffer.append(" ");
            rNSContext_.makeQName("", "tooltip", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.escapeAttrQuot(URelaxer.getString(button.xulButton.getTooltip())));
            buffer.append("\"");
        }
        if (button.xulButton.getTooltiptext() != null) {
            buffer.append(" ");
            rNSContext_.makeQName("", "tooltiptext", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.escapeAttrQuot(URelaxer.getString(button.xulButton.getTooltiptext())));
            buffer.append("\"");
        }
        if (button.xulButton.getTop() != null) {
            buffer.append(" ");
            rNSContext_.makeQName("", "top", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.getString(button.xulButton.getTop()));
            buffer.append("\"");
        }
        if (button.xulButton.getUri() != null) {
            buffer.append(" ");
            rNSContext_.makeQName("", "uri", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.escapeAttrQuot(URelaxer.getString(button.xulButton.getUri())));
            buffer.append("\"");
        }
        if (button.xulButton.getWidth() != null) {
            buffer.append(" ");
            rNSContext_.makeQName("", "width", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.getString(button.xulButton.getWidth()));
            buffer.append("\"");
        }
        if (button.xulButton.getAccesskey() != null) {
            buffer.append(" ");
            rNSContext_.makeQName("", "accesskey", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.escapeAttrQuot(URelaxer.getString(button.xulButton.getAccesskey())));
            buffer.append("\"");
        }
        if (button.xulButton.getCrop() != null) {
            buffer.append(" ");
            rNSContext_.makeQName("", "crop", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.escapeAttrQuot(URelaxer.getString(button.xulButton.getCrop())));
            buffer.append("\"");
        }
        if (button.xulButton.getImage() != null) {
            mixed = button.xulButton.getImage();
            resolvedString = CoveElResolver.getResolver().resolveExpression(mixed);
            buffer.append(" ");
            rNSContext_.makeQName("", "image", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.escapeAttrQuot(URelaxer.getString(resolvedString)));
            buffer.append("\"");
        }
        if (button.xulButton.getLabel() != null) {
            mixed = button.xulButton.getLabel();
            resolvedString = CoveElResolver.getResolver().resolveExpression(mixed);
            buffer.append(" ");
            rNSContext_.makeQName("", "label", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.escapeAttrQuot(URelaxer.getString(resolvedString)));
            buffer.append("\"");
        }
        if (button.xulButton.getChecked() != false) {
            buffer.append(" ");
            rNSContext_.makeQName("", "checked", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.getString(button.xulButton.getChecked()));
            buffer.append("\"");
        }
        if (button.xulButton.getCommand() != null) {
            buffer.append(" ");
            rNSContext_.makeQName("", "command", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.escapeAttrQuot(URelaxer.getString(button.xulButton.getCommand())));
            buffer.append("\"");
        }
        if (button.xulButton.getDisabled() != false) {
            buffer.append(" ");
            rNSContext_.makeQName("", "disabled", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.getString(button.xulButton.getDisabled()));
            buffer.append("\"");
        }
        if (button.xulButton.getTabindex() != null) {
            buffer.append(" ");
            rNSContext_.makeQName("", "tabindex", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.getString(button.xulButton.getTabindex()));
            buffer.append("\"");
        }
        if (button.xulButton.getOnclick() != null) {
            buffer.append(" ");
            rNSContext_.makeQName("", "onclick", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.escapeAttrQuot(URelaxer.getString(button.xulButton.getOnclick())));
            buffer.append("\"");
        }
        if (button.xulButton.getOnmousedown() != null) {
            buffer.append(" ");
            rNSContext_.makeQName("", "onmousedown", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.escapeAttrQuot(URelaxer.getString(button.xulButton.getOnmousedown())));
            buffer.append("\"");
        }
        if (button.xulButton.getOnmouseup() != null) {
            buffer.append(" ");
            rNSContext_.makeQName("", "onmouseup", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.escapeAttrQuot(URelaxer.getString(button.xulButton.getOnmouseup())));
            buffer.append("\"");
        }
        if (button.xulButton.getOnmouseover() != null) {
            buffer.append(" ");
            rNSContext_.makeQName("", "onmouseover", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.escapeAttrQuot(URelaxer.getString(button.xulButton.getOnmouseover())));
            buffer.append("\"");
        }
        if (button.xulButton.getOnmousemove() != null) {
            buffer.append(" ");
            rNSContext_.makeQName("", "onmousemove", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.escapeAttrQuot(URelaxer.getString(button.xulButton.getOnmousemove())));
            buffer.append("\"");
        }
        if (button.xulButton.getOnmouseout() != null) {
            buffer.append(" ");
            rNSContext_.makeQName("", "onmouseout", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.escapeAttrQuot(URelaxer.getString(button.xulButton.getOnmouseout())));
            buffer.append("\"");
        }
        if (button.xulButton.getOncommand() != null) {
            buffer.append(" ");
            rNSContext_.makeQName("", "oncommand", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.escapeAttrQuot(URelaxer.getString(button.xulButton.getOncommand())));
            buffer.append("\"");
        }
        if (button.xulButton.getOnkeypress() != null) {
            buffer.append(" ");
            rNSContext_.makeQName("", "onkeypress", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.escapeAttrQuot(URelaxer.getString(button.xulButton.getOnkeypress())));
            buffer.append("\"");
        }
        if (button.xulButton.getOnkeydown() != null) {
            buffer.append(" ");
            rNSContext_.makeQName("", "onkeydown", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.escapeAttrQuot(URelaxer.getString(button.xulButton.getOnkeydown())));
            buffer.append("\"");
        }
        if (button.xulButton.getOnkeyup() != null) {
            buffer.append(" ");
            rNSContext_.makeQName("", "onkeyup", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.escapeAttrQuot(URelaxer.getString(button.xulButton.getOnkeyup())));
            buffer.append("\"");
        }
        if (button.xulButton.getOnfocus() != null) {
            buffer.append(" ");
            rNSContext_.makeQName("", "onfocus", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.escapeAttrQuot(URelaxer.getString(button.xulButton.getOnfocus())));
            buffer.append("\"");
        }
        if (button.xulButton.getOnblur() != null) {
            buffer.append(" ");
            rNSContext_.makeQName("", "onblur", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.escapeAttrQuot(URelaxer.getString(button.xulButton.getOnblur())));
            buffer.append("\"");
        }
        if (button.xulButton.getOnload() != null) {
            buffer.append(" ");
            rNSContext_.makeQName("", "onload", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.escapeAttrQuot(URelaxer.getString(button.xulButton.getOnload())));
            buffer.append("\"");
        }
        if (button.xulButton.getOnunload() != null) {
            buffer.append(" ");
            rNSContext_.makeQName("", "onunload", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.escapeAttrQuot(URelaxer.getString(button.xulButton.getOnunload())));
            buffer.append("\"");
        }
        if (button.xulButton.getAutocheck() != false) {
            buffer.append(" ");
            rNSContext_.makeQName("", "autocheck", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.getString(button.xulButton.getAutocheck()));
            buffer.append("\"");
        }
        if (button.xulButton.getCheckstate() != null) {
            buffer.append(" ");
            rNSContext_.makeQName("", "checkstate", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.escapeAttrQuot(URelaxer.getString(button.xulButton.getCheckstate())));
            buffer.append("\"");
        }
        if (button.xulButton.getDlgType() != null) {
            buffer.append(" ");
            rNSContext_.makeQName("", "dlgType", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.escapeAttrQuot(URelaxer.getString(button.xulButton.getDlgType())));
            buffer.append("\"");
        }
        if (button.xulButton.getGroup() != null) {
            buffer.append(" ");
            rNSContext_.makeQName("", "group", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.escapeAttrQuot(URelaxer.getString(button.xulButton.getGroup())));
            buffer.append("\"");
        }
        if (button.xulButton.getOpen() != false) {
            buffer.append(" ");
            rNSContext_.makeQName("", "open", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.getString(button.xulButton.getOpen()));
            buffer.append("\"");
        }
        if (button.xulButton.getType() != null) {
            buffer.append(" ");
            rNSContext_.makeQName("", "type", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.escapeAttrQuot(URelaxer.getString(button.xulButton.getType())));
            buffer.append("\"");
        }
        buffer.append(">");
        
        return (new String(buffer)).toCharArray();
    }

    public void encodeChildren(FacesContext context, UIComponent component)
        throws IOException {
        if ((context == null) || (component == null)) {
            throw new NullPointerException();
        }
    }

    public void encodeEnd(FacesContext context, UIComponent component)
        throws IOException {
        if ((context == null) || (component == null)) {
            throw new NullPointerException();
        }
        ButtonComponent button = (ButtonComponent) component;
        char[] encoded = getEncodedEndString(button);
        ResponseWriter writer = context.getResponseWriter();
        writer.writeText(encoded, 0, encoded.length);
    }

    private char[] getEncodedEndString(ButtonComponent button) {
        StringBuffer buffer = new StringBuffer();
        RNSContext rNSContext_ = button.xulButton.rGetRNSContext();
        String prefix =
            rNSContext_.getPrefixByUri(
                "http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul");
        buffer.append("</");
        URelaxer.makeQName(prefix, "button", buffer);
        buffer.append(">");
        return (new String(buffer)).toCharArray();
    }
}
