package com.netpotlet.xul.renderkit;

import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.util.Map;

import javax.faces.FacesException;
import javax.faces.component.UIComponent;
import javax.faces.context.FacesContext;
import javax.faces.context.ResponseWriter;
import javax.faces.render.Renderer;
import javax.servlet.http.HttpServletRequest;

import org.apache.commons.el.parser.ParseException;

import com.netpotlet.cove.application.CoveViewHandler;
import com.netpotlet.cove.el.CoveElResolver;
import com.netpotlet.xul.component.TextboxComponent;
import com.netpotlet.xul.element.RNSContext;
import com.netpotlet.xul.element.URelaxer;

public class TextboxRenderer extends Renderer {

    public TextboxRenderer() {
        super();
    }

    public void decode(FacesContext context, UIComponent component) {
        if ((context == null) || (component == null)) {
            throw new NullPointerException();
        }
        TextboxComponent textbox = (TextboxComponent) component;
        String clientId = textbox.getClientId(context); 
        HttpServletRequest request =
            (HttpServletRequest) context.getExternalContext().getRequest();
        try {
            request.setCharacterEncoding(CoveViewHandler.CHAR_ENCODING);
        } catch (UnsupportedEncodingException e) {
            throw new FacesException(e);
        }
        Map requestParameterMap =
            (Map)context.getExternalContext().getRequestParameterMap();
        if (!requestParameterMap.containsKey(clientId)) return;
        String value = (String)requestParameterMap.get(clientId);
        if (value == null) return;
        textbox.setValue(value);
    }
    
    public void encodeBegin(FacesContext context, UIComponent component)
        throws IOException {
        if ((context == null) || (component == null)) {
            throw new NullPointerException();
        }
        TextboxComponent textbox = (TextboxComponent) component;
        try {
            char[] encoded = getEncodedBeginString(textbox);
            ResponseWriter writer = context.getResponseWriter();
            writer.writeText(encoded, 0, encoded.length);
        } catch (ParseException e) {
            throw new FacesException(e);
        }
    }

    private char[] getEncodedBeginString(TextboxComponent textbox)
        throws ParseException {
        RNSContext rNSContext_ = textbox.xulTextbox.rGetRNSContext();
        String mixed;
        String resolvedString;
        StringBuffer buffer = new StringBuffer();

        String prefix = rNSContext_.getPrefixByUri("http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul");
        buffer.append("<");
        URelaxer.makeQName(prefix, "textbox", buffer);
        rNSContext_.makeNSMappings(buffer);
        if (textbox.xulTextbox.getAlign() != null) {
            buffer.append(" ");
            rNSContext_.makeQName("", "align", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.escapeAttrQuot(URelaxer.getString(textbox.xulTextbox.getAlign())));
            buffer.append("\"");
        }
        if (textbox.xulTextbox.getAllowevents() != false) {
            buffer.append(" ");
            rNSContext_.makeQName("", "allowevents", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.getString(textbox.xulTextbox.getAllowevents()));
            buffer.append("\"");
        }
        if (textbox.xulTextbox.getAllownegativeassertions() != false) {
            buffer.append(" ");
            rNSContext_.makeQName("", "allownegativeassertions", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.getString(textbox.xulTextbox.getAllownegativeassertions()));
            buffer.append("\"");
        }
        if (textbox.xulTextbox.getClassValue() != null) {
            buffer.append(" ");
            rNSContext_.makeQName("", "class", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.escapeAttrQuot(URelaxer.getString(textbox.xulTextbox.getClassValue())));
            buffer.append("\"");
        }
        if (textbox.xulTextbox.getCoalesceduplicatearcs() != false) {
            buffer.append(" ");
            rNSContext_.makeQName("", "coalesceduplicatearcs", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.getString(textbox.xulTextbox.getCoalesceduplicatearcs()));
            buffer.append("\"");
        }
        if (textbox.xulTextbox.getCollapsed() != false) {
            buffer.append(" ");
            rNSContext_.makeQName("", "collapsed", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.getString(textbox.xulTextbox.getCollapsed()));
            buffer.append("\"");
        }
        if (textbox.xulTextbox.getContainer() != null) {
            buffer.append(" ");
            rNSContext_.makeQName("", "container", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.escapeAttrQuot(URelaxer.getString(textbox.xulTextbox.getContainer())));
            buffer.append("\"");
        }
        if (textbox.xulTextbox.getContainment() != null) {
            buffer.append(" ");
            rNSContext_.makeQName("", "containment", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.escapeAttrQuot(URelaxer.getString(textbox.xulTextbox.getContainment())));
            buffer.append("\"");
        }
        if (textbox.xulTextbox.getContext() != null) {
            buffer.append(" ");
            rNSContext_.makeQName("", "context", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.escapeAttrQuot(URelaxer.getString(textbox.xulTextbox.getContext())));
            buffer.append("\"");
        }
        if (textbox.xulTextbox.getContextmenu() != null) {
            buffer.append(" ");
            rNSContext_.makeQName("", "contextmenu", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.escapeAttrQuot(URelaxer.getString(textbox.xulTextbox.getContextmenu())));
            buffer.append("\"");
        }
        if (textbox.xulTextbox.getDatasources() != null) {
            buffer.append(" ");
            rNSContext_.makeQName("", "datasources", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.escapeAttrQuot(URelaxer.getString(textbox.xulTextbox.getDatasources())));
            buffer.append("\"");
        }
        if (textbox.xulTextbox.getDebug() != false) {
            buffer.append(" ");
            rNSContext_.makeQName("", "debug", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.getString(textbox.xulTextbox.getDebug()));
            buffer.append("\"");
        }
        if (textbox.xulTextbox.getDir() != null) {
            buffer.append(" ");
            rNSContext_.makeQName("", "dir", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.escapeAttrQuot(URelaxer.getString(textbox.xulTextbox.getDir())));
            buffer.append("\"");
        }
        if (textbox.xulTextbox.getEmpty() != false) {
            buffer.append(" ");
            rNSContext_.makeQName("", "empty", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.getString(textbox.xulTextbox.getEmpty()));
            buffer.append("\"");
        }
        if (textbox.xulTextbox.getEqualsize() != null) {
            buffer.append(" ");
            rNSContext_.makeQName("", "equalsize", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.escapeAttrQuot(URelaxer.getString(textbox.xulTextbox.getEqualsize())));
            buffer.append("\"");
        }
        if (textbox.xulTextbox.getFlags() != null) {
            buffer.append(" ");
            rNSContext_.makeQName("", "flags", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.escapeAttrQuot(URelaxer.getString(textbox.xulTextbox.getFlags())));
            buffer.append("\"");
        }
        if (textbox.xulTextbox.getFlex() != null) {
            buffer.append(" ");
            rNSContext_.makeQName("", "flex", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.getString(textbox.xulTextbox.getFlex()));
            buffer.append("\"");
        }
        if (textbox.xulTextbox.getHeight() != null) {
            buffer.append(" ");
            rNSContext_.makeQName("", "height", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.getString(textbox.xulTextbox.getHeight()));
            buffer.append("\"");
        }
        if (textbox.xulTextbox.getHidden() != false) {
            buffer.append(" ");
            rNSContext_.makeQName("", "hidden", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.getString(textbox.xulTextbox.getHidden()));
            buffer.append("\"");
        }
        if (textbox.xulTextbox.getId() != null) {
            buffer.append(" ");
            rNSContext_.makeQName("", "id", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.escapeAttrQuot(URelaxer.getString(textbox.xulTextbox.getId())));
            buffer.append("\"");
        }
        if (textbox.xulTextbox.getInsertafter() != null) {
            buffer.append(" ");
            rNSContext_.makeQName("", "insertafter", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.escapeAttrQuot(URelaxer.getString(textbox.xulTextbox.getInsertafter())));
            buffer.append("\"");
        }
        if (textbox.xulTextbox.getInsertbefore() != null) {
            buffer.append(" ");
            rNSContext_.makeQName("", "insertbefore", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.escapeAttrQuot(URelaxer.getString(textbox.xulTextbox.getInsertbefore())));
            buffer.append("\"");
        }
        if (textbox.xulTextbox.getLeft() != null) {
            buffer.append(" ");
            rNSContext_.makeQName("", "left", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.getString(textbox.xulTextbox.getLeft()));
            buffer.append("\"");
        }
        if (textbox.xulTextbox.getMaxheight() != null) {
            buffer.append(" ");
            rNSContext_.makeQName("", "maxheight", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.getString(textbox.xulTextbox.getMaxheight()));
            buffer.append("\"");
        }
        if (textbox.xulTextbox.getMaxwidth() != null) {
            buffer.append(" ");
            rNSContext_.makeQName("", "maxwidth", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.getString(textbox.xulTextbox.getMaxwidth()));
            buffer.append("\"");
        }
        if (textbox.xulTextbox.getMenu() != null) {
            buffer.append(" ");
            rNSContext_.makeQName("", "menu", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.escapeAttrQuot(URelaxer.getString(textbox.xulTextbox.getMenu())));
            buffer.append("\"");
        }
        if (textbox.xulTextbox.getMinheight() != null) {
            buffer.append(" ");
            rNSContext_.makeQName("", "minheight", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.getString(textbox.xulTextbox.getMinheight()));
            buffer.append("\"");
        }
        if (textbox.xulTextbox.getMinwidth() != null) {
            buffer.append(" ");
            rNSContext_.makeQName("", "minwidth", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.getString(textbox.xulTextbox.getMinwidth()));
            buffer.append("\"");
        }
        if (textbox.xulTextbox.getObserves() != null) {
            buffer.append(" ");
            rNSContext_.makeQName("", "observes", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.escapeAttrQuot(URelaxer.getString(textbox.xulTextbox.getObserves())));
            buffer.append("\"");
        }
        if (textbox.xulTextbox.getOrdinal() != null) {
            buffer.append(" ");
            rNSContext_.makeQName("", "ordinal", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.getString(textbox.xulTextbox.getOrdinal()));
            buffer.append("\"");
        }
        if (textbox.xulTextbox.getOrient() != null) {
            buffer.append(" ");
            rNSContext_.makeQName("", "orient", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.escapeAttrQuot(URelaxer.getString(textbox.xulTextbox.getOrient())));
            buffer.append("\"");
        }
        if (textbox.xulTextbox.getPack() != null) {
            buffer.append(" ");
            rNSContext_.makeQName("", "pack", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.escapeAttrQuot(URelaxer.getString(textbox.xulTextbox.getPack())));
            buffer.append("\"");
        }
        if (textbox.xulTextbox.getPersist() != null) {
            buffer.append(" ");
            rNSContext_.makeQName("", "persist", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.escapeAttrQuot(URelaxer.getString(textbox.xulTextbox.getPersist())));
            buffer.append("\"");
        }
        if (textbox.xulTextbox.getPopup() != null) {
            buffer.append(" ");
            rNSContext_.makeQName("", "popup", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.escapeAttrQuot(URelaxer.getString(textbox.xulTextbox.getPopup())));
            buffer.append("\"");
        }
        if (textbox.xulTextbox.getPosition() != null) {
            buffer.append(" ");
            rNSContext_.makeQName("", "position", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.escapeAttrQuot(URelaxer.getString(textbox.xulTextbox.getPosition())));
            buffer.append("\"");
        }
        if (textbox.xulTextbox.getRef() != null) {
            buffer.append(" ");
            rNSContext_.makeQName("", "ref", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.escapeAttrQuot(URelaxer.getString(textbox.xulTextbox.getRef())));
            buffer.append("\"");
        }
        if (textbox.xulTextbox.getRemoveelement() != null) {
            buffer.append(" ");
            rNSContext_.makeQName("", "removeelement", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.escapeAttrQuot(URelaxer.getString(textbox.xulTextbox.getRemoveelement())));
            buffer.append("\"");
        }
        if (textbox.xulTextbox.getSortDirection() != null) {
            buffer.append(" ");
            rNSContext_.makeQName("", "sortDirection", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.escapeAttrQuot(URelaxer.getString(textbox.xulTextbox.getSortDirection())));
            buffer.append("\"");
        }
        if (textbox.xulTextbox.getSortResource() != null) {
            buffer.append(" ");
            rNSContext_.makeQName("", "sortResource", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.escapeAttrQuot(URelaxer.getString(textbox.xulTextbox.getSortResource())));
            buffer.append("\"");
        }
        if (textbox.xulTextbox.getSortResource2() != null) {
            buffer.append(" ");
            rNSContext_.makeQName("", "sortResource2", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.escapeAttrQuot(URelaxer.getString(textbox.xulTextbox.getSortResource2())));
            buffer.append("\"");
        }
        if (textbox.xulTextbox.getStatustext() != null) {
            buffer.append(" ");
            rNSContext_.makeQName("", "statustext", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.escapeAttrQuot(URelaxer.getString(textbox.xulTextbox.getStatustext())));
            buffer.append("\"");
        }
        if (textbox.xulTextbox.getStyle() != null) {
            buffer.append(" ");
            rNSContext_.makeQName("", "style", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.escapeAttrQuot(URelaxer.getString(textbox.xulTextbox.getStyle())));
            buffer.append("\"");
        }
        if (textbox.xulTextbox.getTemplate() != null) {
            buffer.append(" ");
            rNSContext_.makeQName("", "template", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.escapeAttrQuot(URelaxer.getString(textbox.xulTextbox.getTemplate())));
            buffer.append("\"");
        }
        if (textbox.xulTextbox.getTooltip() != null) {
            buffer.append(" ");
            rNSContext_.makeQName("", "tooltip", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.escapeAttrQuot(URelaxer.getString(textbox.xulTextbox.getTooltip())));
            buffer.append("\"");
        }
        if (textbox.xulTextbox.getTooltiptext() != null) {
            buffer.append(" ");
            rNSContext_.makeQName("", "tooltiptext", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.escapeAttrQuot(URelaxer.getString(textbox.xulTextbox.getTooltiptext())));
            buffer.append("\"");
        }
        if (textbox.xulTextbox.getTop() != null) {
            buffer.append(" ");
            rNSContext_.makeQName("", "top", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.getString(textbox.xulTextbox.getTop()));
            buffer.append("\"");
        }
        if (textbox.xulTextbox.getUri() != null) {
            buffer.append(" ");
            rNSContext_.makeQName("", "uri", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.escapeAttrQuot(URelaxer.getString(textbox.xulTextbox.getUri())));
            buffer.append("\"");
        }
        if (textbox.xulTextbox.getWidth() != null) {
            buffer.append(" ");
            rNSContext_.makeQName("", "width", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.getString(textbox.xulTextbox.getWidth()));
            buffer.append("\"");
        }
        if (textbox.xulTextbox.getOnclick() != null) {
            buffer.append(" ");
            rNSContext_.makeQName("", "onclick", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.escapeAttrQuot(URelaxer.getString(textbox.xulTextbox.getOnclick())));
            buffer.append("\"");
        }
        if (textbox.xulTextbox.getOnmousedown() != null) {
            buffer.append(" ");
            rNSContext_.makeQName("", "onmousedown", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.escapeAttrQuot(URelaxer.getString(textbox.xulTextbox.getOnmousedown())));
            buffer.append("\"");
        }
        if (textbox.xulTextbox.getOnmouseup() != null) {
            buffer.append(" ");
            rNSContext_.makeQName("", "onmouseup", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.escapeAttrQuot(URelaxer.getString(textbox.xulTextbox.getOnmouseup())));
            buffer.append("\"");
        }
        if (textbox.xulTextbox.getOnmouseover() != null) {
            buffer.append(" ");
            rNSContext_.makeQName("", "onmouseover", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.escapeAttrQuot(URelaxer.getString(textbox.xulTextbox.getOnmouseover())));
            buffer.append("\"");
        }
        if (textbox.xulTextbox.getOnmousemove() != null) {
            buffer.append(" ");
            rNSContext_.makeQName("", "onmousemove", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.escapeAttrQuot(URelaxer.getString(textbox.xulTextbox.getOnmousemove())));
            buffer.append("\"");
        }
        if (textbox.xulTextbox.getOnmouseout() != null) {
            buffer.append(" ");
            rNSContext_.makeQName("", "onmouseout", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.escapeAttrQuot(URelaxer.getString(textbox.xulTextbox.getOnmouseout())));
            buffer.append("\"");
        }
        if (textbox.xulTextbox.getOncommand() != null) {
            buffer.append(" ");
            rNSContext_.makeQName("", "oncommand", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.escapeAttrQuot(URelaxer.getString(textbox.xulTextbox.getOncommand())));
            buffer.append("\"");
        }
        if (textbox.xulTextbox.getOnkeypress() != null) {
            buffer.append(" ");
            rNSContext_.makeQName("", "onkeypress", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.escapeAttrQuot(URelaxer.getString(textbox.xulTextbox.getOnkeypress())));
            buffer.append("\"");
        }
        if (textbox.xulTextbox.getOnkeydown() != null) {
            buffer.append(" ");
            rNSContext_.makeQName("", "onkeydown", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.escapeAttrQuot(URelaxer.getString(textbox.xulTextbox.getOnkeydown())));
            buffer.append("\"");
        }
        if (textbox.xulTextbox.getOnkeyup() != null) {
            buffer.append(" ");
            rNSContext_.makeQName("", "onkeyup", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.escapeAttrQuot(URelaxer.getString(textbox.xulTextbox.getOnkeyup())));
            buffer.append("\"");
        }
        if (textbox.xulTextbox.getOnfocus() != null) {
            buffer.append(" ");
            rNSContext_.makeQName("", "onfocus", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.escapeAttrQuot(URelaxer.getString(textbox.xulTextbox.getOnfocus())));
            buffer.append("\"");
        }
        if (textbox.xulTextbox.getOnblur() != null) {
            buffer.append(" ");
            rNSContext_.makeQName("", "onblur", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.escapeAttrQuot(URelaxer.getString(textbox.xulTextbox.getOnblur())));
            buffer.append("\"");
        }
        if (textbox.xulTextbox.getOnload() != null) {
            buffer.append(" ");
            rNSContext_.makeQName("", "onload", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.escapeAttrQuot(URelaxer.getString(textbox.xulTextbox.getOnload())));
            buffer.append("\"");
        }
        if (textbox.xulTextbox.getOnunload() != null) {
            buffer.append(" ");
            rNSContext_.makeQName("", "onunload", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.escapeAttrQuot(URelaxer.getString(textbox.xulTextbox.getOnunload())));
            buffer.append("\"");
        }
        if (textbox.xulTextbox.getCols() != null) {
            buffer.append(" ");
            rNSContext_.makeQName("", "cols", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.getString(textbox.xulTextbox.getCols()));
            buffer.append("\"");
        }
        if (textbox.xulTextbox.getDisabled() != false) {
            buffer.append(" ");
            rNSContext_.makeQName("", "disabled", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.getString(textbox.xulTextbox.getDisabled()));
            buffer.append("\"");
        }
        if (textbox.xulTextbox.getMaxlength() != null) {
            buffer.append(" ");
            rNSContext_.makeQName("", "maxlength", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.getString(textbox.xulTextbox.getMaxlength()));
            buffer.append("\"");
        }
        if (textbox.xulTextbox.getMultiline() != false) {
            buffer.append(" ");
            rNSContext_.makeQName("", "multiline", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.getString(textbox.xulTextbox.getMultiline()));
            buffer.append("\"");
        }
        if (textbox.xulTextbox.getOnchange() != null) {
            buffer.append(" ");
            rNSContext_.makeQName("", "onchange", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.escapeAttrQuot(URelaxer.getString(textbox.xulTextbox.getOnchange())));
            buffer.append("\"");
        }
        if (textbox.xulTextbox.getOninput() != null) {
            buffer.append(" ");
            rNSContext_.makeQName("", "oninput", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.escapeAttrQuot(URelaxer.getString(textbox.xulTextbox.getOninput())));
            buffer.append("\"");
        }
        if (textbox.xulTextbox.getReadonly() != false) {
            buffer.append(" ");
            rNSContext_.makeQName("", "readonly", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.getString(textbox.xulTextbox.getReadonly()));
            buffer.append("\"");
        }
        if (textbox.xulTextbox.getRows() != null) {
            buffer.append(" ");
            rNSContext_.makeQName("", "rows", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.getString(textbox.xulTextbox.getRows()));
            buffer.append("\"");
        }
        if (textbox.xulTextbox.getSize() != null) {
            buffer.append(" ");
            rNSContext_.makeQName("", "size", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.getString(textbox.xulTextbox.getSize()));
            buffer.append("\"");
        }
        if (textbox.xulTextbox.getTabindex() != null) {
            buffer.append(" ");
            rNSContext_.makeQName("", "tabindex", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.getString(textbox.xulTextbox.getTabindex()));
            buffer.append("\"");
        }
        if (textbox.xulTextbox.getTimeout() != null) {
            buffer.append(" ");
            rNSContext_.makeQName("", "timeout", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.getString(textbox.xulTextbox.getTimeout()));
            buffer.append("\"");
        }
        if (textbox.xulTextbox.getType() != null) {
            buffer.append(" ");
            rNSContext_.makeQName("", "type", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.escapeAttrQuot(URelaxer.getString(textbox.xulTextbox.getType())));
            buffer.append("\"");
        }
        if (textbox.xulTextbox.getValue() != null) {
            mixed = textbox.xulTextbox.getValueAsString();
            resolvedString = CoveElResolver.getResolver().resolveExpression(mixed);
            buffer.append(" ");
            rNSContext_.makeQName("", "value", buffer);
            buffer.append("=\"");
            buffer.append(URelaxer.escapeAttrQuot(URelaxer.getString(resolvedString)));
            buffer.append("\"");
        }
        buffer.append(">");
        return (new String(buffer)).toCharArray();
    }

    public void encodeChildren(FacesContext context, UIComponent component)
        throws IOException {
        if ((context == null) || (component == null)) {
            throw new NullPointerException();
        }
    }

    public void encodeEnd(FacesContext context, UIComponent component)
        throws IOException {
        if ((context == null) || (component == null)) {
            throw new NullPointerException();
        }
        TextboxComponent textbox = (TextboxComponent) component;
        char[] encoded = getEncodedEndString(textbox);
        ResponseWriter writer = context.getResponseWriter();
        writer.writeText(encoded, 0, encoded.length);
    }

    private char[] getEncodedEndString(TextboxComponent textbox) {
        StringBuffer buffer = new StringBuffer();
        RNSContext rNSContext_ = textbox.xulTextbox.rGetRNSContext();
        String prefix =
            rNSContext_.getPrefixByUri(
                "http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul");
        buffer.append("</");
        URelaxer.makeQName(prefix, "textbox", buffer);
        buffer.append(">");
        return (new String(buffer)).toCharArray();
    }
}
